@@ -18,12 +18,17 @@
   (equalp (make-inet-address "242.1.211.3")  #(242 1 211 3))
   t)
 
-#-win32
 (deftest make-inet6-address.1
     (equalp (make-inet6-address "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff")
             #(255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255))
   t)
 
+(deftest unparse-inet6-address
+    (string= (sb-bsd-sockets::unparse-inet6-address
+              (make-inet6-address "fe80::abcd:1234"))
+             "fe80::abcd:1234")
+  t)
+
 (deftest get-protocol-by-name/tcp
     (integerp (get-protocol-by-name "tcp"))
   t)
@@ -45,12 +50,13 @@
       nil))
   t)
 
-(when (handler-case (make-instance 'inet-socket
-                                   :type :stream
-                                   :protocol (get-protocol-by-name "tcp"))
-        (error nil)
-        (:no-error (x) x))
-  (push :ipv4-support *features*))
+(eval-when (:compile-toplevel :execute)
+  (when (handler-case (make-instance 'inet-socket
+                                     :type :stream
+                                     :protocol (get-protocol-by-name "tcp"))
+          (error nil)
+          (:no-error (x) x))
+    (push :ipv4-support *features*)))
 
 #+ipv4-support
 (deftest make-inet-socket.smoke
@@ -102,7 +108,6 @@
       (:no-error nil))
   t)
 
-#-win32
 (deftest make-inet6-socket.smoke
   (handler-case
       (let ((s (make-instance 'inet6-socket :type :stream :protocol (get-protocol-by-name "tcp"))))
@@ -110,7 +115,6 @@
     ((or address-family-not-supported protocol-not-supported-error) () t))
   t)
 
-#-win32
 (deftest make-inet6-socket.keyword
   (handler-case
       (let ((s (make-instance 'inet6-socket :type :stream :protocol :tcp)))
@@ -146,7 +150,6 @@
       (socket-close s2)))
   t)
 
-#-win32
 (deftest inet6-socket-bind
   (handler-case
       (let* ((tcp (get-protocol-by-name "tcp"))
@@ -273,7 +276,6 @@
       t)
   t)
 
-
 ;;; these require that the internet (or bits of it, at least) is available
 
 #+internet-available
@@ -390,7 +392,8 @@
 
 #+(and ipv4-support sb-thread)
 (deftest interrupt-io
-    (let (result)
+    (let (result
+          (sem (sb-thread:make-semaphore)))
       (labels
           ((client (port)
              (setf result
@@ -405,11 +408,8 @@
                        (handler-case
                            (prog1
                                (catch 'stop
-                                 (progn
-                                   (read-char stream)
-                                   (sleep 0.1)
-                                   (sleep 0.1)
-                                   (sleep 0.1)))
+                                 (sb-thread:signal-semaphore sem)
+                                 (read-char stream))
                              (close stream))
                          (error (c)
                            c))))))
@@ -428,43 +428,42 @@
                         (stream (socket-make-stream r
                                                     :input t
                                                     :output t
-                                                    :buffering :none))
-                        (ok :ok))
+                                                    :buffering :none)))
                    (socket-close s)
-                   (sleep 5)
+                   (sb-thread:wait-on-semaphore sem)
+                   (sleep 0.1)
                    (sb-thread:interrupt-thread client
-                                               (lambda () (throw 'stop ok)))
-                   (sleep 5)
-                   (setf ok :not-ok)
+                                               (lambda () (throw 'stop :ok)))
+                   (unless (sb-ext:wait-for (null (sb-thread::thread-interruptions client)) :timeout 5)
+                     (setf result :timeout))
                    (write-char #\x stream)
                    (close stream)
-                   (socket-close r))))))
+                   (socket-close r)
+                   (sb-thread:join-thread client :timeout 5))))))
         (server))
       result)
   :ok)
 
-(defmacro with-client-and-server ((server-socket-var client-socket-var) &body body)
-  (let ((listen-socket (gensym "LISTEN-SOCKET")))
-    `(let ((,listen-socket (make-instance 'inet-socket
-                                          :type :stream
-                                          :protocol :tcp))
-           (,client-socket-var (make-instance 'inet-socket
-                                              :type :stream
-                                              :protocol :tcp))
-           (,server-socket-var))
-      (unwind-protect
-           (progn
-             (setf (sockopt-reuse-address ,listen-socket) t)
-             (socket-bind ,listen-socket (make-inet-address "127.0.0.1") 0)
-             (socket-listen ,listen-socket 5)
-             (socket-connect ,client-socket-var (make-inet-address "127.0.0.1")
-                             (nth-value 1 (socket-name ,listen-socket)))
-             (setf ,server-socket-var (socket-accept ,listen-socket))
-             ,@body)
-        (socket-close ,client-socket-var)
-        (socket-close ,listen-socket)
-        (when ,server-socket-var
-          (socket-close ,server-socket-var))))))
+(defmacro with-client-and-server (((socket-class &rest common-initargs)
+                                   (listen-socket-var &rest listen-address)
+                                   (client-socket-var &rest client-address)
+                                   server-socket-var)
+                                  &body body)
+  `(let ((,listen-socket-var (make-instance ',socket-class ,@common-initargs))
+         (,client-socket-var (make-instance ',socket-class ,@common-initargs))
+         (,server-socket-var))
+     (unwind-protect
+          (progn
+            (setf (sockopt-reuse-address ,listen-socket-var) t)
+            (socket-bind ,listen-socket-var ,@listen-address)
+            (socket-listen ,listen-socket-var 5)
+            (socket-connect ,client-socket-var ,@client-address)
+            (setf ,server-socket-var (socket-accept ,listen-socket-var))
+            ,@body)
+       (socket-close ,client-socket-var)
+       (socket-close ,listen-socket-var)
+       (when ,server-socket-var
+         (socket-close ,server-socket-var)))))
 
 ;; For stream sockets, make sure a shutdown of the output direction
 ;; translates into an END-OF-FILE on the other end, no matter which
@@ -474,18 +473,23 @@
 (macrolet
     ((define-shutdown-test (name who-shuts-down who-reads element-type direction)
        `(deftest ,name
-          (with-client-and-server (client server)
-            (socket-shutdown ,who-shuts-down :direction ,direction)
-            (handler-case
-                (sb-ext:with-timeout 2
-                  (,(if (eql element-type 'character)
-                        'read-char 'read-byte)
-                   (socket-make-stream
-                    ,who-reads :input t :output t
-                    :element-type ',element-type)))
-              (end-of-file ()
-                :ok)
-              (sb-ext:timeout () :timeout)))
+          (let ((address (make-inet-address "127.0.0.1")))
+            (with-client-and-server
+                ((inet-socket :protocol :tcp :type :stream)
+                 (listener address 0)
+                 (client   address (nth-value 1 (socket-name listener)))
+                 server)
+              (socket-shutdown ,who-shuts-down :direction ,direction)
+              (handler-case
+                  (sb-ext:with-timeout 2
+                    (,(if (eql element-type 'character)
+                          'read-char 'read-byte)
+                      (socket-make-stream
+                       ,who-reads :input t :output t
+                       :element-type ',element-type)))
+                (end-of-file ()
+                  :ok)
+                (sb-ext:timeout () :timeout))))
           :ok))
      (define-shutdown-tests (direction)
        (flet ((make-name (name)
@@ -503,3 +507,30 @@
 
   (define-shutdown-tests :output)
   (define-shutdown-tests :io))
+
+(defun poor-persons-random-address ()
+  (let ((base (expt 36 8)))
+    (format nil "~36R" (+ base (random base (make-random-state t))))))
+
+#+linux
+(deftest abstract.smoke
+    (let* ((address (poor-persons-random-address))
+           (message "message")
+           (buffer (make-string (length message))))
+      (with-client-and-server ((local-abstract-socket :type :stream)
+                               (listener address)
+                               (client address)
+                               server)
+        (socket-send client message nil)
+        (string= (socket-receive server buffer nil) message)))
+  t)
+
+#+linux
+(deftest abstract.socket-peername
+    (let ((address (poor-persons-random-address)))
+      (with-client-and-server ((local-abstract-socket :type :stream)
+                               (listener address)
+                               (client address)
+                               server)
+        (string= (sb-ext:octets-to-string (socket-peername client)) address)))
+  t)
