@@ -22,10 +22,9 @@
           :OS-PROVIDES-POLL
           ;; The final batch of symbols is strictly for C. The prefix of
           ;; "LISP_FEATURE_" on the corresponding #define is unfortunate.
-          :GCC-TLS
+          :GCC-TLS :USE-SYS-MMAP
           :RESTORE-FS-SEGMENT-REGISTER-FROM-TLS ; only for 'src/runtime/thread.h'
-          :OS-PROVIDES-BLKSIZE-T ; only for 'src/runtime/wrap.h'
-          :OS-PROVIDES-PUTWC)) ; only for 'src/runtime/backtrace.c'
+          :OS-PROVIDES-BLKSIZE-T)) ; only for 'src/runtime/wrap.h'
        (public-features
         (cons
          sb-impl::!sbcl-architecture
@@ -36,7 +35,7 @@
            :FREEBSD :GNU-KFREEBSD :OPENBSD :NETBSD :DRAGONFLY :HAIKU :HPUX
            :MACH-O :ELF ; obj file format: pick zero or one
            ;; I would argue that this should not be exposed,
-           ;; but I would also anticipate blowblack from removing it.
+           ;; but I would also anticipate blowback from removing it.
            :CHENEYGC :GENCGC ; GC: pick one and only one
            ;; Can't use s-l-a-d :compression safely without it
            :SB-CORE-COMPRESSION
@@ -378,6 +377,7 @@ Please check that all strings which were not recognizable to the compiler
                             sb-vm::thread-profile-data-slot
                             sb-vm::thread-alloc-region-slot
                             sb-vm::primitive-object-size
+                            sb-vm::reconstitute-object
                             ;; need this for defining a vop which
                             ;; tests the x86-64 allocation profiler
                             sb-vm::pseudo-atomic
@@ -390,10 +390,19 @@ Please check that all strings which were not recognizable to the compiler
         #.(find-package "SB-ASSEM")
         #.(find-package "SB-DISASSEM")
         #.(find-package "SB-IMPL")
+        #.(find-package "SB-PCL")
+        #.(find-package "SB-MOP")
         #.(find-package "SB-PRETTY")
         #.(find-package "SB-KERNEL"))
        ;; Assume all and only external symbols must be retained
        (eq accessibility :external))
+      (#.(find-package "SB-THREAD")
+       (or (eq accessibility :external)
+           ;; for some reason a recent change caused the tree-shaker to drop MAKE-SPINLOCK
+           ;; which makes sense. I'm not sure what was rooting the symbol.
+           ;; However the :spinlock-api test in threads.impure asserts that spinlock symbols
+           ;; exist despite being internal symbols.
+           (sb-int:info :function :deprecated symbol)))
       (#.(find-package "SB-FASL")
        ;; Retain +BACKEND-FASL-FILE-IMPLEMENTATION+ and +FASL-FILE-VERSION+
        ;; (and anything else otherwise reachable)
