@@ -12,6 +12,12 @@
 
 (in-package "SB-ALIEN")
 
+;;; ALIEN-CALLBACK is supposed to be external in SB-ALIEN-INTERNALS, but the
+;;; export gets lost, and then 'chill' gets a conflict with SB-ALIEN over it.
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (export (intern "ALIEN-CALLBACK" "SB-ALIEN-INTERNALS")
+          "SB-ALIEN-INTERNALS"))
+
 ;;;; ALIEN CALLBACKS
 ;;;;
 ;;;; See "Foreign Linkage / Callbacks" in the SBCL Internals manual.
@@ -289,10 +295,33 @@ the alien callback for that function with the given alien type."
 (in-package "SB-THREAD")
 #+sb-thread
 (defun enter-foreign-callback (index return arguments)
-  (let ((thread (without-gcing
-                  ;; Hold off GCing until *current-thread* is set up
-                  (setf *current-thread*
-                        (make-foreign-thread :name "foreign callback")))))
-    (dx-flet ((enter ()
-                (sb-alien::enter-alien-callback index return arguments)))
-      (new-lisp-thread-trampoline thread nil #'enter nil))))
+  (let ((thread
+          #+sb-safepoint ; cons a FOREIGN-THREAD
+          (without-gcing (init-thread-local-storage (make-foreign-thread)))
+
+          ;; Deferrable signals are blocked. STOP_FOR_GC is not blocked but can't
+          ;; occur because GC could only send it after acquiring the all_threads lock,
+          ;; which it can't get because this thread owns it for the moment.
+          #-sb-safepoint ; use the preallocated FOREIGN-THREAD
+          (let ((thread (init-thread-local-storage *foreign-thread*)))
+            ;; GC would be ok to happen now, but it can't yet because of the
+            ;; all_threads_lock acting both as a guard on the preallocated thread
+            ;; and the stop-the-world inhibitor.
+            ;; There's no point in splitting that into two locks.
+            (setq *foreign-thread* (make-foreign-thread)) ; "Pay it forward"
+            (alien-funcall (extern-alien "release_all_threads_lock"
+                                         (function void)))
+            thread)))
+    #+pauseless-threadstart
+    (dx-let ((startup-info (vector nil ; trampoline is n/a
+                                   nil ; cell in *STARTING-THREADS* is n/a
+                                   #'sb-alien::enter-alien-callback
+                                   (list index return arguments)
+                                   nil nil))) ; sigmask + fpu state bits
+      (copy-primitive-thread-fields thread)
+      (setf (thread-startup-info thread) startup-info)
+      (update-all-threads (get-lisp-obj-address sb-vm:*control-stack-start*) thread)
+      (run))
+    #-pauseless-threadstart
+    (dx-let ((args (list index return arguments)))
+      (run thread nil #'sb-alien::enter-alien-callback args))))
