@@ -41,16 +41,6 @@
 ;;; cdr is the layout itself.
 (defvar *!initial-layouts*)
 
-;;; a generator for random values suitable for the CLOS-HASH slots of
-;;; LAYOUTs. We use our own RANDOM-STATE here because we'd like
-;;; pseudo-random values to come the same way in the target even when
-;;; we make minor changes to the system, in order to reduce the
-;;; mysteriousness of possible CLOS bugs.
-;;; However, we now try very hard to use deterministic hashes
-;;; based on the characters in the name of the class, if a symbol.
-(define-load-time-global *layout-clos-hash-random-state*
-    (make-random-state))
-
 ;;; a table mapping class names to layouts for classes we have
 ;;; referenced but not yet loaded. This is initialized from an alist
 ;;; created by genesis describing the layouts that genesis created at
@@ -65,7 +55,7 @@
                  (/show0 "processing *!INITIAL-LAYOUTS*")
                  (setq *forward-referenced-layouts* (make-hash-table :test 'equal))
                  (dovector (x *!initial-layouts*)
-                   (let ((expected (randomish-layout-clos-hash (car x)))
+                   (let ((expected (hash-layout-name (car x)))
                          (actual (layout-clos-hash (cdr x))))
                      (unless (= actual expected)
                        (bug "XC layout hash calculation failed")))
@@ -86,26 +76,6 @@
   (defun layout-proper-name (layout)
     (classoid-proper-name (layout-classoid layout))))
 
-;;;; support for the hash values used by CLOS when working with LAYOUTs
-
-(defun randomish-layout-clos-hash (name)
-  ;; A hash of 0 occurs with probability 1 in 2^25 for a 32-bit hash
-  ;; or 2^58 for a 64-bit hash. It must be changed to something else
-  ;; because 0 has means that the layout is invalid.
-  ;; See paper by Kiczales and Rodriguez, "Efficient Method Dispatch in PCL", 1990
-  (1+ (if (typep name '(and symbol (not null)))
-          (let ((package (sb-xc:symbol-package name)))
-            ;; TODO: We should use murmur_hash (in the C runtime)
-            ;; for better hashiness than our sxhash on strings.
-            (rem (logxor (sb-impl::%sxhash-simple-string
-                          (cond #+sb-xc ((eq package *cl-package*) "COMMON-LISP")
-                                ((not package) "uninterned")
-                                (t (package-name package))))
-                         (sb-impl::%sxhash-simple-string
-                          (symbol-name name)))
-                 (1- layout-clos-hash-limit)))
-          (random (1- layout-clos-hash-limit) *layout-clos-hash-random-state*))))
-
 ;;; If we can't find any existing layout, then we create a new one
 ;;; storing it in *FORWARD-REFERENCED-LAYOUTS*. In classic CMU CL, we
 ;;; used to immediately check for compatibility, but for
@@ -126,7 +96,7 @@
         (or (and classoid (classoid-layout classoid))
             (values (ensure-gethash name table
                                     (make-layout
-                                     (randomish-layout-clos-hash name)
+                                     (hash-layout-name name)
                                      (or classoid
                                          (make-undefined-classoid name))))))))))
 
@@ -149,14 +119,13 @@
   (cond ((eq (layout-invalid layout) :uninitialized)
          ;; There was no layout before, we just created one which
          ;; we'll now initialize with our information.
-         (setf (layout-inherits layout) inherits
-               (layout-classoid layout) classoid)
+         (setf (layout-classoid layout) classoid)
+         (set-layout-inherits layout inherits)
          (assign-layout-slots layout :depthoid depthoid :length length :flags flags)
          #-sb-xc-host ; Assign target-only slots
          (when (logtest +structure-layout-flag+ flags)
            ;; STRING-STREAM and FILE-STREAM have depthoid=4 but are not structure types
-           (setf (layout-bitmap layout) bitmap)
-           (populate-layout-ancestors layout inherits depthoid))
+           (setf (layout-bitmap layout) bitmap))
          ;; Finally, make it valid.
          (setf (layout-invalid layout) nil))
         ;; FIXME: Now that LAYOUTs are born :UNINITIALIZED, maybe this
@@ -365,10 +334,9 @@ between the ~A definition and the ~A definition"
             #-64-bit (setf (layout-depthoid destruct-layout) (layout-depthoid layout)
                            (layout-length destruct-layout) (layout-length layout))
             (setf (layout-flags destruct-layout) (layout-flags layout)
-                  (layout-inherits destruct-layout) inherits
                   (layout-info destruct-layout) (layout-info layout)
                   (layout-bitmap destruct-layout) (layout-bitmap layout))
-            (populate-layout-ancestors destruct-layout inherits)
+            (set-layout-inherits destruct-layout inherits)
             (setf (layout-invalid destruct-layout) nil
                   (classoid-layout classoid) destruct-layout))
           (setf (layout-invalid layout) nil
@@ -376,11 +344,16 @@ between the ~A definition and the ~A definition"
 
       (dovector (super-layout (layout-inherits layout))
         (let* ((super (layout-classoid super-layout))
-               (subclasses (or (classoid-subclasses super)
-                               (setf (classoid-subclasses super)
-                                     (make-hash-table :test 'eq
-                                                      #-sb-xc-host #-sb-xc-host
-                                                      :synchronized t)))))
+               (subclasses
+                 (or (classoid-subclasses super)
+                     (setf (classoid-subclasses super)
+                           #+sb-xc-host (make-hash-table :test 'eq)
+                           ;; Might as well use CTYPE-HASH-VALUE as a
+                           ;; stable hash since we have it.
+                           #-sb-xc-host
+                           (make-hash-table :hash-function #'type-hash-value
+                                            :test 'eq
+                                            :synchronized t)))))
           (when (and (eq (classoid-state super) :sealed)
                      (not (gethash classoid subclasses)))
             (warn "unsealing sealed class ~S in order to subclass it"
@@ -640,9 +613,10 @@ between the ~A definition and the ~A definition"
 ;;; Again, this should be compiler-only, but easier to make this
 ;;; thread-safe.
 (defun insured-find-classoid (name predicate constructor)
-  (declare (type function predicate constructor))
+  (declare (type function predicate)
+           (type (or function symbol) constructor))
   (let ((table *forward-referenced-layouts*))
-    (with-locked-system-table (table)
+    (with-system-mutex ((hash-table-lock table))
       (let* ((old (find-classoid name nil))
              (res (if (and old (funcall predicate old))
                       old
@@ -703,7 +677,7 @@ between the ~A definition and the ~A definition"
       (if (typep classoid 'standard-classoid)
           (let ((class (classoid-pcl-class classoid)))
             (cond
-              ((sb-pcl:class-finalized-p class)
+              ((sb-mop:class-finalized-p class)
                (sb-pcl::%force-cache-flushes class)
                t)
               ((sb-pcl::class-has-a-forward-referenced-superclass-p class)
@@ -715,7 +689,7 @@ between the ~A definition and the ~A definition"
                       error-context))
                nil)
               (t
-               (sb-pcl:finalize-inheritance class)
+               (sb-mop:finalize-inheritance class)
                t)))
           (bug "~@<Don't know how to ensure validity of ~S (not a STANDARD-CLASSOID) ~
                 for ~A.~%~:@>"
@@ -732,13 +706,6 @@ between the ~A definition and the ~A definition"
                  (%ensure-classoid-valid class2 layout2 errorp))
       (return-from %ensure-both-classoids-valid nil))))
 
-#-sb-xc-host ; No such thing as LAYOUT-OF, never mind the rest
-(defun update-object-layout-or-invalid (object layout)
-  ;; FIXME: explain why this isn't (LAYOUT-FOR-STD-CLASS-P LAYOUT).
-  (if (layout-for-std-class-p (layout-of object))
-      (sb-pcl::check-wrapper-validity object)
-      (sb-c::%layout-invalid-error object layout)))
-
 ;;; Simple methods for TYPE= and SUBTYPEP should never be called when
 ;;; the two classes are equal, since there are EQ checks in those
 ;;; operations.
@@ -1089,6 +1056,16 @@ between the ~A definition and the ~A definition"
       :inherits (symbol list sequence)
       :direct-superclasses (symbol list)
       :prototype-form 'nil)
+
+     ;; KLUDGE: the length must match the subsequent defstruct.
+     (pathname :depth 1
+               :length ,(+ 7 sb-vm:instance-data-start)
+               :prototype-form (make-pathname))
+     (logical-pathname :depth 2
+                       :length ,(+ 7 sb-vm:instance-data-start)
+                       :prototype-form (make-pathname :host "SYS")
+                       :inherits (pathname))
+
      ;; These last few are strange. STREAM has only T as an ancestor,
      ;; so you'd think it would be at depth 1. FILE- and STRING-STREAM
      ;; each have STREAM and T as ancestors, so you'd think they'd be at depth
@@ -1144,6 +1121,7 @@ between the ~A definition and the ~A definition"
               codes
               state
               depth
+              (length 0)
               prototype-form
               (hierarchical-p t) ; might be modified below
               (direct-superclasses (if inherits
@@ -1194,7 +1172,7 @@ between the ~A definition and the ~A definition"
            (find-and-init-or-check-layout name
                                           depthoid
                                           0 ; flags
-                                          0 ; length
+                                          length
                                           +layout-all-tagged+
                                           inherits-vector)
            :invalidate nil)))))
@@ -1231,6 +1209,9 @@ between the ~A definition and the ~A definition"
   #+sb-xc-host (warn "Why are we invalidating layout ~S?" layout)
   (setf (layout-invalid layout) t)
   (assign-layout-slots layout :depthoid -1)
+  ;; Ensure that the INVALID slot conveying ancillary data describing the
+  ;; invalidity reason is published before causing the invalid layout trap.
+  (sb-thread:barrier (:write))
   (setf (layout-clos-hash layout) 0)
   (let ((inherits (layout-inherits layout))
         (classoid (layout-classoid layout)))
