@@ -9,11 +9,12 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
-(macrolet ((def (name constructor access src-type)
+(macrolet ((def (name constructor access src-type &optional explicit-check)
              `(defun ,name (object type)
                 (declare (type ,src-type object))
+                ,@(when explicit-check `((declare (explicit-check))))
                 (do* ((index 0 (1+ index))
                       (length (length object))
                       (result ,constructor)
@@ -27,31 +28,27 @@
                            (vector '(aref in-object index))
                            (sequence '(elt in-object index))))))))
 
-  (def list-to-vector* (make-sequence type length) aref list)
+  (def list-to-vector* (make-sequence type length) aref list t)
 
-  (def vector-to-vector* (make-sequence type length) aref vector)
+  (def vector-to-vector* (make-sequence type length) aref vector t)
 
   (def sequence-to-vector* (make-sequence type length) aref sequence))
 
 (defun vector-to-list* (object)
   (declare (type vector object))
-  (let ((result (list nil))
-        (length (length object)))
-    (declare (fixnum length))
-    (do ((index 0 (1+ index))
-         (splice result (cdr splice)))
-        ((>= index length) (cdr result))
-      (declare (fixnum index))
-      (rplacd splice (list (aref object index))))))
+  (dx-let ((result (list nil)))
+    (let ((splice result))
+      (do-vector-data (elt object (cdr result))
+        (let ((cell (list elt)))
+          (setf (cdr splice) cell splice cell))))))
 
 (defun sequence-to-list (sequence)
   (declare (type sequence sequence))
-  (let* ((result (list nil))
-         (splice result))
-    (sb!sequence:dosequence (i sequence)
-      (rplacd splice (list i))
-      (setf splice (cdr splice)))
-    (cdr result)))
+  (dx-let ((result (list nil)))
+    (let ((splice result))
+      (sb-sequence:dosequence (elt sequence (cdr result))
+        (let ((cell (list elt)))
+          (setf (cdr splice) cell splice cell))))))
 
 ;;; These are used both by the full DEFUN function and by various
 ;;; optimization transforms in the constant-OUTPUT-TYPE-SPEC case.
@@ -118,11 +115,13 @@
 
 ;;; old working version
 (defun coerce (object output-type-spec)
-  #!+sb-doc
   "Coerce the Object to an object of type Output-Type-Spec."
+  (declare (explicit-check))
   (flet ((coerce-error ()
+           (declare (optimize allow-non-returning-tail-call))
            (error 'simple-type-error
-                  :format-control "~S can't be converted to type ~S."
+                  :format-control "~S can't be converted to type ~
+                                    ~/sb-impl:print-type-specifier/."
                   :format-arguments (list object output-type-spec)
                   :datum object
                   :expected-type output-type-spec)))
@@ -146,7 +145,7 @@
               (unless (typep res output-type-spec)
                 (coerce-error))
               res))
-           #!+long-float
+           #+long-float
            ((csubtypep type (specifier-type 'long-float))
             (let ((res (%long-float object)))
               (unless (typep res output-type-spec)
@@ -166,7 +165,7 @@
                      ((csubtypep type (specifier-type '(complex double-float)))
                       (complex (%double-float (realpart object))
                                (%double-float (imagpart object))))
-                     #!+long-float
+                     #+long-float
                      ((csubtypep type (specifier-type '(complex long-float)))
                       (complex (%long-float (realpart object))
                                (%long-float (imagpart object))))
@@ -207,7 +206,7 @@
                                                          (length object))))
                ((cons-type-p type)
                 (multiple-value-bind (min exactp)
-                    (sb!kernel::cons-type-length-info type)
+                    (sb-kernel::cons-type-length-info type)
                   (let ((length (length object)))
                     (if exactp
                         (unless (= length min)
@@ -219,7 +218,7 @@
              (if (sequencep object)
                  (cond
                    ((type= type (specifier-type 'list))
-                    (sb!sequence:make-sequence-like
+                    (sb-sequence:make-sequence-like
                      nil (length object) :initial-contents object))
                    ((type= type (specifier-type 'null))
                     (if (= (length object) 0)
@@ -228,14 +227,14 @@
                                                              (length object))))
                    ((cons-type-p type)
                     (multiple-value-bind (min exactp)
-                        (sb!kernel::cons-type-length-info type)
+                        (sb-kernel::cons-type-length-info type)
                       (let ((length (length object)))
                         (if exactp
                             (unless (= length min)
                               (sequence-type-length-mismatch-error type length))
                             (unless (>= length min)
                               (sequence-type-length-mismatch-error type length)))
-                        (sb!sequence:make-sequence-like
+                        (sb-sequence:make-sequence-like
                          nil length :initial-contents object))))
                    (t (sequence-type-too-hairy (type-specifier type))))
                  (coerce-error))))
@@ -250,80 +249,13 @@
             (coerce-error))))
         ((and (csubtypep type (specifier-type 'sequence))
               (find-class output-type-spec nil))
-         (let ((prototype (sb!mop:class-prototype
-                           (sb!pcl:ensure-class-finalized
+         (let ((prototype (sb-mop:class-prototype
+                           (sb-pcl:ensure-class-finalized
                             (find-class output-type-spec)))))
-           (sb!sequence:make-sequence-like
+           (sb-sequence:make-sequence-like
             prototype (length object) :initial-contents object)))
         ((csubtypep type (specifier-type 'function))
          (coerce-to-fun object))
         (t
          (coerce-error))))))
 
-;;; new version, which seems as though it should be better, but which
-;;; does not yet work
-#+nil
-(defun coerce (object output-type-spec)
-  #!+sb-doc
-  "Coerces the Object to an object of type Output-Type-Spec."
-  (flet ((coerce-error ()
-           (error 'simple-type-error
-                  :format-control "~S can't be converted to type ~S."
-                  :format-arguments (list object output-type-spec)))
-         (check-result (result)
-           #!+high-security (aver (typep result output-type-spec))
-           result))
-    (let ((type (specifier-type output-type-spec)))
-      (cond
-        ((%%typep object type)
-         object)
-        ((eq type *empty-type*)
-         (coerce-error))
-        ((csubtypep type (specifier-type 'character))
-         (character object))
-        ((csubtypep type (specifier-type 'function))
-         (coerce-to-fun object))
-        ((numberp object)
-         (let ((res
-                (cond
-                  ((csubtypep type (specifier-type 'single-float))
-                   (%single-float object))
-                  ((csubtypep type (specifier-type 'double-float))
-                   (%double-float object))
-                  #!+long-float
-                  ((csubtypep type (specifier-type 'long-float))
-                   (%long-float object))
-                  ((csubtypep type (specifier-type 'float))
-                   (%single-float object))
-                  ((csubtypep type (specifier-type '(complex single-float)))
-                   (complex (%single-float (realpart object))
-                            (%single-float (imagpart object))))
-                  ((csubtypep type (specifier-type '(complex double-float)))
-                   (complex (%double-float (realpart object))
-                            (%double-float (imagpart object))))
-                  #!+long-float
-                  ((csubtypep type (specifier-type '(complex long-float)))
-                   (complex (%long-float (realpart object))
-                            (%long-float (imagpart object))))
-                  ((csubtypep type (specifier-type 'complex))
-                   (complex object))
-                  (t
-                   (coerce-error)))))
-           ;; If RES has the wrong type, that means that rule of
-           ;; canonical representation for complex rationals was
-           ;; invoked. According to the ANSI spec, (COERCE 7/2
-           ;; 'COMPLEX) returns 7/2. Thus, if the object was a
-           ;; rational, there is no error here.
-           (unless (or (typep res output-type-spec) (rationalp object))
-             (coerce-error))
-           res))
-        ((csubtypep type (specifier-type 'list))
-         (coerce-to-list object))
-        ((csubtypep type (specifier-type 'string))
-         (check-result (coerce-to-simple-string object)))
-        ((csubtypep type (specifier-type 'bit-vector))
-         (check-result (coerce-to-bit-vector object)))
-        ((csubtypep type (specifier-type 'vector))
-         (check-result (coerce-to-vector object output-type-spec)))
-        (t
-         (coerce-error))))))
