@@ -14,6 +14,17 @@
 
 ;;; Forward declarations
 
+(defun make-system-hash-table (&rest args)
+  (let ((args (copy-list args)))
+    (remf args :weakness)
+    (remf args :synchronized)
+    (remf args :finalizer)
+    (let ((hash-fun (getf args :hash-function)))
+      (when hash-fun
+        (assert (eq (getf args :test) 'eq))
+        (remf args :hash-function)))
+    (apply 'make-hash-table args)))
+
 ;;; In correct code, TRULY-THE has only a performance impact and can
 ;;; be safely degraded to ordinary THE.
 (defmacro truly-the (type expr)
@@ -23,9 +34,7 @@
   (declare (ignore name))
   `#'(lambda ,args ,@body))
 
-(defmacro with-locked-system-table ((table) &body body)
-  (declare (ignore table))
-  `(progn ,@body))
+(defmacro define-thread-local (&rest rest) `(defvar ,@rest))
 
 (defmacro defglobal (name value &rest doc)
   `(eval-when (:compile-toplevel :load-toplevel :execute)
@@ -77,6 +86,17 @@
 (defun funcallable-instance-p (x)
   (error "Called FUNCALLABLE-INSTANCE-P ~s" x))
 
+(defun simple-fun-p (x)
+  (if (symbolp x) nil (error "Called SIMPLE-FUN-P on ~S" x)))
+(defun closurep (x)
+  (if (symbolp x) nil (error "Called CLOSUREP on ~S" x)))
+(defun unbound-marker-p (x)
+  (if (symbolp x) nil (error "Called UNBOUND-MARKER-P on ~S" x)))
+(defun vector-with-fill-pointer-p (x)
+  (if (symbolp x) nil (error "Called VECTOR-WITH-FILL-POINTER-P on ~S" x)))
+
+(defparameter sb-vm::*backend-cross-foldable-predicates* nil)
+
 ;; The definition of TYPE-SPECIFIER for the target appears in the file
 ;; 'deftypes-for-target' - it allows CLASSes and CLASOIDs as specifiers.
 ;; Instances are never used as specifiers when building SBCL,
@@ -218,8 +238,9 @@
 ;;; For macro lambdas that are processed by the host
 (declaim (declaration top-level-form))
 
-;;; The opposite of the whitelist - if certain full calls are seen, it is probably
-;;; the result of a missed transform and/or misconfiguration.
+;;; The opposite of *undefined-fun-allowlist* - if certain full calls
+;;; are seen, it is probably the result of a missed transform and/or
+;;; misconfiguration.
 (defparameter *full-calls-to-warn-about*
   '(;mask-signed-field ;; Too many to fix
     ))
@@ -326,6 +347,11 @@
 
 (defmacro sb-format:tokens (string) string)
 
+;;; For a use in EARLY-TYPE
+(defmacro with-system-mutex ((lock) &body body)
+  (declare (ignore lock))
+  `(progn ,@body))
+
 ;;; Used by our lockfree memoization functions (define-hash-cache)
 (defmacro sb-thread:barrier ((kind) &body body)
   (declare (ignore kind))
@@ -335,3 +361,8 @@
 (defmacro %primitive (name arg)
   (ecase name
     (print `(format t "~A~%" ,arg))))
+
+(defmacro %with-output-to-string ((var) &body body)
+  ;; Let's suppose that the host lisp knows what it's doing to efficiently
+  ;; compile the standard macro. Don't try to outdo it.
+  `(with-output-to-string (,var) ,@body))
