@@ -266,6 +266,7 @@ statistics are appended to it."
   ;; dying thread, so we still need the guard for that, but not
   ;; the guard for whether interupts are enabled.
   (when (sb-thread:thread-alive-p sb-thread:*current-thread*)
+    #+sb-thread (alien-funcall (extern-alien "empty_thread_recyclebin" (function void)))
     (let ((threadp #+sb-thread (%instancep sb-impl::*finalizer-thread*)))
       (when threadp
         ;; It's OK to frob a condition variable regardless of
@@ -274,7 +275,45 @@ statistics are appended to it."
         ;; if there is no finalizer thread and interrupts are disabled.
         ;; That's my excuse anyway, not having looked more in-depth.
         (run-pending-finalizers))
-      (when *allow-with-interrupts*
+      ;; Here's one reason that MAX_INTERRUPTS is as high as it is.
+      ;; If the thread that performed GC runs post-GC hooks which cons enough to
+      ;; cause another GC while in the hooks, then as soon as interrupts are allowed
+      ;; again, a GC can be invoked "recursively" - while there is a maybe_gc() on
+      ;; the C call stack. It's not even true that _this_ thread had to cons a ton -
+      ;; any thread could have, causing this thread to hit the GC trigger which sets the
+      ;; P-A interrupted bit which causes the interrupt instruction at the end of a
+      ;; pseudo-atomic sequence to take a signal hit. So with interrupts eanbled,
+      ;; we get back into the GC, which calls post-GC, which might cons ...
+      ;; See the example at the bottom of src/code/final for a clear picture.
+      ;;
+      ;; To mitigate that problem, if you have no hooks, we can avoid inducing overflow
+      ;; of the interrupt contexts. Relegating post-GC actions to their own thread
+      ;; (as suggested above) would largely solve this - the post-GC action would be to
+      ;; just "kick" that thread to do its thing. If it doesn't finish in time (by the
+      ;; time we kick it again), that's its problem, not the GC's problem.
+      ;;
+      ;; If we do implement a general post-GC thread, there remains a question of
+      ;; of what to do with post-GC actions for #-sb-thread that cons too much,
+      ;; risking recursive invocation of GC. Pretty much don't do that.
+      ;;
+      (when (and *allow-with-interrupts*
+                 ;; Continue if any of the following:
+                 ;; - you want a finalizer thread but it hasn't been started
+                 ;; - you don't want a finalizer thread, and we're not already
+                 ;;   in SCAN-FINALIZERS
+                 ;; - there are some hooks to run
+                 ;; Logically, the finalizer existence test belongs in src/code/final,
+                 ;; but the flaw in that is we're not going to call that code until unmasking
+                 ;; interrupts, which is precisely the thing we need to NOT do if already
+                 ;; in post-GC code of any kind (be it finalizer or other).
+                 (or #+sb-thread (if sb-impl::*finalizer-thread*
+                                     (not threadp)
+                                     (not sb-impl::*in-a-finalizer*))
+                     #-sb-thread (and (sb-impl::hash-table-culled-values
+                                       (sb-impl::finalizer-id-map
+                                        sb-impl::**finalizer-store**))
+                                      (not sb-impl::*in-a-finalizer*))
+                     *after-gc-hooks*))
         (sb-thread::without-thread-waiting-for ()
          (with-interrupts
            (unless threadp
