@@ -19,7 +19,7 @@
 ;;; to running into any errors, or else you've got twice the trouble.
 ;;;
 ;;; There are other thread-locals which are used by C code, and those are all
-;;; defined in !PER-THREAD-C-INTERFACE-SYMBOLS. Those other symbols must have
+;;; defined in PER-THREAD-C-INTERFACE-SYMBOLS. Those other symbols must have
 ;;; initial values of either a fixnum or a static symbol (typically T or NIL).
 ;;;
 ;;; In contrast, DEFINE-THREAD-LOCAL allows more-or-less an arbitrary form,
@@ -65,15 +65,15 @@
       (let ((found (assq symbol (cdr list))))
         (if found
             (setf (cadr found) initform)
-            (let* ((thread-struct (find 'sb-vm::thread sb-vm::*primitive-objects*
+            (let* ((thread-struct (find 'sb-vm::thread sb-vm:*primitive-objects*
                                         :key #'sb-vm:primitive-object-name))
                    (n-fixed (+ (sb-vm:primitive-object-length thread-struct)
                                (count-if-not
                                 (lambda (x)
                                   (find (car (ensure-list x))
                                         (sb-vm:primitive-object-slots thread-struct)
-                                        :key #'sb-vm::slot-special))
-                                sb-vm::!per-thread-c-interface-symbols)))
+                                        :key #'sb-vm:slot-special))
+                                sb-vm::per-thread-c-interface-symbols)))
                    (tls-slot (+ n-fixed (length (cdr list)))))
               (nconc list (list (list symbol initform)))
               (setf (info :variable :wired-tls symbol) (ash tls-slot sb-vm:word-shift))
