@@ -18,7 +18,7 @@
 ;;;; -- WHN 20000127
 
 (declaim (maybe-inline
-          tree-equal %setnth nthcdr
+          tree-equal %setnth nthcdr nth
           tailp union
           nunion intersection nintersection set-difference nset-difference
           set-exclusive-or nset-exclusive-or subsetp acons
@@ -175,7 +175,7 @@
   (endp object))
 
 (defun list-length (list)
-  "Return the length of the given List, or Nil if the List is circular."
+  "Return the length of LIST, or NIL if LIST is circular."
   (do ((n 0 (+ n 2))
        (y list (cddr y))
        (z list (cdr z)))
@@ -188,8 +188,23 @@
 
 (defun nth (n list)
   "Return the nth object in a list where the car is the zero-th element."
-  (declare (explicit-check))
-  (car (nthcdr n list)))
+  (declare (explicit-check)
+           (optimize speed))
+  (typecase n
+    ((and fixnum unsigned-byte)
+     (block nil
+       (let ((i n)
+             (result list))
+         (tagbody
+          loop
+            (the list result)
+            (if (plusp i)
+                (psetq i (1- i)
+                       result (cdr result))
+                (return (car result)))
+            (go loop)))))
+    (t
+     (car (nthcdr n list)))))
 
 (defun first (list)
   "Return the 1st object in a list or NIL if the list is empty."
@@ -227,41 +242,34 @@
 
 (defun nthcdr (n list)
   "Performs the cdr function n times on a list."
+  (declare (explicit-check n)
+           (optimize speed))
   (flet ((fast-nthcdr (n list)
-           (declare (type index n))
            (do ((i n (1- i))
                 (result list (cdr result)))
-               ((not (plusp i)) result)
-             (declare (type index i)))))
+               ((not (plusp i)) result))))
     (typecase n
-      (index (fast-nthcdr n list))
+      ((and fixnum unsigned-byte)
+       (fast-nthcdr n list))
       ;; Such a large list can only be circular
-      (t (do ((i 0 (1+ i))
+      (t
+       (locally (declare (unsigned-byte n))
+         (do ((i 0 (1+ i))
               (r-i list (cdr r-i))
               (r-2i list (cddr r-2i)))
              ((and (eq r-i r-2i) (not (zerop i)))
               (fast-nthcdr (mod n i) r-i))
-           (declare (type index i)))))))
+           (declare (type fixnum i))))))))
 
 ;;; For [n]butlast
 (defun dotted-nthcdr (n list)
-  (flet ((fast-nthcdr (n list)
-           (declare (type index n))
-           (do ((i n (1- i))
-                (result list (cdr result)))
-               ((not (plusp i)) result)
-             (declare (type index i))
-             (when (atom result)
-               (return)))))
-    (typecase n
-      (index (fast-nthcdr n list))
-      ;; Such a large list can only be circular
-      (t (do ((i 0 (1+ i))
-              (r-i list (cdr r-i))
-              (r-2i list (cddr r-2i)))
-             ((and (eq r-i r-2i) (not (zerop i)))
-              (fast-nthcdr (mod n i) r-i))
-           (declare (type index i)))))))
+  (declare (fixnum n))
+  (do ((i n (1- i))
+       (result list (cdr result)))
+      ((not (plusp i)) result)
+    (declare (type fixnum i))
+    (when (atom result)
+      (return))))
 
 ;;; LAST
 ;;;
@@ -302,15 +310,15 @@
                 returned-list)))
 
   (defun %last0 (list)
-    (declare (optimize speed (sb-c::verify-arg-count 0)))
+    (declare (optimize speed (sb-c:verify-arg-count 0)))
     (last0-macro))
 
   (defun %last1 (list)
-    (declare (optimize speed (sb-c::verify-arg-count 0)))
+    (declare (optimize speed (sb-c:verify-arg-count 0)))
     (last1-macro))
 
   (defun %lastn/fixnum (list n)
-    (declare (optimize speed (sb-c::verify-arg-count 0))
+    (declare (optimize speed (sb-c:verify-arg-count 0))
              (type (and unsigned-byte fixnum) n))
     (case n
       (1 (last1-macro))
@@ -318,7 +326,7 @@
       (t (lastn-macro fixnum))))
 
   (defun %lastn/bignum (list n)
-    (declare (optimize speed (sb-c::verify-arg-count 0))
+    (declare (optimize speed (sb-c:verify-arg-count 0))
              (type (and unsigned-byte bignum) n))
     (lastn-macro unsigned-byte))
 
@@ -343,7 +351,7 @@
       form))
 
 (defun list (&rest args)
-  "Return constructs and returns a list of its arguments."
+  "Construct and return a list containing the objects ARGS."
   args)
 
 ;;; LIST* is done the same as LIST, except that the last cons is made
@@ -372,7 +380,7 @@
              result)))))
 
 (defun make-list (size &key initial-element)
-  "Constructs a list with size elements each set to value"
+  "Construct and return a list with SIZE elements each set to INITIAL-ELEMENT."
   (declare (explicit-check))
   (%make-list size initial-element))
 ;;; This entry point is to be preferred, irrespective of
@@ -385,7 +393,7 @@
     (declare (type index count))))
 
 (defun append (&rest lists)
-  "Construct a new list by concatenating the list arguments"
+  "Construct and return a list by concatenating LISTS."
   (let* ((result (list nil))
          (tail result)
          (index 0)
@@ -411,7 +419,7 @@
        (cdr result)))))
 
 (defun append2 (x y)
-  (declare (optimize (sb-c::verify-arg-count 0)))
+  (declare (optimize (sb-c:verify-arg-count 0)))
   (if (null x)
       y
       (let ((result (list (car x))))
@@ -523,14 +531,17 @@
 
 
 (defun butlast (list &optional (n 1))
-  (cond ((zerop n)
-         (copy-list list))
-        ((not (typep n 'index))
+  (declare (optimize speed)
+           (explicit-check n))
+  (cond ((not (typep n '(and fixnum unsigned-byte)))
+         (the unsigned-byte n)
          nil)
+        ((zerop n)
+         (copy-list list))
         (t
          (let ((head (dotted-nthcdr (1- n) list)))
-           (and (consp head)      ; there are at least n
-                (collect ((copy)) ; conses; copy!
+           (and (consp head)            ; there are at least n
+                (collect ((copy))       ; conses; copy!
                   (do ((trail list (cdr trail))
                        (head head (cdr head)))
                       ;; HEAD is n-1 conses ahead of TRAIL;
@@ -541,14 +552,17 @@
                     (copy (car trail)))))))))
 
 (defun nbutlast (list &optional (n 1))
-  (cond ((zerop n)
-         list)
-        ((not (typep n 'index))
+  (declare (optimize speed)
+           (explicit-check n))
+  (cond ((not (typep n '(and fixnum unsigned-byte)))
+         (the unsigned-byte n)
          nil)
+        ((zerop n)
+         list)
         (t
          (let ((head (dotted-nthcdr (1- n) list)))
-           (and (consp head)       ; there are more than n
-                (consp (cdr head)) ; conses.
+           (and (consp head)            ; there are more than n
+                (consp (cdr head))      ; conses.
                 ;; TRAIL trails by n cons to be able to
                 ;; cut the list at the cons just before.
                 (do ((trail list (cdr trail))
@@ -1400,7 +1414,7 @@
                                  body-loop)))
                   `(defun ,(intern (format nil "%~A~{-~A~}~@[-~A~]" name funs variant))
                        (x list ,@funs)
-                     (declare (optimize speed (sb-c::verify-arg-count 0)))
+                     (declare (optimize speed (sb-c:verify-arg-count 0)))
                      ,@(when funs `((declare (function ,@funs)
                                              (dynamic-extent ,@funs))))
                      ,@(unless (member name '(member assoc adjoin rassoc))
