@@ -71,6 +71,14 @@
               (examine var keyword))))
         (nreverse caller-dxable)))))
 
+(defun block-compilation-non-entry-point (name)
+  (and (boundp 'sb-c:*compilation*)
+       (let* ((compilation sb-c:*compilation*)
+              (entry-points (sb-c::entry-points compilation)))
+         (and (sb-c::block-compile compilation)
+              entry-points
+              (not (member name entry-points :test #'equal))))))
+
 (flet ((defun-expander (env name lambda-list body snippet)
   (multiple-value-bind (forms decls doc) (parse-body body t)
     ;; Maybe kill docstring, but only under the cross-compiler.
@@ -108,9 +116,12 @@
       `(progn
          (eval-when (:compile-toplevel)
            (sb-c:%compiler-defun ',name t ,inline-thing ,extra-info))
-         ,(if (sb-c::block-compilation-non-entry-point name)
+         ,(if (block-compilation-non-entry-point name)
               `(progn
-                 ,named-lambda
+                 ;; Tell the compiler to convert the lambda without
+                 ;; referencing it, so no stray reference stays around
+                 ;; and find-initial-dfo works correctly.
+                 (sb-c::%fun-name-leaf ,named-lambda)
                  ',name)
               `(%defun ',name ,named-lambda
                        ,@(when (or inline-thing extra-info) `(,inline-thing))
@@ -143,7 +154,82 @@
   (sb-xc:defmacro sb-c:xdefun (&environment env name snippet lambda-list &body body)
     (defun-expander env name lambda-list body snippet)))
 
-;;;; DEFVAR and DEFPARAMETER
+;;;; DEFCONSTANT, DEFVAR and DEFPARAMETER
+
+(sb-xc:defmacro defconstant (name value &optional (doc nil docp))
+  "Define a global constant, saying that the value is constant and may be
+  compiled into code. If the variable already has a value, and this is not
+  EQL to the new value, the code is not portable (undefined behavior). The
+  third argument is an optional documentation string for the variable."
+  (check-designator name defconstant)
+  `(eval-when (:compile-toplevel :load-toplevel :execute)
+     (sb-c::%defconstant ',name ,value (sb-c:source-location)
+                         ,@(and docp `(',doc)))))
+
+
+(declaim (ftype (sfunction (symbol t &optional t t) null)
+                about-to-modify-symbol-value))
+;;; the guts of DEFCONSTANT
+
+(defun sb-c::%defconstant (name value source-location &optional (doc nil docp))
+  #+sb-xc-host (declare (ignore doc docp))
+  (unless (symbolp name)
+    (error "The constant name is not a symbol: ~S" name))
+  (with-single-package-locked-error (:symbol name
+                                             "defining ~s as a constant")
+   (when (looks-like-name-of-special-var-p name)
+     (style-warn 'asterisks-around-constant-variable-name
+                 :format-control "Defining ~S as a constant"
+                 :format-arguments (list name)))
+   (when source-location
+     (setf (info :source-location :constant name) source-location))
+   (let ((kind (info :variable :kind name)))
+     (case kind
+       (:constant
+        ;; Note: This behavior (discouraging any non-EQL modification)
+        ;; is unpopular, but it is specified by ANSI (i.e. ANSI says a
+        ;; non-EQL change has undefined consequences). If people really
+        ;; want bindings which are constant in some sense other than
+        ;; EQL, I suggest either just using DEFVAR (which is usually
+        ;; appropriate, despite the un-mnemonic name), or defining
+        ;; something like the DEFCONSTANT-EQX macro used in SBCL (which
+        ;; is occasionally more appropriate). -- WHN 2001-12-21
+        (if (boundp name)
+            (if (typep name '(or boolean keyword))
+                ;; Non-continuable error.
+                (about-to-modify-symbol-value name 'defconstant)
+                (let ((old (symbol-value name)))
+                  (unless (eql value old)
+                    (multiple-value-bind (ignore aborted)
+                        (with-simple-restart (abort "Keep the old value.")
+                          (cerror "Go ahead and change the value."
+                                  'defconstant-uneql
+                                  :name name
+                                  :old-value old
+                                  :new-value value))
+                      (declare (ignore ignore))
+                      (when aborted
+                        (return-from sb-c::%defconstant name))))))
+            (warn "redefining a MAKUNBOUND constant: ~S" name)))
+       (:unknown
+        ;; (This is OK -- undefined variables are of this kind. So we
+        ;; don't warn or error or anything, just fall through.)
+        )
+       (t (warn "redefining ~(~A~) ~S to be a constant" kind name)))))
+  ;; We ought to be consistent in treating any change of :VARIABLE :KIND
+  ;; as a continuable error. The above CASE expression pre-dates the
+  ;; existence of symbol-macros (I believe), but at a bare minimum,
+  ;; INFO should return NIL for its second value if requesting the
+  ;; :macro-expansion of something that is getting defined as constant.
+  (clear-info :variable :macro-expansion name)
+  (clear-info :source-location :symbol-macro name)
+  #-sb-xc-host
+  (progn
+    (when docp
+      (setf (documentation name 'variable) doc))
+    (%set-symbol-value name value))
+  (setf (info :variable :kind name) :constant)
+  name)
 
 (sb-xc:defmacro defvar (var &optional (val nil valp) (doc nil docp))
   "Define a special variable at top level. Declare the variable
@@ -262,8 +348,9 @@ evaluated as a PROGN."
 (sb-xc:defmacro prog1 (result &body body)
   (let ((n-result (gensym)))
     `(let ((,n-result ,result))
-       ,@body
-       ,n-result)))
+       (progn
+         ,@body
+         ,n-result))))
 
 (sb-xc:defmacro prog2 (form1 result &body body)
   `(prog1 (progn ,form1 ,result) ,@body))
@@ -767,6 +854,16 @@ symbol-case giving up: case=((V U) (F))
                     (values score1 answer1) ; not improved
                     (values score2 answer2))))))))) ; 2 bytes = better
 
+;;; TODO: see if it's possible to not use SXHASH on layouts here.
+;;;  (etypecase x
+;;;    ((or named-type numeric-type member-type classoid ..) something)
+;;; -> (SB-IMPL::%SEALED-STRUCT-TYPECASE-INDEX
+;;;     ((named-type numeric-type member-type classoid ...)CHARACTER-SET-TYPE
+;;; -> (LET* ((ARRAY
+;;;            #(4048 NIL NIL NIL NIL NIL
+;;;              ((#<LAYOUT for ALIEN-TYPE-TYPE {50202A03}> . 1)) NIL
+;;;              ((#<LAYOUT for NAMED-TYPE {50202503}> . 1)) NIL NIL
+;;;
 (defun build-sealed-struct-typecase-map (type-unions)
   (let* ((layout-lists
           (mapcar (lambda (x) (mapcar #'find-layout x)) type-unions))
@@ -1174,11 +1271,13 @@ symbol-case giving up: case=((V U) (F))
              (block ,block
                (when ,is-hashable
                  (let ((,hash ,calc-hash))
+                   (declare (sb-c::no-constraints ,hash))
                    ;; At most two probes are required, and usually just 1
                    (when ,(ecase maxprobes
                             (1 `(eq (svref ,symbol-vector ,hash) ,symbol))
                             (2 `(let ((,vector ,symbol-vector)
                                       (,hash (ash ,hash 1)))
+                                  (declare (sb-c::no-constraints ,hash))
                                   (or (eq (svref ,vector ,hash) ,symbol)
                                       (eq (svref ,vector (1+ ,hash)) ,symbol)))))
                      (return-from ,block
@@ -1466,60 +1565,6 @@ symbol-case giving up: case=((V U) (F))
                 (setq ,abortp nil))
            (when ,stream
              (close ,stream :abort ,abortp)))))))
-
-(sb-xc:defmacro with-input-from-string ((var string &key index start end)
-                                            &body forms-decls)
-  (multiple-value-bind (forms decls) (parse-body forms-decls nil)
-    `(let ((,var
-            ;; Should (WITH-INPUT-FROM-STRING (stream str :start nil :end 5))
-            ;; pass the explicit NIL, and thus get an error? It's logical
-            ;; because an explicit NIL does not mean "default" in any other
-            ;; string operation. So why does it here?
-            ,(if (null end)
-                 `(make-string-input-stream ,string ,@(if start (list start)))
-                 `(make-string-input-stream ,string ,(or start 0) ,end))))
-         ,@decls
-         (multiple-value-prog1
-             (unwind-protect
-                  (progn ,@forms)
-               (close ,var))
-           ,@(when index
-               `((setf ,index (string-input-stream-current ,var))))))))
-
-(sb-xc:defmacro with-output-to-string
-    ((var &optional string &key (element-type ''character))
-     &body forms-decls)
-  (multiple-value-bind (forms decls) (parse-body forms-decls nil)
-    (if string
-        (let ((element-type-var (gensym)))
-          `(let ((,var (make-fill-pointer-output-stream ,string))
-                 ;; ELEMENT-TYPE isn't currently used for anything
-                 ;; (see FILL-POINTER-OUTPUT-STREAM FIXME in stream.lisp),
-                 ;; but it still has to be evaluated for side-effects.
-                 (,element-type-var ,element-type))
-             (declare (ignore ,element-type-var))
-             ,@decls
-             (unwind-protect
-                  (progn ,@forms)
-               (close ,var))))
-        ;; I don't see why we need the unwind-protect.
-        ;; CLHS says: "The output string stream to which the variable /var/
-        ;; is bound has dynamic extent; its extent ends when the form is exited."
-        ;; So technically you can't reference the stream after the string
-        ;; is returned. And since the implication is that we can legally DXify
-        ;; the stream, what difference does it make whether it's open or closed?
-        ;; Certainly in code that is compiled in SAFETY < 3 we should
-        ;; just not bother with the unwind-protect or the close.
-        `(let ((,var (make-string-output-stream
-                      ;; CHARACTER is the default element-type of
-                      ;; string-ouput-stream, save a few bytes when passing it
-                      ,@(and (not (equal element-type ''character))
-                             `(:element-type ,element-type)))))
-           ,@decls
-           (unwind-protect
-                (progn ,@forms)
-             (close ,var))
-           (get-output-stream-string ,var)))))
 
 ;;;; Iteration macros:
 
@@ -1676,9 +1721,6 @@ symbol-case giving up: case=((V U) (F))
                  `(sb-c::%proclaim ',spec (sb-c:source-location)))
                specs)))
 
-;; Avoid unknown return values in emitted code for PRINT-UNREADABLE-OBJECT
-(sb-xc:proclaim '(ftype (sfunction (t t t &optional t) null)
-                        %print-unreadable-object))
 (sb-xc:defmacro print-unreadable-object ((object stream &key type identity)
                                              &body body)
   "Output OBJECT to STREAM with \"#<\" prefix, \">\" suffix, optionally
@@ -1691,7 +1733,7 @@ symbol-case giving up: case=((V U) (F))
                 ,object ,stream (logior (if ,type 1 0) (if ,identity 2 0)))))
     (if body
         (let ((fun (make-symbol "THUNK")))
-          `(dx-flet ((,fun () ,@body)) (,@call #',fun)))
+          `(dx-flet ((,fun () (progn ,@body))) (,@call #',fun)))
         call)))
 
 ;; A macroexpander helper. Not sure where else to put this.
@@ -1729,3 +1771,33 @@ symbol-case giving up: case=((V U) (F))
                     ;; is cheaper than a branch around a write.
                     (rplacd (truly-the cons ,splice) ,orig)))
             ,copy))))))
+
+(defun expand-with-output-to-string (var element-type body wild-result-type)
+  ;; This is simpler than trying to arrange transforms that cause
+  ;; MAKE-STRING-OUTPUT-STREAM to be DXable. While that might be awesome,
+  ;; this macro exists for a reason.
+  (let ((initial-buffer '#:buf)
+        (dummy '#:stream)
+        (string-let (or #+c-stack-is-control-stack 'dx-let 'let))
+        (string-ctor
+          (if (and (sb-xc:constantp element-type)
+                   (let ((ctype (sb-c::careful-specifier-type
+                                 (constant-form-value element-type))))
+                     (and ctype
+                          (csubtypep ctype (specifier-type 'character)))))
+              ;; Using MAKE-ARRAY avoids a style-warning if et is 'STANDARD-CHAR:
+              ;; "The default initial element #\Nul is not a STANDARD-CHAR."
+              'make-array ; hooray! it's known be a valid string type
+              ;; Force a runtime STRINGP check unless futher transforms
+              ;; deduce a known type. You'll get "could not stack allocate"
+              ;; perhaps, but that's acceptable.
+              'make-string)))
+    ;; A full call to MAKE-STRING-OUTPUT-STREAM uses a larger initial buffer
+    ;; if BASE-CHAR but I really don't care to think about that here.
+    `(,string-let ((,initial-buffer (,string-ctor 31 :element-type ,element-type)))
+       (dx-let ((,dummy (%allocate-string-ostream)))
+         (let ((,var (%init-string-output-stream ,dummy ,initial-buffer
+                                                 ,wild-result-type)))
+           (declare (ignorable ,var))
+           ,@body)
+         (get-output-stream-string ,dummy)))))
