@@ -73,6 +73,11 @@
     '#(sb-impl::*exit-lock*
        sb-thread::*make-thread-lock*
        sb-thread::*initial-thread*
+       ;; Saving *JOINABLE-THREADS* could cause catastophic failure on restart.
+       ;; SAVE-LISP-AND-DIE should have cleaned up, but there's a timing problem
+       ;; with the finalizer thread, and I'm loathe to put in a SLEEP delay.
+       sb-thread::*joinable-threads*
+       sb-thread::*foreign-thread*
        sb-thread::*all-threads*
        sb-thread::*session*
        sb-kernel::*gc-epoch*))
@@ -187,7 +192,8 @@ sufficiently motivated to do lengthy fixes."
   (declare (ignore purify) (ignorable root-structures))
   ;; If the toplevel function is not defined, this will signal an
   ;; error before saving, not at startup time.
-  (let ((toplevel (%coerce-callable-to-fun toplevel)))
+  (let ((toplevel (%coerce-callable-to-fun toplevel))
+        *streams-closed-by-slad*)
     #+sb-core-compression
     (check-type compression (or boolean (integer -1 9)))
     #-sb-core-compression
@@ -269,6 +275,7 @@ sufficiently motivated to do lengthy fixes."
 
     ;; Something went very wrong -- reinitialize to have a prayer
     ;; of being able to report the error.
+    (restore-fd-streams)
     (reinit)
     (error 'save-error)))
 
@@ -298,14 +305,16 @@ sufficiently motivated to do lengthy fixes."
   (call-hooks "save" *save-hooks*)
   #+sb-wtimer
   (itimer-emulation-deinit)
+  ;; FIXME: this logic should probably be surrounded by the *MAKE-THREAD-LOCK*
+  ;; but introducing potential new sources of deadlock requires due diligence.
   #+sb-thread
   (progn
     ;; Terminate finalizer thread now, especially given that the thread runs
     ;; user-supplied code that might not even work in later steps of deinit.
     ;; See also the comment at definition of THREAD-EPHEMERAL-P.
     (finalizer-thread-stop)
+    #+pauseless-threadstart (sb-thread::join-pthread-joinables #'identity)
     (let ((threads (sb-thread:list-all-threads)))
-      (when (cdr threads)
         ;; Despite calling FINALIZER-THREAD-STOP, an unlucky thread schedule
         ;; courtesy of the OS might notionally start the finalizer thread but not
         ;; run it until now. Suppose it barely got to its lisp trampoline,
@@ -316,26 +325,35 @@ sufficiently motivated to do lengthy fixes."
         ;; A newly started finalizer thread can do nothing but exit immediately
         ;; whenever *FINALIZER-THREAD* is NIL so we don't actually care
         ;; what that thread is doing at this point.
-        (let ((finalizer (find-if (lambda (x)
-                                    (and (sb-thread::thread-%ephemeral-p x)
-                                         (string= (sb-thread:thread-name x) "finalizer")))
-                                  threads)))
-          (when finalizer
-            (sb-thread:join-thread finalizer)))
-        ;; Regardless of what happened above, grab the all-threads list again.
-        (setq threads (sb-thread:list-all-threads)))
-      (when (cdr threads)
-        (let* ((interactive (sb-thread::interactive-threads))
-               (other (set-difference threads interactive)))
-          (error 'save-with-multiple-threads-error
-                 :interactive-threads interactive
-                 :other-threads other)))))
+      (flet ((system-thread-p (thread)
+               (and (sb-thread::thread-%ephemeral-p thread)
+                    (string= (sb-thread:thread-name thread) "finalizer"))))
+        (when (cdr threads)
+          (let ((finalizer (find-if #'system-thread-p threads)))
+            (when finalizer
+              (sb-thread:join-thread finalizer)))
+          ;; Regardless of what happened above, grab the all-threads list again.
+          (setq threads (sb-thread:list-all-threads)))
+        (let ((starting
+                (setq sb-thread::*starting-threads* ; ordinarily pruned in MAKE-THREAD
+                      (delete 0 sb-thread::*starting-threads*)))
+              (joinable
+                (remove-if #'system-thread-p sb-thread::*joinable-threads*)))
+          (when (or (cdr threads) starting joinable)
+            (let* ((interactive (sb-thread::interactive-threads))
+                   (other (union (set-difference threads interactive)
+                                 (union starting joinable))))
+              (error 'save-with-multiple-threads-error
+                     :interactive-threads interactive
+                     :other-threads other)))))))
   (tune-image-for-dump)
   (float-deinit)
   (profile-deinit)
   (foreign-deinit)
-  (finalizers-deinit)
-  (fill *pathnames* nil)
+  ;; To have any hope of making pathname interning actually work,
+  ;; this CLRHASH would need to be removed. But removing it causes excess
+  ;; garbage retention because weakness doesn't work. It's a catch-22.
+  (clrhash *pathnames*)
   ;; Clean up the simulated weak list of covered code components.
   (rplacd sb-c:*code-coverage-info*
           (delete-if-not #'weak-pointer-value (cdr sb-c:*code-coverage-info*)))
@@ -347,6 +365,8 @@ sufficiently motivated to do lengthy fixes."
   ;; Perform static linkage. Functions become un-statically-linked
   ;; on demand, for TRACE, redefinition, etc.
   #+immobile-code (sb-vm::statically-link-core)
+  (invalidate-fd-streams)
+  (finalizers-deinit)
   ;; Do this last, to have some hope of printing if we need to.
   (stream-deinit)
   (setf * nil ** nil *** nil
@@ -388,8 +408,9 @@ sufficiently motivated to do lengthy fixes."
                     ;; ctype, because that's already a canonical object.
                     (not (minusp (type-hash-value part)))))
              (coalesce (type &aux
-                             ;; Deal with ctypes instances whose unparser fails.
-                             (spec (ignore-errors (type-specifier type))))
+                               ;; Deal with ctypes instances whose unparser fails.
+                               (spec (and (not (contains-unknown-type-p type))
+                                          (ignore-errors (type-specifier type)))))
                ;; There are ctypes that unparse to the same s-expression
                ;; but are *NOT* TYPE=. Some examples:
                ;;   classoid LIST  vs UNION-TYPE LIST  = (OR CONS NULL)
@@ -470,6 +491,7 @@ sufficiently motivated to do lengthy fixes."
 
 sb-c::
 (defun coalesce-debug-info ()
+  #+cheneygc (clrhash sb-di::*compiled-debug-funs*)
   (flet ((debug-source= (a b)
            (and (equal (debug-source-plist a) (debug-source-plist b))
                 (eql (debug-source-created a) (debug-source-created b)))))
@@ -487,6 +509,12 @@ sb-c::
          (declare (ignore size))
          (case widetag
           (#.sb-vm:code-header-widetag
+           (let ((di (sb-vm::%%code-debug-info obj)))
+             ;; Discard memoized debugger's debug info
+             (when (typep di 'sb-c::compiled-debug-info)
+               (let ((thing (sb-c::compiled-debug-info-tlf-num+offset di)))
+                 (when (consp thing)
+                   (setf (sb-c::compiled-debug-info-tlf-num+offset di) (car thing))))))
            (dotimes (i (sb-kernel:code-n-entries obj))
              (let* ((fun (sb-kernel:%code-entry-point obj i))
                     (arglist (%simple-fun-arglist fun))
@@ -505,7 +533,7 @@ sb-c::
             (compiled-debug-info
              (let ((source (compiled-debug-info-source obj)))
                (typecase source
-                 (core-debug-source)    ; skip
+                 (core-debug-source)    ; skip - uh, why?
                  (debug-source
                   (let* ((namestring (debug-source-namestring source))
                          (canonical-repr
