@@ -14,7 +14,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; Return T if FUN names a DEFSTRUCT slot accessor that we should
 ;;; transform from SETF into %INSTANCE-SET directly - bypassing
@@ -29,8 +29,8 @@
            (slot-info (structure-instance-accessor-p fun)))
       (when (and slot-info (not (dsd-read-only (cdr slot-info))))
         (dx-let ((setter `(setf ,fun)))
-          (when (and (not (sb!c::fun-locally-defined-p setter env))
-                     (not (sb!c::fun-lexically-notinline-p setter env)))
+          (when (and (not (sb-c::fun-locally-defined-p setter env))
+                     (not (sb-c::fun-lexically-notinline-p setter env)))
             slot-info)))))) ; caller needs the (DD . DSD) pair
 
 ;;; The inverse for a generalized-variable reference function is stored in
@@ -46,15 +46,14 @@
 ;;; of value forms, a list of the single store-value form, a storing function,
 ;;; and an accessing function.
 (declaim (ftype (function (t &optional lexenv-designator))
-                sb!xc:get-setf-expansion))
-(defun sb!xc:get-setf-expansion (form &optional environment)
-  #!+sb-doc
+                sb-xc:get-setf-expansion))
+(defun sb-xc:get-setf-expansion (form &optional environment)
   "Return five values needed by the SETF machinery: a list of temporary
    variables, a list of values with which to fill them, a list of temporaries
    for the new values, the setting function, and the accessing function."
   (named-let retry ((form form))
     (labels ((newvals (count)
-               (let ((sb!xc:*gensym-counter* 1))
+               (let ((sb-xc:*gensym-counter* 1))
                  (make-gensym-list count "NEW")))
              ;; Produce the expansion of a SETF form that calls either
              ;; #'(SETF name) or an inverse given by short form DEFSETF.
@@ -64,19 +63,19 @@
                  (values temp-vars temp-vals vals
                          `(,.call ,@(funcall arg-maker (car vals) args))
                          `(,(car form) ,@args)))))
-      (declare (ftype (function (t) list) newvals))
+      (declare (ftype (sfunction (t) list) newvals))
       (if (atom form)
           (multiple-value-bind (expansion expanded)
               ;; Previously this called %MACROEXPAND, but the two operations
               ;; are equivalent on atoms, so do the one that is "less".
-              (sb!xc:macroexpand-1 form environment)
+              (sb-xc:macroexpand-1 form environment)
             (if expanded
                 (retry expansion)
                 (let ((vals (newvals 1)))
                   (values nil nil vals `(setq ,form ,(car vals)) form))))
           (let ((fname (car form)))
             ;; Local functions inhibit global SETF methods.
-            (unless (sb!c::fun-locally-defined-p fname environment)
+            (unless (sb-c::fun-locally-defined-p fname environment)
               (awhen (info :setf :expander fname)
                 (return-from retry
                   (typecase it
@@ -158,8 +157,7 @@
                          (list form)))
                    forms)))
 
-  (sb!xc:defmacro setf (&whole form &rest args &environment env)
-  #!+sb-doc
+  (sb-xc:defmacro setf (&whole form &rest args &environment env)
   "Takes pairs of arguments like SETQ. The first is a place and the second
   is the value that is supposed to go into that place. Returns the last
   value. The place argument may be any of the access forms for which SETF
@@ -168,14 +166,14 @@
       (return-from setf nil))
     (destructuring-bind (place value-form . more) args
       (when more
-        (return-from setf `(progn ,@(sb!c::explode-setq form 'error))))
+        (return-from setf `(progn ,@(sb-c::explode-setq form 'error))))
       (when (atom (setq place (macroexpand-for-setf place env)))
         (return-from setf `(setq ,place ,value-form)))
 
       (let ((fun (car place)))
         (when (and (symbolp fun)
                    ;; Local definition of FUN precludes global knowledge.
-                   (not (sb!c::fun-locally-defined-p fun env)))
+                   (not (sb-c::fun-locally-defined-p fun env)))
           (let ((inverse (info :setf :expander fun)))
             ;; NIL is not a valid setf inverse name, for two reasons:
             ;;  1. you can't define a function named NIL,
@@ -188,14 +186,13 @@
                :setf (list (cadr place) value-form) it)))))
 
       (multiple-value-bind (temps vals newval setter)
-          (sb!xc:get-setf-expansion place env)
+          (sb-xc:get-setf-expansion place env)
         (car (gen-let* (mapcar #'list temps vals)
                        (gen-mv-bind newval value-form (forms-list setter)))))))
 
   ;; various SETF-related macros
 
-  (sb!xc:defmacro shiftf (&whole form &rest args &environment env)
-  #!+sb-doc
+  (sb-xc:defmacro shiftf (&whole form &rest args &environment env)
   "One or more SETF-style place expressions, followed by a single
    value expression. Evaluates all of the expressions in turn, then
    assigns the value of each expression to the place on its left,
@@ -205,7 +202,7 @@
   (collect ((let*-bindings) (mv-bindings) (setters) (getters))
     (dolist (arg (butlast args))
       (multiple-value-bind (temps subforms store-vars setter getter)
-          (sb!xc:get-setf-expansion arg env)
+          (sb-xc:get-setf-expansion arg env)
         (let*-bindings (mapcar #'list  temps subforms))
         (mv-bindings store-vars)
         (setters setter)
@@ -219,7 +216,7 @@
                                 (thunk (cdr mv-bindings) (cdr getters) setters))
                    setters)))
       (let ((outputs (loop for i below (length (car (mv-bindings)))
-                           collect (sb!xc:gensym "OUT"))))
+                           collect (sb-xc:gensym "OUT"))))
         (car (gen-let* (reduce #'nconc (let*-bindings))
                        (gen-mv-bind outputs (car (getters))
                                     (thunk (mv-bindings) (cdr (getters))
@@ -240,11 +237,9 @@
              (let ((place (car a))
                    (value-form (cadr a)))
                (when (and (not (symbolp place)) (eq operator 'psetq))
-                 (error 'simple-program-error
-                        :format-control "Place ~S in PSETQ is not a SYMBOL"
-                        :format-arguments (list place)))
+                 (%program-error "Place ~S in PSETQ is not a SYMBOL" place))
                (multiple-value-bind (temps vals stores setter)
-                   (sb!xc:get-setf-expansion place env)
+                   (sb-xc:get-setf-expansion place env)
                  (let*-bindings (mapcar #'list temps vals))
                  (mv-bindings (cons stores value-form))
                  (setters setter))))
@@ -258,24 +253,21 @@
                                            setters)))
              `(,@setters nil))))
 
-  (sb!xc:defmacro psetf (&rest pairs &environment env)
-  #!+sb-doc
+  (sb-xc:defmacro psetf (&rest pairs &environment env)
   "This is to SETF as PSETQ is to SETQ. Args are alternating place
   expressions and values to go into those places. All of the subforms and
   values are determined, left to right, and only then are the locations
   updated. Returns NIL."
     (expand pairs env 'psetf 'setf))
 
-  (sb!xc:defmacro psetq (&rest pairs &environment env)
-  #!+sb-doc
+  (sb-xc:defmacro psetq (&rest pairs &environment env)
   "PSETQ {var value}*
    Set the variables to the values, like SETQ, except that assignments
    happen in parallel, i.e. no assignments take place until all the
    forms have been evaluated."
     (expand pairs env 'psetq 'setq))))
 
-(sb!xc:defmacro rotatef (&rest args &environment env)
-  #!+sb-doc
+(sb-xc:defmacro rotatef (&rest args &environment env)
   "Takes any number of SETF-style place expressions. Evaluates all of the
    expressions in turn, then assigns to each place the value of the form to
    its right. The rightmost form gets the value of the leftmost.
@@ -284,7 +276,7 @@
     (collect ((let*-bindings) (mv-bindings) (setters) (getters))
       (dolist (arg args)
         (multiple-value-bind (temps subforms store-vars setter getter)
-            (sb!xc:get-setf-expansion arg env)
+            (sb-xc:get-setf-expansion arg env)
           (let*-bindings (mapcar #'list temps subforms))
           (mv-bindings store-vars)
           (setters setter)
@@ -299,8 +291,7 @@
         `(let* ,(reduce #'append (let*-bindings))
            ,@(thunk (mv-bindings) (cdr (getters))))))))
 
-(sb!xc:defmacro push (obj place &environment env)
-  #!+sb-doc
+(sb-xc:defmacro push (obj place &environment env)
   "Takes an object and a location holding a list. Conses the object onto
   the list, returning the modified list. OBJ is evaluated before PLACE."
   ;; If PLACE has multiple store locations, what should we do?
@@ -310,11 +301,13 @@
   ;; - At least two produce an incorrect expansion that doesn't even work.
   (expand-rmw-macro 'cons (list obj) place '() nil env '(item)))
 
-(sb!xc:defmacro pushnew (obj place &rest keys &environment env)
-  #!+sb-doc
+(sb-xc:defmacro pushnew (obj place &rest keys &environment env)
   "Takes an object and a location holding a list. If the object is
   already in the list, does nothing; otherwise, conses the object onto
   the list. Keyword arguments are accepted as per the ADJOIN function."
+  ;; Can't specify the actual keywords above since, apparently,
+  ;; non-constant keywords should be accepted.
+  (declare (sb-c::lambda-list (obj place &key key test test-not)))
   ;; Passing AFTER-ARGS-BINDP = NIL causes the forms subsequent to PLACE
   ;; to be inserted literally as-is, giving the (apparently) desired behavior
   ;; of *not* evaluating them before the Read/Modify/Write of PLACE, which
@@ -322,14 +315,13 @@
   ;; The spec only mentions that ITEM is eval'd before PLACE.
   (expand-rmw-macro 'adjoin (list obj) place keys nil env '(item)))
 
-(sb!xc:defmacro pop (place &environment env)
-  #!+sb-doc
+(sb-xc:defmacro pop (place &environment env)
   "The argument is a location holding a list. Pops one item off the front
   of the list and returns it."
   (if (symbolp (setq place (macroexpand-for-setf place env)))
       `(prog1 (car ,place) (setq ,place (cdr ,place)))
       (multiple-value-bind (temps vals stores setter getter)
-          (sb!xc:get-setf-expansion place env)
+          (sb-xc:get-setf-expansion place env)
         (let ((list (copy-symbol 'list))
               (ret (copy-symbol 'car)))
           `(let* (,@(mapcar #'list temps vals)
@@ -340,14 +332,13 @@
              ,setter
              ,ret)))))
 
-(sb!xc:defmacro remf (place indicator &environment env)
-  #!+sb-doc
+(sb-xc:defmacro remf (place indicator &environment env)
   "Place may be any place expression acceptable to SETF, and is expected
   to hold a property list or (). This list is destructively altered to
   remove the property specified by the indicator. Returns T if such a
   property was present, NIL if not."
   (multiple-value-bind (temps vals newval setter getter)
-      (sb!xc:get-setf-expansion place env)
+      (sb-xc:get-setf-expansion place env)
     (let* ((flag (make-symbol "FLAG"))
            (body `(multiple-value-bind (,(car newval) ,flag)
               ;; See ANSI 5.1.3 for why we do out-of-order evaluation
@@ -389,28 +380,26 @@
          (if (symbolp (setq place (macroexpand-for-setf place env)))
              `(setq ,place (,operator ,delta ,place))
              (multiple-value-bind (dummies vals newval setter getter)
-                 (sb!xc:get-setf-expansion place env)
+                 (sb-xc:get-setf-expansion place env)
                `(let* (,@(mapcar #'list dummies vals)
                        (,(car newval) (,operator ,delta ,getter))
                        ,@(cdr newval))
                   ,setter)))))
-  (sb!xc:defmacro incf (place &optional (delta 1) &environment env)
-  #!+sb-doc
+  (sb-xc:defmacro incf (place &optional (delta 1) &environment env)
   "The first argument is some location holding a number. This number is
   incremented by the second argument, DELTA, which defaults to 1."
     (expand place delta env '+))
 
-  (sb!xc:defmacro decf (place &optional (delta 1) &environment env)
-  #!+sb-doc
+  (sb-xc:defmacro decf (place &optional (delta 1) &environment env)
   "The first argument is some location holding a number. This number is
   decremented by the second argument, DELTA, which defaults to 1."
     (expand place delta env 'xsubtract)))
 
 ;;;; DEFINE-MODIFY-MACRO stuff
 
-(sb!xc:defmacro sb!xc:define-modify-macro (name lambda-list function &optional doc-string)
-  #!+sb-doc
+(sb-xc:defmacro define-modify-macro (name lambda-list function &optional doc-string)
   "Creates a new read-modify-write macro like PUSH or INCF."
+  (check-designator name define-modify-macro)
   (binding* (((nil required optional rest)
               (parse-lambda-list
                lambda-list
@@ -421,59 +410,57 @@
                                    optional)))
              (place (make-symbol "PLACE"))
              (env (make-symbol "ENV")))
-    `(sb!xc:defmacro ,name (,place ,@lambda-list &environment ,env)
+    `(sb-xc:defmacro ,name (,place ,@lambda-list &environment ,env)
        ,@(when doc-string (list (the string doc-string)))
        (expand-rmw-macro ',function '() ,place
                          (list* ,@args ,(car rest)) t ,env ',args))))
 
 ;;;; DEFSETF
 
-(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-  ;;; Assign SETF macro information for NAME, making all appropriate checks.
-  (defun %defsetf (name expander &optional doc)
-    (with-single-package-locked-error
-        (:symbol name "defining a setf-expander for ~A"))
-    (let ((setf-fn-name `(setf ,name)))
-      (multiple-value-bind (where-from present-p)
-          (info :function :where-from setf-fn-name)
-        ;; One might think that :DECLARED merits a style warning, but SBCL
-        ;; provides ~58 standard accessors as both (SETF F) and a macro.
-        ;; So allow the user to declaim an FTYPE and we'll hush up.
-        ;; What's good for the the goose is good for the gander.
-        (case where-from
-          (:assumed
-           ;; This indicates probable user error. Compilation assumed something
-           ;; to be functional; a macro says otherwise. Because :where-from's
-           ;; default can be :assumed, PRESENT-P disambiguates "defaulted" from
-           ;; "known" to have made an existence assumption.
-           (when present-p
-             (warn "defining setf macro for ~S when ~S was previously ~
+;;; Assign SETF macro information for NAME, making all appropriate checks.
+(defun %defsetf (name expander &optional doc)
+  (declare (ignorable doc))
+  (with-single-package-locked-error
+      (:symbol name "defining a setf-expander for ~A"))
+  (let ((setf-fn-name `(setf ,name)))
+    (multiple-value-bind (where-from present-p)
+        (info :function :where-from setf-fn-name)
+      ;; One might think that :DECLARED merits a style warning, but SBCL
+      ;; provides ~58 standard accessors as both (SETF F) and a macro.
+      ;; So allow the user to declaim an FTYPE and we'll hush up.
+      ;; What's good for the the goose is good for the gander.
+      (case where-from
+        (:assumed
+         ;; This indicates probable user error. Compilation assumed something
+         ;; to be functional; a macro says otherwise. Because :where-from's
+         ;; default can be :assumed, PRESENT-P disambiguates "defaulted" from
+         ;; "known" to have made an existence assumption.
+         (when present-p
+           (warn "defining setf macro for ~S when ~S was previously ~
              treated as a function" name setf-fn-name)))
-          ;; This is a useless and unavoidable warning during self-build.
-          ;; cf. similar disabling of warning in WARN-IF-SETF-MACRO.
-          #-sb-xc-host
-          (:defined
-           ;; Somebody defined (SETF F) but then also said F has a macro.
-           ;; A soft warning seems appropriate because in this case it's
-           ;; at least in theory not wrong to call the function.
-           ;; The user can declare an FTYPE if both things are intentional.
-           (style-warn "defining setf macro for ~S when ~S is also defined"
-                       name setf-fn-name)))))
-    (setf (info :setf :expander name) expander)
-    (when doc
-      (setf (fdocumentation name 'setf) doc))
-    name))
+        ;; This is a useless and unavoidable warning during self-build.
+        ;; cf. similar disabling of warning in WARN-IF-SETF-MACRO.
+        #-sb-xc-host
+        (:defined
+         ;; Somebody defined (SETF F) but then also said F has a macro.
+         ;; A soft warning seems appropriate because in this case it's
+         ;; at least in theory not wrong to call the function.
+         ;; The user can declare an FTYPE if both things are intentional.
+         (style-warn "defining setf macro for ~S when ~S is also defined"
+                     name setf-fn-name)))))
+  (setf (info :setf :expander name) expander)
+  #-sb-xc-host
+  (when doc
+    (setf (documentation name 'setf) doc))
+  name)
 
 ;;; This is pretty broken if there are keyword arguments (lp#1452947)
 ;;; but the bug seems to be due to irreconcilable problems in the spec.
 ;;; Everybody seems to interpret the spec the way we do though.
-(sb!xc:defmacro sb!xc:defsetf (access-fn &rest rest)
-  #!+sb-doc
+(sb-xc:defmacro defsetf (access-fn &rest rest)
   "Associates a SETF update function or macro with the specified access
   function or macro. The format is complex. See the manual for details."
-  (unless (symbolp access-fn)
-    (error "~S access-function name ~S is not a symbol."
-           'sb!xc:defsetf access-fn))
+  (check-designator access-fn defsetf "access-function")
   (typecase rest
     ((cons (and symbol (not null)) (or null (cons string null)))
      `(eval-when (:load-toplevel :compile-toplevel :execute)
@@ -499,7 +486,7 @@
                       (cons ,(length stores)
                             (named-lambda (%defsetf ,access-fn)
                                           (,subforms ,env-var ,@stores)
-                              (declare (sb!c::lambda-list ,lambda-list))
+                              (declare (sb-c::lambda-list ,lambda-list))
                               ,@(if outer-decls (list outer-decls))
                               ,@(unless env `((declare (ignore ,env-var))))
                               (apply (lambda ,lambda-list
@@ -532,7 +519,7 @@
       (let ((mask 0) (bit 1))
         (dolist (form sexprs (values (temp-vars) (temp-vals) (call-arguments)
                                      mask))
-          (call-arguments (if (sb!xc:constantp form environment)
+          (call-arguments (if (sb-xc:constantp form environment)
                               (progn (next-name-hint) form) ; Skip one hint.
                               (let ((temp (nice-tempname form)))
                                 (setq mask (logior mask bit))
@@ -554,7 +541,7 @@
      (binding* (((before-temps before-vals before-args)
                  (collect-setf-temps before-arg-forms environment name-hints))
                 ((place-temps place-subforms stores setter getter)
-                 (sb!xc:get-setf-expansion place environment))
+                 (sb-xc:get-setf-expansion place environment))
                 ((after-temps after-vals after-args)
                  (if after-args-bindp
                      (collect-setf-temps after-arg-forms environment name-hints)
@@ -571,7 +558,7 @@
                            (every (lambda (x)
                                     (or (member x place-temps)
                                         (eq x newval-temp)
-                                        (sb!xc:constantp x environment)))
+                                        (sb-xc:constantp x environment)))
                                   (cdr setter)))))
          (setq newval-binding nil
                setter (substitute compute newval-temp setter)))
@@ -587,19 +574,17 @@
 ;;;; DEFMACRO DEFINE-SETF-EXPANDER and various DEFINE-SETF-EXPANDERs
 
 ;;; DEFINE-SETF-EXPANDER is a lot like DEFMACRO.
-(sb!xc:defmacro sb!xc:define-setf-expander (access-fn lambda-list &body body)
-  #!+sb-doc
+(sb-xc:defmacro define-setf-expander (access-fn lambda-list &body body)
   "Syntax like DEFMACRO, but creates a setf expander function. The body
   of the definition must be a form that returns five appropriate values."
-  (unless (symbolp access-fn)
-    (error "~S access-function name ~S is not a symbol."
-           'sb!xc:define-setf-expander access-fn))
+  (check-designator access-fn define-setf-expander "access-function")
   (multiple-value-bind (def doc)
       ;; Perhaps it would be more elegant to keep the docstring attached
       ;; to the expander function, as for CAS?
       (make-macro-lambda `(setf-expander ,access-fn) lambda-list body
-                         'sb!xc:define-setf-expander access-fn
+                         'sb-xc:define-setf-expander access-fn
                          :doc-string-allowed :external)
+    ;; Maybe kill docstring, but only under the cross-compiler.
+    #+(and (not sb-doc) sb-xc-host) (setq doc nil)
     `(eval-when (:compile-toplevel :load-toplevel :execute)
        (%defsetf ',access-fn ,def ,@(and doc `(,doc))))))
-
