@@ -7,7 +7,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 (declaim (special *read-suppress*))
 
@@ -48,7 +48,7 @@
 
 (defun sharp-star (stream ignore numarg)
   (declare (ignore ignore))
-  (declare (type (or null fixnum) numarg))
+  (declare (type (or null integer) numarg))
   (binding* (((buffer escape-appearedp) (read-extended-token stream))
              (input-len (token-buf-fill-ptr buffer))
              (bstring (token-buf-string buffer)))
@@ -70,7 +70,7 @@
                                  1 0)))
                 (i 0 (1+ i)))
                ((= i input-len) bvec)
-             (declare (index i) (optimize (sb!c::insert-array-bounds-checks 0)))
+             (declare (index i) (optimize (sb-c::insert-array-bounds-checks 0)))
              (let ((char (char bstring i)))
                (setf (elt bvec i)
                      (case char
@@ -86,37 +86,50 @@
             numarg
             (copy-token-buf-string buffer))))))
 
-(defun sharp-A (stream ignore dimensions)
+(defun sharp-A (stream ignore rank)
   (declare (ignore ignore))
   (when *read-suppress*
     (read stream t nil t)
     (return-from sharp-A nil))
-  (unless dimensions
-    (simple-reader-error stream "No dimensions argument to #A."))
-  (collect ((dims))
-    (let* ((*bq-error*
-            (if (zerop *backquote-depth*)
-                *bq-error*
-                "Comma inside a backquoted array (not a list or general vector.)"))
-           (*backquote-depth* 0)
-           (contents (read stream t nil t))
-           (seq contents))
-      (dotimes (axis dimensions
-                     (make-array (dims) :initial-contents contents))
-        (unless (typep seq 'sequence)
-          (simple-reader-error stream
-                               "#~WA axis ~W is not a sequence:~%  ~S"
-                               dimensions axis seq))
-        (let ((len (length seq)))
-          (dims len)
-          (unless (or (= axis (1- dimensions))
-                      ;; ANSI: "If some dimension of the array whose
-                      ;; representation is being parsed is found to be
-                      ;; 0, all dimensions to the right (i.e., the
-                      ;; higher numbered dimensions) are also
-                      ;; considered to be 0."
-                      (= len 0))
-            (setq seq (elt seq 0))))))))
+  (let* ((*bq-error*
+           (if (zerop *backquote-depth*)
+               *bq-error*
+               "Comma inside a backquoted array (not a list or general vector.)"))
+         (*backquote-depth* 0)
+         (contents (read stream t nil t)))
+    (cond
+      (rank
+       (collect ((dims))
+         (let ((seq contents))
+           (dotimes (axis rank
+                          (make-array (dims) :initial-contents contents))
+             (unless (typep seq 'sequence)
+               (simple-reader-error stream
+                                    "#~WA axis ~W is not a sequence:~%  ~S"
+                                    rank axis seq))
+             (let ((len (length seq)))
+               (dims len)
+               (unless (or (= axis (1- rank))
+                           ;; ANSI: "If some dimension of the array whose
+                           ;; representation is being parsed is found to be
+                           ;; 0, all dimensions to the right (i.e., the
+                           ;; higher numbered dimensions) are also
+                           ;; considered to be 0."
+                           (= len 0))
+                 (setq seq (elt seq 0))))))))
+      ;; It's not legal to have #A without the rank, use that for
+      ;; #A(dimensions element-type contents) to avoid using #. when
+      ;; printing specialized arrays readably.
+      ((list-of-length-at-least-p contents 2)
+       (destructuring-bind (dimensions type &rest contents) contents
+         (make-array dimensions :initial-contents contents
+                                :element-type type)))
+      (t
+       (simple-reader-error stream
+                            "~@<Array literal is neither of the ~
+                             standard form #<rank>A<contents> nor the ~
+                             SBCL-specific form #A(dimensions ~
+                             element-type . contents).~@:>")))))
 
 ;;;; reading structure instances: the #S readmacro
 
@@ -223,6 +236,9 @@
          ;; though the docs say this is undefined behavior, so it's ok,
          ;; other than it being something we should complain about
          ;; for portability reasons.
+         ;; Some other things that shouldn't work:
+         ;; * (read-from-string "#x a") => 10
+         ;; * (read-from-string "#x #+foo a b") => 11
          (let ((res (let ((*read-base* radix))
                       (read stream t nil t))))
            (unless (typep res 'rational)
@@ -249,47 +265,58 @@
 (declaim (freeze-type sharp-equal-wrapper))
 
 ;; This function is kind of like NSUBLIS, but checks for circularities and
-;; substitutes in arrays and structures as well as lists. The first arg is an
-;; alist of the things to be replaced assoc'd with the things to replace them.
-(defun circle-subst (circle-table tree)
-  (cond ((and (sharp-equal-wrapper-p tree)
-              (not (eq (sharp-equal-wrapper-value tree) +sharp-equal-marker+)))
-         (sharp-equal-wrapper-value tree))
-        ((null (gethash tree circle-table))
-         (setf (gethash tree circle-table) t)
-         (cond ((consp tree)
-                (let ((a (circle-subst circle-table (car tree)))
-                      (d (circle-subst circle-table (cdr tree))))
-                  (unless (eq a (car tree))
-                    (rplaca tree a))
-                  (unless (eq d (cdr tree))
-                    (rplacd tree d))))
-               ((arrayp tree)
-                (with-array-data ((data tree) (start) (end))
-                  (declare (fixnum start end))
-                  (do ((i start (1+ i)))
-                      ((>= i end))
-                    (let* ((old (aref data i))
-                           (new (circle-subst circle-table old)))
-                      (unless (eq old new)
-                        (setf (aref data i) new))))))
-               ((typep tree 'instance)
-                ;; We don't grovel the layout.
-                (do-instance-tagged-slot (i tree)
-                  (let* ((old (%instance-ref tree i))
-                         (new (circle-subst circle-table old)))
-                    (unless (eq old new)
-                      (setf (%instance-ref tree i) new)))))
-               ((typep tree 'funcallable-instance)
-                (do ((i 1 (1+ i))
-                     (end (- (1+ (get-closure-length tree)) sb!vm:funcallable-instance-info-offset)))
-                    ((= i end))
-                  (let* ((old (%funcallable-instance-info tree i))
-                         (new (circle-subst circle-table old)))
-                    (unless (eq old new)
-                      (setf (%funcallable-instance-info tree i) new))))))
-         tree)
-        (t tree)))
+;; substitutes in arrays and structures as well as lists.
+(defun circle-subst (tree)
+  (declare (inline alloc-xset))
+  (dx-let ((circle-table (alloc-xset)))
+    (named-let recurse ((tree tree))
+      (when (and (sharp-equal-wrapper-p tree)
+                 (neq (sharp-equal-wrapper-value tree) +sharp-equal-marker+))
+        (return-from recurse (sharp-equal-wrapper-value tree)))
+      ;; pick off types that never need to be sought in or added to the xset.
+      ;; (there are others, but these are common and quick to check)
+      (when (typep tree '(or number symbol))
+        (return-from recurse tree))
+      (unless (xset-member-p tree circle-table)
+        (add-to-xset tree circle-table)
+        (macrolet ((process (place)
+                     `(let* ((old ,place)
+                             (new (recurse old)))
+                        (unless (eq old new)
+                          (setf ,place new)))))
+          (typecase tree
+            (cons
+             (process (car tree))
+             (process (cdr tree)))
+            ((array t)
+             (with-array-data ((data tree) (start) (end))
+               (declare (fixnum start end))
+               (do ((i start (1+ i)))
+                   ((>= i end))
+                 (process (aref data i)))))
+            (instance
+             ;; Don't refer to the DD-SLOTS unless there is reason to,
+             ;; that is, unless some slot might be raw.
+             (if (/= +layout-all-tagged+ (layout-bitmap (%instance-layout tree)))
+                 (let ((dd (layout-info (%instance-layout tree))))
+                   (dolist (dsd (dd-slots dd))
+                     (when (eq (dsd-raw-type dsd) t)
+                       (process (%instance-ref tree (dsd-index dsd))))))
+                 (do ((len (%instance-length tree))
+                      (i sb-vm:instance-data-start (1+ i)))
+                     ((>= i len))
+                   (process (%instance-ref tree i)))))
+             ;; It is not safe to iterate over %FUNCALLABLE-INSTANCE-INFO without knowing
+             ;; where the raw slots are, if any. We can safely process FSC-INSTANCE-SLOTS
+             ;; though, and that's the *only* slot to look at.
+             ;; No other funcallable structure (%METHOD-FUNCTION, INTERPRETED-FUNCTION, etc)
+             ;; has a reader syntax.
+             ;; ASSUMPTION: all funcallable structures have at least 1 slot beyond
+             ;; %FUNCALLABLE-INSTANCE-FUN (overlapping with FSC-INSTANCE-SLOTS)
+             ;; and that slot is never a raw slot.
+            (funcallable-instance
+             (process (sb-pcl::%fsc-instance-slots tree))))))
+      tree)))
 
 ;;; Sharp-equal works as follows.
 ;;; When creating a new label a SHARP-EQUAL-WRAPPER is pushed onto
@@ -316,7 +343,7 @@
                            "Must label something more than just #~D#"
                            label))
     (setf (sharp-equal-wrapper-value wrapper) obj)
-    (circle-subst (make-hash-table :test 'eq) obj)))
+    (circle-subst obj)))
 
 (defun sharp-sharp (stream ignore label)
   (declare (ignore ignore))
@@ -340,6 +367,33 @@
 
 ;;;; conditional compilation: the #+ and #- readmacros
 
+;;; If X is a symbol, see whether it is present in *FEATURES*. Also
+;;; handle arbitrary combinations of atoms using NOT, AND, OR.
+(defun featurep (x)
+  (typecase x
+    (cons
+     (case (car x)
+       ((:not not)
+        (cond
+          ((cddr x)
+           (error "too many subexpressions in feature expression: ~S" x))
+          ((null (cdr x))
+           (error "too few subexpressions in feature expression: ~S" x))
+          (t (not (featurep (cadr x))))))
+       ((:and and) (every #'featurep (cdr x)))
+       ((:or or) (some #'featurep (cdr x)))
+       (t
+        (error "unknown operator in feature expression: ~S." x))))
+    (symbol
+     (let ((present (memq x *features*)))
+       (cond (present
+              t)
+             ((and (boundp '+internal-features+)
+                   (memq x (symbol-value '+internal-features+)))
+              (warn "~s is no longer present in ~s" x '*features*)))))
+    (t
+     (error "invalid feature expression: ~S" x))))
+
 (defun sharp-plus-minus (stream sub-char numarg)
     (ignore-numarg sub-char numarg)
     (if (char= (if (featurep (let ((*package* *keyword-package*)
@@ -380,10 +434,10 @@
         #'(lambda (decoding-error)
             (declare (ignorable decoding-error))
             (style-warn
-             'sb!kernel::character-decoding-error-in-dispatch-macro-char-comment
+             'sb-kernel::character-decoding-error-in-dispatch-macro-char-comment
              :sub-char sub-char :position (file-position stream) :stream stream)
             (invoke-restart 'attempt-resync))))
-    (let ((stream (in-synonym-of stream)))
+    (let ((stream (in-stream-from-designator stream)))
       (macrolet ((munch (get-char &optional finish)
                    `(do ((level 1)
                          (prev ,get-char char)
@@ -451,7 +505,6 @@
                (make-symbol token)))))))
 
 (defvar *read-eval* t
-  #!+sb-doc
   "If false, then the #. read macro is disabled.")
 
 (defun sharp-dot (stream sub-char numarg)
