@@ -87,9 +87,13 @@ distinct from the global value. Can also be SETF."
                    (char= (schar string 2) #\L)))))
       (return-from compute-symbol-hash (sxhash nil)))
   ;; And make a symbol's hash not the same as (sxhash name) in general.
-  (let ((sxhash (logand (lognot (%sxhash-simple-substring string 0 length))
+  (let ((sxhash (logxor (%sxhash-simple-substring string 0 length)
                         sb-xc:most-positive-fixnum)))
-    (if (zerop sxhash) #x55AA sxhash))) ; arbitrary substitute for 0
+    ;; The low 32 bits of the word in memory should have at least a 1 bit somewhere.
+    ;; If not, OR in a constant value.
+    (if (ldb-test (byte (- 32 sb-vm:n-fixnum-tag-bits) 0) sxhash)
+        sxhash
+        (logior sxhash #x55AA)))) ; arbitrary
 
 ;; Return SYMBOL's hash, a strictly positive fixnum, computing it if not stored.
 ;; The inlined code for (SXHASH symbol) only calls ENSURE-SYMBOL-HASH if
@@ -101,10 +105,6 @@ distinct from the global value. Can also be SETF."
           (%set-symbol-hash symbol (compute-symbol-hash name (length name))))
       hash)))
 
-;;; Interpreter stub: Return whatever is in the SYMBOL-HASH slot of SYMBOL.
-(defun symbol-hash (symbol)
-  (symbol-hash symbol))
-
 (defun symbol-function (symbol)
   "Return SYMBOL's current function definition. Settable with SETF."
   (%coerce-name-to-fun symbol symbol-fdefn))
@@ -207,7 +207,7 @@ distinct from the global value. Can also be SETF."
   #+(vop-translates cl:symbol-plist)
   (symbol-plist symbol)
   #-(vop-translates cl:symbol-plist)
-  (let ((list (car (truly-the list (symbol-info symbol))))) ; a white lie
+  (let ((list (car (truly-the list (symbol-info symbol))))) ; a harmless lie
     ;; Just ensure the result is not a fixnum, and we're done.
     (if (fixnump list) nil list)))
 
@@ -476,7 +476,7 @@ distinct from the global value. Can also be SETF."
                            (code-char (+ (char-code #\0) r)))
                      s)))
           (recurse 1 counter)))
-      (with-simple-output-to-string (s)
+      (%with-output-to-string (s)
         (write-string prefix s)
         (%output-integer-in-base counter 10 s)))))
 
