@@ -34,12 +34,12 @@
      :all))
   0)
 
+;;; Given ADDR which is a pointer to a 'struct thread'
+;;; return the corresponding lisp thread.
 (defun find-lisp-thread-from-thread-struct (addr)
-  ;; It is of course possible to do this without consing the list
-  ;; of all threads, but I don't care.
-  (dolist (thread (sb-thread:list-all-threads))
-    (when (= (sb-thread::thread-primitive-thread thread) addr)
-      (return thread))))
+  (declare (type sb-ext:word addr))
+  (sap-ref-lispobj (int-sap addr)
+                   (ash sb-vm::thread-lisp-thread-slot sb-vm:word-shift)))
 
 ;;; Convert each path to (TARGET . NODES)
 ;;; where the first node in NODES is one of:
