@@ -217,7 +217,7 @@
   (let ((kind (fun-subtype fun)))
     (or (and (eql kind sb-vm:funcallable-instance-widetag)
              ;; if the FIN has no raw words then it has no internal trampoline
-             (eql (layout-bitmap (%funcallable-instance-layout fun))
+             (eql (layout-bitmap (%fun-layout fun))
                   sb-kernel:+layout-all-tagged+))
         (eql kind sb-vm:closure-widetag))))
 
@@ -266,7 +266,7 @@
     ;; Assert that raw bytes will not cause GC invariant lossage
     (aver (/= (layout-bitmap layout) +layout-all-tagged+))
     ;; Set layout prior to writing raw slots
-    (setf (%funcallable-instance-layout gf) layout)
+    (setf (%fun-layout gf) layout)
     ;; just being pedantic - liveness is preserved by the stack reference.
     (with-pinned-objects (gf)
       (let* ((addr (logandc2 (get-lisp-obj-address gf) lowtag-mask))
@@ -388,6 +388,7 @@
   (declare (ignorable code fixups))
   (unless (immobile-space-obj-p code)
     (return-from statically-link-code-obj code))
+  #+immobile-code
   (let* ((fdefns-start (+ code-constants-offset
                           (* code-slots-per-simple-fun (code-n-entries code))))
          (fdefns-count (code-n-named-calls code))
@@ -400,7 +401,7 @@
     ;;   that function is in immobile space and needs no trampoline.
     ;; * whether the replacement creates ambiguitity - if #'F and #'G are the same
     ;;   function, then substituting that function in for the fdefn of F and G
-    ;;   requires storing locations at which to replacement was done
+    ;;   requires storing locations at which replacement was done
     (dotimes (i fdefns-count)
       (let* ((fdefn (code-header-ref code (+ fdefns-start i)))
              (fun (when (fdefn-p fdefn) (fdefn-fun fdefn))))
