@@ -192,6 +192,10 @@
     "COERCE" "EXP" "EXPT" "LOG" "SIGNUM" "IMAGPART" "REALPART"
     "ZEROP" "ABS" "SIGNUM" "FLOAT-SIGN"
     "CEILING" "FLOOR" "ROUND" "TRUNCATE" "MOD" "REM"
+    ;; We always want irrational functions to use target floats.
+    "ACOS" "ACOSH" "ASIN" "ASINH" "ATAN" "ATANH"  "CIS" "CONJUGATE"
+    "COS" "COSH"  "FCEILING" "FFLOOR" "FROUND" "FTRUNCATE"
+    "PHASE" "RATIONALIZE" "SIN" "SINH" "SQRT" "TAN" "TANH"
     ;;
     "SXHASH" ; must package-qualify if you mean CL:SXHASH
     ;;
@@ -233,11 +237,6 @@
 ;;; see by default, so that using them by accident fails.
 (defparameter *undefineds*
   '("SYMBOL-PACKAGE"
-    ;; Irrational functions are never used in the cross-compiler (yet).
-    ;; Prove that by making them undefined.
-    "ACOS" "ACOSH" "ASIN" "ASINH" "ATAN" "ATANH"  "CIS" "CONJUGATE"
-    "COS" "COSH"  "FCEILING" "FFLOOR" "FROUND" "FTRUNCATE"
-    "PHASE" "RATIONALIZE" "SIN" "SINH" "SQRT" "TAN" "TANH"
     ;; Float decoding: don't want to see these used either.
     "DECODE-FLOAT" "INTEGER-DECODE-FLOAT"
     "FLOAT-DIGITS" "FLOAT-PRECISION" "FLOAT-RADIX"
@@ -470,8 +469,8 @@
           (reexport x))
         (assert (= (length done) (length package-data-list)))))))
 
-(export '*undefined-fun-whitelist*)
-(defvar *undefined-fun-whitelist* (make-hash-table :test 'equal))
+(export '*undefined-fun-allowlist*)
+(defvar *undefined-fun-allowlist* (make-hash-table :test 'equal))
 (let ((list
        (with-open-file (data (prepend-genfile-path "package-data-list.lisp-expr"))
          ;; There's no need to use the precautionary READ-FROM-FILE function
@@ -479,7 +478,7 @@
          (create-target-packages (let ((*readtable* *xc-readtable*)) (read data)))
          (let ((*readtable* *xc-readtable*)) (read data)))))
   (dolist (name (apply #'append list))
-    (setf (gethash name *undefined-fun-whitelist*) t)))
+    (setf (gethash name *undefined-fun-allowlist*) t)))
 
 (defvar *asm-package-use-list*
   '("SB-ASSEM" "SB-DISASSEM"
