@@ -33,8 +33,11 @@
   ;; whatever it is that show would have shown. Comment this out if you need.
     (when s (set s nil))))
 
-(assert (zerop (deref (extern-alien "lowtag_for_widetag" (array char 64))
-                      (ash sb-vm:character-widetag -2))))
+(let ((byte (deref (extern-alien "widetag_lowtag" (array char 256))
+                   sb-vm:character-widetag)))
+  (assert (not (logbitp 7 byte))) ; not a headered object
+  (assert (= (logand byte sb-vm:lowtag-mask) sb-vm:list-pointer-lowtag)))
+(gc :full t)
 
 ;;; Verify that all defstructs except for one were compiled in a null lexical
 ;;; environment. Compiling any call to a structure constructor would like to
@@ -64,16 +67,29 @@
       ;; Ensure that we're reading the correct variant of the file
       ;; in case there is more than one set of floating-point formats.
       (assert (eq (read stream) :default))
-      (let ((*package* (find-package "SB-KERNEL")))
-        (dolist (expr (read stream))
-          (destructuring-bind (fun args result) expr
-            (let ((actual (apply fun (sb-int:ensure-list args))))
-              (unless (eql actual result)
-                (#+sb-devel cerror #+sb-devel ""
-                 #-sb-devel format #-sb-devel t
-                 "FLOAT CACHE LINE ~S vs COMPUTED ~S~%"
-                 expr actual)))))))))
+      (sb-kernel::with-float-traps-masked (:overflow :divide-by-zero)
+        (let ((*package* (find-package "SB-KERNEL")))
+          (dolist (expr (read stream))
+            (destructuring-bind (fun args . result) expr
+              (let ((result (if (eq (first result) 'sb-kernel::&values)
+                                (rest result)
+                                result))
+                    (actual (multiple-value-list (apply fun (sb-int:ensure-list args)))))
+                (unless (equalp actual result)
+                  (#+sb-devel cerror #+sb-devel ""
+                   #-sb-devel format #-sb-devel t
+                   "FLOAT CACHE LINE ~S vs COMPUTED ~S~%"
+                   expr actual))))))))))
 
+(when (if (boundp '*compile-files-p*) *compile-files-p* t)
+  (with-open-file (output "output/cold-vop-usage.txt" :if-does-not-exist nil)
+    (when output
+      (setq sb-c::*static-vop-usage-counts* (make-hash-table))
+      (loop (let ((line (read-line output nil)))
+              (unless line (return))
+              (let ((count (read-from-string line))
+                    (name (read-from-string line t nil :start 8)))
+                (setf (gethash name sb-c::*static-vop-usage-counts*) count)))))))
 
 ;;;; compiling and loading more of the system
 
@@ -129,11 +145,21 @@
               retry-compile-file
                 (multiple-value-bind (output-truename warnings-p failure-p)
                     (ecase (if (boundp '*compile-files-p*) *compile-files-p* t)
-                     ((t)   (let ((sb-c::*source-namestring* fullname))
-                              (ensure-directories-exist output)
-                              (compile-file stem :output-file output)))
+                     ((t)
+                      (let ((sb-c::*source-namestring* fullname)
+                            (sb-ext:*derive-function-types*
+                              (unless (search "/pcl/" stem)
+                                t)))
+                        (ensure-directories-exist output)
+                        ;; Like PROCLAIM-TARGET-OPTIMIZATION in 'compile-cold-sbcl'
+                        ;; We should probably stash a copy of the POLICY instance from
+                        ;; make-host-2 in a global var and apply it here.
+                        (proclaim '(optimize
+                                    (safety 2) (speed 2)
+                                    (sb-c:insert-step-conditions 0)
+                                    (sb-c:alien-funcall-saves-fp-and-pc #+x86 3 #-x86 0)))
+                        (compile-file stem :output-file output)))
                      ((nil) output))
-                  (declare (ignore warnings-p))
                   (cond ((not output-truename)
                          (error "COMPILE-FILE of ~S failed." stem))
                         (failure-p
@@ -151,8 +177,12 @@
                            (when (and failure-p (probe-file output-truename))
                                  (delete-file output-truename)
                                  (format t "~&deleted ~S~%" output-truename))))
-                        ;; Otherwise: success, just fall through.
-                        (t nil))
+                        (warnings-p
+                         ;; Maybe we should escalate more warnings to errors
+                         ;; (see HANDLER-BIND for SIMPLE-WARNING below)
+                         ;; rather than asking what to do here?
+                         #+(or x86 x86-64) ;; these should complete without warnings
+                         (cerror "Ignore warnings" "Compile completed with warnings")))
                   (unless (handler-bind
                               ((sb-kernel:redefinition-with-defgeneric
                                 #'muffle-warning))
@@ -163,11 +193,22 @@
 
   (let ((*compile-print* nil))
     (dolist (group sources)
-      (handler-bind ((simple-warning
+      (handler-bind ((#+x86-64 warning #-x86-64 simple-warning
                       (lambda (c)
                         ;; escalate "undefined variable" warnings to errors.
                         ;; There's no reason to allow them in our code.
-                        (when (search "undefined variable"
-                                      (write-to-string c :escape nil))
+                        (when (and #-x86-64 ; Don't allow any warnings on x86-64.
+                                   (search "undefined variable"
+                                           (write-to-string c :escape nil)))
                           (cerror "Finish warm compile ignoring the problem" c)))))
         (with-compilation-unit () (do-srcs group)))))))
+
+(when (hash-table-p sb-c::*static-vop-usage-counts*)
+  (with-open-file (output "output/warm-vop-usage.txt"
+                          :direction :output :if-exists :supersede)
+    (let (list)
+      (sb-int:dohash ((name vop) sb-c::*backend-parsed-vops*)
+        (declare (ignore vop))
+        (push (cons (gethash name sb-c::*static-vop-usage-counts* 0) name) list))
+      (dolist (cell (sort list #'> :key #'car))
+        (format output "~7d ~s~%" (car cell) (cdr cell))))))
