@@ -346,11 +346,11 @@
                           node))
 
 (defoptimizer (make-array-header* derive-type) ((&rest inits))
-  (let* ((data-position #.(sb-vm::slot-offset
+  (let* ((data-position #.(sb-vm:slot-offset
                            (find 'sb-vm::data (sb-vm:primitive-object-slots
                                                (find 'array sb-vm:*primitive-objects*
                                                      :key 'sb-vm:primitive-object-name))
-                                 :key 'sb-vm::slot-name)))
+                                 :key 'sb-vm:slot-name)))
          (data (nth data-position inits))
          (type (lvar-type data)))
     (when (array-type-p type)
@@ -874,9 +874,10 @@
                   ;; didn't realize this.
                   (cond
                     (initial-element
-                     (compiler-warn "~S ~S is not a ~S"
-                                    :initial-element default-initial-element
-                                    elt-spec))
+                     (compiler-warn 'array-initial-element-mismatch
+                                    :format-control "~S ~S is not a ~S"
+                                    :format-arguments
+                                    (list :initial-element default-initial-element elt-spec)))
                     ;; For the default initial element, only warn if
                     ;; any array elements are initialized using it.
                     ((and (not (eql c-length 0))
@@ -884,9 +885,9 @@
                           ;; then fill-array means it was supplied initial-contents
                           (not (lvar-matches-calls (combination-lvar call)
                                                    '(make-array-header* fill-array))))
-                     (compiler-style-warn "The default initial element ~S is not a ~S."
-                                          default-initial-element
-                                          elt-spec))))
+                     (compiler-style-warn 'initial-element-mismatch-style-warning
+                                          :format-control "The default initial element ~S is not a ~S."
+                                          :format-arguments (list default-initial-element elt-spec)))))
              (let ((lambda-list `(length ,@(eliminate-keywords))))
                `(lambda ,lambda-list
                   (declare (ignorable ,@lambda-list))
@@ -905,12 +906,22 @@
                           :node node)
   (delay-ir1-transform node :constraint)
   (when (and initial-contents initial-element)
-    (compiler-warn "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS")
-    (give-up-ir1-transform))
+    (abort-ir1-transform "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS"))
   (when (and displaced-index-offset
              (not displaced-to))
-    (compiler-warn "Can't specify :DISPLACED-INDEX-OFFSET without :DISPLACED-TO")
-    (give-up-ir1-transform))
+    (abort-ir1-transform "Can't specify :DISPLACED-INDEX-OFFSET without :DISPLACED-TO"))
+  (when (and displaced-to
+             (or initial-element initial-contents))
+    (if (types-equal-or-intersect (lvar-type displaced-to)
+                                  (specifier-type 'null))
+        (give-up-ir1-transform ":DISPLACED-TO potentially used with ~S"
+                               (if initial-element
+                                   :initial-element
+                                   :initial-contents))
+        (abort-ir1-transform "Can't specify :DISPLACED-TO and ~S"
+                             (if initial-element
+                                 :initial-element
+                                 :initial-contents))))
   (let ((fp-type (and fill-pointer
                       (lvar-type fill-pointer)) ))
     (when (and fp-type
@@ -985,8 +996,9 @@
                    ((not (ctypep value eltype-type))
                     ;; this case will not cause an error at runtime, but
                     ;; it's still worth STYLE-WARNing about.
-                    (compiler-style-warn "~S is not a ~S."
-                                         value eltype)))))
+                    (compiler-style-warn 'initial-element-mismatch-style-warning
+                                         :format-control "~S is not a ~S."
+                                         :format-arguments (list value eltype))))))
              `(let ((array ,creation-form))
                 (multiple-value-bind (vector)
                     (%data-vector-and-index array 0)
@@ -1416,15 +1428,18 @@
              (ref-p array-ref)
              (ref-p index-ref)
              (or
-              (let ((index-leaf (ref-leaf index-ref)))
+              (let* ((index-leaf (ref-leaf index-ref))
+                     (index-value (and (constant-p index-leaf)
+                                       (constant-value index-leaf)))
+                     (index-value (and (integerp index-value)
+                                       index-value)))
                 (loop for constraint in (ref-constraints array-ref)
                       for y = (constraint-y constraint)
                       thereis (and
                                (eq (constraint-kind constraint) 'array-in-bounds-p)
-                               (if (constant-p index-leaf)
+                               (if index-value
                                    (and (constant-p y)
-                                        (<= (constant-value index-leaf)
-                                            (constant-value y)))
+                                        (<= index-value (constant-value y)))
                                    (eq index-leaf y)))))
               (loop for constraint in (ref-constraints index-ref)
                     thereis (and (eq (constraint-kind constraint) 'array-in-bounds-p)
