@@ -41,7 +41,7 @@
 ;;; is not seen by the "#." expression a few lines down.
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defconstant info-number-bits 6))
-(def!type info-number () `(unsigned-byte ,info-number-bits))
+(deftype info-number () `(unsigned-byte ,info-number-bits))
 
 ;;; A map from info-number to its META-INFO object.
 ;;; The reverse mapping is obtained by reading the META-INFO.
@@ -81,7 +81,7 @@
 (defconstant +info-metainfo-type-num+ 0)
 
 ;; Refer to info-vector.lisp for the meaning of this constant.
-(defconstant +no-auxilliary-key+ 0)
+(defconstant +no-auxiliary-key+ 0)
 
 ;;; Return the globaldb info for SYMBOL. With respect to the state diagram
 ;;; presented at the definition of SYMBOL-PLIST, if the object in SYMBOL's
@@ -122,7 +122,7 @@
 (defmacro !get-meta-infos (kind)
   `(let* ((info-vector (symbol-info-vector ,kind))
           (index (if info-vector
-                     (packed-info-value-index info-vector +no-auxilliary-key+
+                     (packed-info-value-index info-vector +no-auxiliary-key+
                                               +info-metainfo-type-num+))))
      (if index (svref info-vector index))))
 
@@ -266,7 +266,7 @@
 ;;;; operations, we represent the attributes as bits in a fixnum.
 
 (in-package "SB-C")
-(def!type attributes () 'fixnum)
+(deftype attributes () 'fixnum)
 
 ;;; Given a list of attribute names and an alist that translates them
 ;;; to masks, return the OR of the masks.
