@@ -562,7 +562,7 @@
 (defknown (every notany notevery) (function-designator proper-sequence &rest proper-sequence) boolean
   (foldable unsafely-flushable call))
 
-(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key)
+(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key :value :initial-value)
                                         (nth-arg 1 :sequence t :key :key)))
                   proper-sequence &rest t &key (:from-end t)
                   (:start (inhibit-flushing index 0))
@@ -2089,7 +2089,19 @@
     (t (function-designator () * :no-function-conversion t) &key (:dont-save t))
     *)
 
+(defknown (sb-impl::%with-standard-io-syntax
+           sb-impl::%with-rebound-io-syntax
+           sb-impl::call-with-sane-io-syntax)
+    (function) *)
+(defknown sb-debug::funcall-with-debug-io-syntax (function &rest t) *)
+(defknown sb-impl::%print-unreadable-object (t t t &optional function) null)
+
 #+sb-thread
-(defknown sb-thread::call-with-recursive-lock (function t t t) *)
-#+sb-thread
-(defknown sb-thread::call-with-mutex (function t t t t) *)
+(progn
+(defknown (sb-thread::call-with-mutex sb-thread::call-with-recursive-lock)
+    (function t t t) *)
+(defknown (sb-thread::call-with-system-mutex
+           sb-thread::call-with-system-mutex/allow-with-interrupts
+           sb-thread::call-with-system-mutex/without-gcing
+           sb-thread::call-with-recursive-system-lock)
+    (function t) *))
