@@ -152,14 +152,14 @@
     sb-unix::signal-handler-callback)
   #'equal)
 
-;;; (potentially) static symbols that C code must be able to assign to,
+;;; (potentially) static symbols that C code must be able to set/get
 ;;; as contrasted with static for other reasons such as:
 ;;;  - garbage collections roots (namely NIL)
 ;;;  - other symbols that Lisp codegen must hardwire (T)
 ;;;  - static for efficiency of access but need not be
 ;;; On #+sb-thread builds, these are not static, because access to them
 ;;; is via the TLS, not the symbol.
-(defconstant-eqx !per-thread-c-interface-symbols
+(defconstant-eqx per-thread-c-interface-symbols
   `((*free-interrupt-context-index* 0)
     (sb-sys:*allow-with-interrupts* t)
     (sb-sys:*interrupts-enabled* t)
@@ -184,12 +184,13 @@
     ;; These symbols are accessed from C only through TLS,
     ;; never the symbol-value slot
     #-sb-thread ,@(mapcar (lambda (x) (car (ensure-list x)))
-                           !per-thread-c-interface-symbols)
-    ;; NLX variables are thread slots on x86-64.  A static sym is needed
+                           per-thread-c-interface-symbols)
+    ;; NLX variables are thread slots on x86-64 and RISC-V.  A static sym is needed
     ;; for arm64, ppc, and x86 because we haven't implemented TLS index fixups,
     ;; so must lookup the TLS index given the symbol.
-    #+(and sb-thread (not x86-64)) ,@'(*current-catch-block*
-                                        *current-unwind-protect-block*)
+    #+(and sb-thread (not x86-64) (not riscv))
+    ,@'(*current-catch-block*
+        *current-unwind-protect-block*)
 
     ;; sb-safepoint in addition to accessing this symbol via TLS,
     ;; uses the symbol itself as a value. Kinda weird.
@@ -207,10 +208,7 @@
     #-sb-thread *stepping*
 
     ;; threading support
-    #+sb-thread *free-tls-index*
-    ;; Keep in sync with 'code/target-thread.lisp':
-    ;;  "only PPC uses a separate symbol for the TLS index lock"
-    #+(and sb-thread (or ppc ppc64)) *tls-index-lock*
+    #+sb-thread ,@'(sb-thread::*starting-threads* *free-tls-index*)
 
     ;; dynamic runtime linking support
     #+linkage-table +required-foreign-symbols+
