@@ -111,6 +111,11 @@
     ;; If ANSWER is NIL, go for the global value
     (eq (or answer (info :function :inlinep name)) 'notinline)))
 
+#-sb-devel
+(declaim (start-block find-free-fun find-lexically-apparent-fun
+                      ;; needed by with-fun-name-leaf
+                      find-global-fun))
+
 (defun maybe-defined-here (name where)
   (if (and (eq :defined where)
            (boundp '*compilation*)
@@ -254,6 +259,8 @@
           (t
            (find-free-fun name context)))))
 
+(declaim (end-block))
+
 (defun maybe-find-free-var (name)
   (let ((found (gethash name (free-vars *ir1-namespace*))))
     (unless (eq found :deprecated)
@@ -545,7 +552,6 @@
          (setq trail (cdr trail)))))))
 
 ;;;; IR1-CONVERT, macroexpansion and special form dispatching
-
 (declaim (ftype (sfunction (ctran ctran (or lvar null) t)
                            (values))
                 ir1-convert))
@@ -629,7 +635,11 @@
   (assure-leaf-live-p leaf)
   (let* ((type (lexenv-find leaf type-restrictions))
          (leaf (or (and (defined-fun-p leaf)
-                        (not (eq (defined-fun-inlinep leaf) 'notinline))
+                        (neq (defined-fun-inlinep leaf) 'notinline)
+                        ;; Only reference defined functionals from named-lambda,
+                        ;; everything else should do this through recognize-known-call,
+                        ;; avoiding copying references to global functions
+                        (defined-fun-named-lambda-p leaf)
                         (let ((functional (defined-fun-functional leaf)))
                           (when (and functional (not (functional-kind functional)))
                             (maybe-reanalyze-functional functional))))
@@ -639,6 +649,11 @@
                      (maybe-reanalyze-functional leaf))
                    leaf))
          (ref (make-ref leaf name)))
+    (when (and result
+               (lambda-var-p leaf)
+               (lambda-var-constant leaf))
+      (push (make-lvar-lambda-var-annotation :lambda-var leaf)
+            (lvar-annotations result)))
     (push ref (leaf-refs leaf))
     (when (and (functional-p leaf)
                (functional-ignore leaf))
@@ -658,13 +673,6 @@
                   (use-continuation cast next result)))
           (t (use-continuation ref next result)))))
 
-(defun always-boundp (name)
-  (case (info :variable :always-bound name)
-    (:always-bound t)
-    ;; Compiling to fasl considers a symbol always-bound if its
-    ;; :always-bound info value is now T or will eventually be T.
-    (:eventually (producing-fasl-file))))
-
 ;;; Convert a reference to a symbolic constant or variable. If the
 ;;; symbol is entered in the LEXENV-VARS we use that definition,
 ;;; otherwise we find the current global definition. This is also
@@ -1154,6 +1162,12 @@
 
 ;;;; PROCESS-DECLS
 
+#-sb-devel
+(declaim (start-block process-decls make-new-inlinep
+                      find-in-bindings
+                      process-muffle-decls
+                      %processing-decls))
+
 ;;; Given a list of LAMBDA-VARs and a variable name, return the
 ;;; LAMBDA-VAR for that name, or NIL if it isn't found. We return the
 ;;; *last* variable with that name, since LET* bindings may be
@@ -1341,6 +1355,24 @@
           (t
            res))))
 
+(defun process-no-constraints-decl (spec vars)
+  (dolist (name (rest spec))
+    (let ((var (find-in-bindings vars name)))
+      (cond
+        ((not var)
+         (warn "No ~s variable" name))
+        (t
+         (setf (lambda-var-no-constraints var) t))))))
+
+(defun process-constant-decl (spec vars)
+  (dolist (name (rest spec))
+    (let ((var (find-in-bindings vars name)))
+      (cond
+        ((not var)
+         (warn "No ~s variable" name))
+        (t
+         (setf (lambda-var-constant var) t))))))
+
 ;;; Return a DEFINED-FUN which copies a GLOBAL-VAR but for its INLINEP
 ;;; (and TYPE if notinline), plus type-restrictions from the lexenv.
 (defun make-new-inlinep (var inlinep local-type)
@@ -1515,12 +1547,7 @@
                (etypecase fun
                  (leaf
                   (if bound-fun
-                      #+stack-allocatable-closures
                       (setf (leaf-extent bound-fun) extent)
-                      #-stack-allocatable-closures
-                      (compiler-notify
-                       "Ignoring DYNAMIC-EXTENT declaration on function ~S ~
-                        (not supported on this platform)." fname)
                       (compiler-notify
                        "Ignoring free DYNAMIC-EXTENT declaration: ~S" name)))
                  (cons
@@ -1596,6 +1623,12 @@
         (make-lexenv
          :default res
          :flushable (cdr spec)))
+       (no-constraints
+        (process-no-constraints-decl spec vars)
+        res)
+       (constant-value
+        (process-constant-decl spec vars)
+        res)
        ;; We may want to detect LAMBDA-LIST and VALUES decls here,
        ;; and report them as "Misplaced" rather than "Unrecognized".
        (t
@@ -1607,6 +1640,17 @@
               res))))
      optimize-qualities)))
 
+(defun process-muffle-decls (decls lexenv)
+  (flet ((process-it (spec)
+           (cond ((atom spec))
+                 ((member (car spec) '(muffle-conditions unmuffle-conditions))
+                  (setq lexenv
+                        (process-1-decl spec lexenv nil nil nil nil))))))
+    (dolist (decl decls)
+      (dolist (spec (rest decl))
+        (process-it spec))))
+  lexenv)
+
 ;;; Use a list of DECLARE forms to annotate the lists of LAMBDA-VAR
 ;;; and FUNCTIONAL structures which are being bound. In addition to
 ;;; filling in slots in the leaf structures, we return a new LEXENV,
@@ -1688,17 +1732,6 @@
     (values lexenv result-type (cdr post-binding-lexenv)
             lambda-list explicit-check source-form)))
 
-(defun process-muffle-decls (decls lexenv)
-  (flet ((process-it (spec)
-           (cond ((atom spec))
-                 ((member (car spec) '(muffle-conditions unmuffle-conditions))
-                  (setq lexenv
-                        (process-1-decl spec lexenv nil nil nil nil))))))
-    (dolist (decl decls)
-      (dolist (spec (rest decl))
-        (process-it spec))))
-  lexenv)
-
 (defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
   (multiple-value-bind (*lexenv* result-type post-binding-lexenv)
       (process-decls decls vars fvars :binding-form-p binding-form-p)
@@ -1747,3 +1780,5 @@
          (make-global-var :kind :special
                           :%source-name name
                           :where-from :declared))))
+
+(declaim (end-block))
