@@ -58,8 +58,8 @@
 
 ;;; Linear VARiable. Multiple-value (possibly of unknown number)
 ;;; temporal storage.
-(def!struct (lvar (:constructor make-lvar (&optional dest))
-                  (:copier nil))
+(defstruct (lvar (:constructor make-lvar (&optional dest))
+                 (:copier nil))
   ;; The node which receives this value. NIL only temporarily.
   (dest nil :type (or node null))
   ;; cached type of this lvar's value. If NIL, then this must be
@@ -138,6 +138,11 @@
             (:include lvar-type-annotation)
             (:copier nil)))
 
+(defstruct (lvar-lambda-var-annotation
+            (:include lvar-annotation)
+            (:copier nil))
+  lambda-var)
+
 (defmethod print-object ((x lvar) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (when (boundp '*compilation*)
@@ -411,13 +416,6 @@
 ;;;   size of flow analysis problems, this allows back-end data
 ;;;   structures to be reclaimed after the compilation of each
 ;;;   component.
-(locally
-  ;; This is really taking the low road. I couldn't think of a way to
-  ;; avoid a style warning regarding IR2-COMPONENT other than to declare
-  ;; the INFO slot as :type (or (satisfies ir2-component-p) ...)
-  ;; During make-host-2, the solution to this is the same hack
-  ;; as for everything else: use DEF!STRUCT for IR2-COMPONENT.
-  #+host-quirks-sbcl (declare (host-sb-ext:muffle-conditions style-warning))
 (defstruct (component (:copier nil)
                       (:constructor make-component
                        (head
@@ -501,13 +499,8 @@
   (reanalyze nil :type boolean)
   ;; some sort of name for the code in this component
   (name "<unknown>" :type t)
-  ;; When I am a child, this is :NO-IR2-YET.
-  ;; In my adulthood, IR2 stores notes to itself here.
-  ;; After I have left the great wheel and am staring into the GC, this
-  ;;   is set to :DEAD to indicate that it's a gruesome error to operate
-  ;;   on me (e.g. by using me as *CURRENT-COMPONENT*, or by pushing
-  ;;   LAMBDAs onto my NEW-FUNCTIONALS, as in sbcl-0.pre7.115).
-  (info :no-ir2-yet :type (or ir2-component (member :no-ir2-yet :dead)))
+  ;; some kind of info used by the back end.
+  (info nil)
   ;; a map from combination nodes to things describing how an
   ;; optimization of the node failed. The description is an alist
   ;; (TRANSFORM . ARGS), where TRANSFORM is the structure describing
@@ -527,7 +520,7 @@
   ;; The default LOOP in the component.
   (outer-loop (missing-arg) :type cloop)
   ;; The current sset index
-  (sset-number 0 :type fixnum)))
+  (sset-number 0 :type fixnum))
 (defprinter (component :identity t)
   name
   (reanalyze :test reanalyze))
@@ -665,14 +658,10 @@
 ;;; non-local exits. This is effectively an annotation on the
 ;;; continuation, although it is accessed by searching in the
 ;;; PHYSENV-NLX-INFO.
-(def!struct (nlx-info
-             (:copier nil)
-             (:constructor make-nlx-info (cleanup
-                                          exit
-                                          &aux
-                                          (block
-                                           (first (block-succ
-                                                   (node-block exit)))))))
+(defstruct (nlx-info
+            (:copier nil)
+            (:constructor make-nlx-info
+                (cleanup exit &aux (block (first (block-succ (node-block exit)))))))
   ;; the cleanup associated with this exit. In a catch or
   ;; unwind-protect, this is the :CATCH or :UNWIND-PROTECT cleanup,
   ;; and not the cleanup for the escape block. The CLEANUP-KIND of
@@ -781,8 +770,9 @@
 ;;; Since the same constant leaf may be shared between named and anonymous
 ;;; constants, %SOURCE-NAME is never used.
 (defstruct (constant (:constructor make-constant (value
-                                                  &aux
+                                                  &optional
                                                   (type (ctype-of value))
+                                                  &aux
                                                   (%source-name '.anonymous.)
                                                   (where-from :defined)))
                      (:copier nil)
@@ -850,7 +840,8 @@
   ;; conversion of a NAMED-LAMBDA, or from inline-expansion (see
   ;; RECOGNIZE-KNOWN-CALL) - we need separate functionals for each policy in
   ;; which the function is used.
-  (functionals nil :type list))
+  (functionals nil :type list)
+  (named-lambda-p nil))
 (defprinter (defined-fun :identity t
              :pretty-ir-printer (pretty-print-global-var structure stream))
   %source-name
@@ -1299,7 +1290,10 @@
   ;; that refer to it are not dynamic-extent.  Note that both
   ;; attributes must be set for the value-cell object to be created.
   explicit-value-cell
-  )
+  ;; Do not propagate constraints for this var
+  no-constraints
+  ;; Does it hold a constant that should't be destructively modified
+  constant)
 
 (defstruct (lambda-var (:include basic-var) (:copier nil))
   (flags (lambda-var-attributes)
@@ -1352,6 +1346,11 @@
   `(lambda-var-attributep (lambda-var-flags ,var) deleted))
 (defmacro lambda-var-explicit-value-cell (var)
   `(lambda-var-attributep (lambda-var-flags ,var) explicit-value-cell))
+(defmacro lambda-var-no-constraints (var)
+  `(lambda-var-attributep (lambda-var-flags ,var) no-constraints))
+(defmacro lambda-var-constant (var)
+  `(lambda-var-attributep (lambda-var-flags ,var) constant))
+
 
 ;;;; basic node types
 
