@@ -275,9 +275,11 @@
 
       #-sb-xc-host ;; Supress &OPTIONAL + &KEY syle-warning on xc host
       (when (and (logtest (bits &key) seen) optional (not silent))
-        ;; FIXME: add a condition class for this
         (style-warn-once
-         list "&OPTIONAL and &KEY found in the same lambda list: ~S" list))
+         list
+         (make-condition '&optional-and-&key-in-lambda-list
+                         :format-control "&OPTIONAL and &KEY found in the same lambda list: ~S"
+                         :format-arguments (list list))))
 
       ;; For CONTEXT other than :VALUES-TYPE/:FUNCTION-TYPE we reject
       ;; illegal list elements. Type specifiers have arbitrary shapes,
@@ -545,7 +547,7 @@
                       (let ((def (cdr spec)))
                         (when (and def (not remove-defaults))
                           (list (car def))))) ; Remove any supplied-p var.
-                    (recurse (x) (unparse-ds-lambda-list x :cache cache))
+                    (recurse (x) (unparse-ds-lambda-list x :cache cache :remove-defaults remove-defaults))
                     (memoize (input output)
                       (when cache (push (cons input output) (cdr cache)))
                       output))
@@ -773,7 +775,7 @@
            (let* ((suppliedp (car sup-p-var)) ; could be nil
                   (vals (gen-test sense sup-p-form
                                   (if sup-p-var `(values ,val-form t) val-form)
-                                  def)))
+                                  (if sup-p-var `(values ,def nil) def))))
              (cond ((not sup-p-var) (bind-pat var vals))
                    ((not (symbolp var))
                     (let ((var-temp (sb-xc:gensym))
@@ -1127,6 +1129,11 @@
                       (when whole `((,(car whole) ,ll-whole)))))
              ;; Drop &WHOLE and &ENVIRONMENT
              (new-ll (make-lambda-list llks nil req opt rest keys aux))
+             #-sb-xc-host
+             (*lexenv* (process-muffle-decls decls
+                                             (if (boundp '*lexenv*)
+                                                 *lexenv*
+                                                 (make-null-lexenv))))
              (parse (parse-ds-lambda-list new-ll))
              ((declared-lambda-list decls)
               (let ((ll
@@ -1144,7 +1151,8 @@
                      (loop for (declare . declarations) in decls
                            collect (list* declare
                                           (remove 'lambda-list declarations :key #'car)))
-                     decls)))))
+                     decls))))
+             (variables (ds-lambda-list-variables parse nil)))
     ;; Signal a style warning for duplicate names, but disregard &AUX variables
     ;; because most folks agree that (LET* ((X (F)) (X (G X))) ..) makes sense
     ;; - some would even say that it is idiomatic - and &AUX bindings are just
@@ -1156,7 +1164,7 @@
             (when (memq (car tail) (cdr tail))
               (style-warn-once lambda-list "variable ~S occurs more than once"
                                (car tail))))
-          (append whole env (ds-lambda-list-variables parse nil)))
+          (append whole env variables))
     ;; Maybe kill docstring, but only under the cross-compiler.
     #+(and (not sb-doc) sb-xc-host) (setq docstring nil)
     ;; Note that we *NEVER* declare macro lambdas as a toplevel named lambda.
@@ -1183,7 +1191,9 @@
                                           `(:special-form . ,name)
                                           `(:macro ,name . ,kind)))
                      '(destructuring-bind))
-                  ,new-ll (,accessor ,ll-whole)
+               ,new-ll (,accessor ,ll-whole)
+               #-sb-xc-host
+               (declare (constant-value ,@variables))
                ,@decls
                ,@(if wrap-block
                      `((block ,(fun-name-block-name name) ,@forms))
@@ -1217,3 +1227,52 @@
          ;; It is harmful to the space-saving effect of this function
          ;; if reconstituting the list results in an unnecessary copy.
          (if (equal new lambda-list) lambda-list new))))))
+
+(declaim (ftype (sfunction * function) make-repeated-name-check))
+(defun make-repeated-name-check (&key
+                                   (kind "variable")
+                                   (context "lambda list")
+                                   (signal-via #'compiler-error))
+  (let ((seen '()))
+    (lambda (name)
+      (when (member name seen :test #'eq)
+        (funcall signal-via "~@<The ~A ~S occurs more than once in ~
+                             the ~A.~@:>"
+                 kind name context))
+      (push name seen)
+      name)))
+
+;;; Verify that NAME is a legal name for a variable.
+(declaim (ftype (function (t &key
+                             (:context t) (:allow-special t) (:allow-symbol-macro t)
+                             (:signal-via (or symbol function)))
+                          (values symbol keyword))
+                check-variable-name-for-binding))
+(defun check-variable-name-for-binding (name
+                                        &key
+                                          context
+                                          (allow-special t)
+                                          (allow-symbol-macro t)
+                                          (signal-via #'compiler-error))
+  (check-variable-name name :signal-via signal-via)
+  (flet ((lose (kind)
+           (funcall signal-via
+                    (sb-format:tokens "~@<~/sb-ext:print-symbol-with-prefix/ names a ~
+                               ~A, and cannot be used in ~A.~:@>")
+                    name kind context)))
+    (let ((kind (info :variable :kind name)))
+      (case kind
+        (:macro
+         (unless allow-symbol-macro
+           (program-assert-symbol-home-package-unlocked
+            :compile name (format nil "lexically binding global ~
+                                       symbol-macro ~~A in ~A"
+                                  context))))
+        ((:constant)
+         (lose "defined constant"))
+        ((:global)
+         (lose "global lexical variable"))
+        (:special
+         (unless allow-special
+           (lose "special variable"))))
+      (values name kind))))
