@@ -57,29 +57,49 @@
                 (used-p (sb-c::ir2-component-normal-tns comp))
                 (used-p (sb-c::ir2-component-wired-tns comp))))))))
 
-(defun %alloc-tramp (node result-tn size lowtag)
-  (cond ((typep size '(and integer (not (signed-byte 32))))
-         ;; MOV accepts large immediate operands, PUSH does not
-         (inst mov result-tn size)
-         (inst push result-tn))
-        (t
-         (inst push size)))
-  ;; This really would be better if it recognized TEMP-REG-TN as the "good" case
-  ;; rather than specializing on R11, which just happens to be the temp reg.
-  ;; But the assembly routine is hand-written, not generated, and it has to match,
-  ;; so there's not much that can be done to generalize it.
-  (let ((to-r11 (location= result-tn r11-tn)))
-    (invoke-asm-routine 'call (cond
-                                #+avx2
-                                ((avx-registers-used-p)
-                                 (if to-r11 'alloc-tramp-r11-avx2 'alloc-tramp-avx2))
-                                (t
-                                 (if to-r11 'alloc-tramp-r11 'alloc-tramp))) node)
+;;; Call an allocator trampoline and get the result in the proper register.
+;;; There are 2x2x2 choices of trampoline:
+;;;  - invoke alloc() or alloc_list() in C
+;;;  - place result into R11, or leave it on the stack
+;;;  - preserve YMM registers around the call, or don't
+;;; Rather than have 8 different DEFINE-ASSEMBLY-ROUTINEs, there are only 4,
+;;; and each has 2 entry points. The earlier entry adds 1 more instruction
+;;; to set the non-cons bit (the first of the binary choices mentioned above).
+;;; Most of the time, the inline allocation sequence wants to use the trampoline
+;;; that returns a result in TEMP-REG-TN (R11) which saves one move and
+;;; clears the size argument from the stack in the RET instruction.
+;;; If the result is returned on the stack, then we pop it below.
+(defun %alloc-tramp (type node result-tn size lowtag)
+  (let ((consp (eq type 'list))
+        (to-r11 (location= result-tn r11-tn)))
+    (when (typep size 'integer)
+      (aver (= (align-up size (* 2 n-word-bytes)) size))
+      (when (neq type 'list)
+        (incf size) ; the low bit means we're allocating a non-cons object
+        ;; Jump into the cons entry point which saves one instruction because why not.
+        (setq consp t)))
+    (cond ((typep size '(and integer (not (signed-byte 32))))
+           ;; MOV accepts large immediate operands, PUSH does not
+           (inst mov result-tn size)
+           (inst push result-tn))
+          (t
+           (inst push size)))
+    (invoke-asm-routine
+     'call
+     (cond #+avx2
+           ((avx-registers-used-p)
+            (if to-r11
+                (if consp 'cons->r11.avx2 'alloc->r11.avx2)
+                (if consp 'cons->rnn.avx2 'alloc->rnn.avx2)))
+           (t
+            (if to-r11
+                (if consp 'cons->r11 'alloc->r11)
+                (if consp 'cons->rnn 'alloc->rnn))))
+     node)
     (unless to-r11
       (inst pop result-tn)))
   (unless (eql lowtag 0)
-    (inst or :byte result-tn lowtag))
-  (values))
+    (inst or :byte result-tn lowtag)))
 
 ;;; Insert allocation profiler instrumentation
 (defun instrument-alloc (size node)
@@ -117,34 +137,36 @@
 ;;; NODE may be used to make policy-based decisions.
 ;;; This function should only be used inside a pseudo-atomic section,
 ;;; which to the degree needed should also cover subsequent initialization.
-(defun allocation (alloc-tn size node &optional dynamic-extent lowtag)
+;;; CONSP says whether we're allocating conses. But we also need LOWTAG
+;;; because the vop could want the CONS to have 0 or list-pointer lowtag.
+;;;
+;;; A mnemonic device for the argument pattern here:
+;;; 1. what to allocate: type, size, lowtag describe the object
+;;; 2. how to allocate it: policy and how to invoke the trampoline
+;;; 3. where to put the result
+(defun allocation (type size lowtag node dynamic-extent alloc-tn)
   (when dynamic-extent
     (stack-allocation alloc-tn size lowtag)
     (return-from allocation (values)))
   (aver (and (not (location= alloc-tn temp-reg-tn))
              (or (integerp size) (not (location= size temp-reg-tn)))))
 
+  (aver (not (sb-assem::assembling-to-elsewhere-p)))
   ;; Otherwise do the normal inline allocation thing
   (let ((NOT-INLINE (gen-label))
         (DONE (gen-label))
-        ;; Yuck.
-        (in-elsewhere (sb-assem::assembling-to-elsewhere-p))
         ;; thread->alloc_region.free_pointer
         (free-pointer
          #+sb-thread (thread-slot-ea thread-alloc-region-slot)
-         #-sb-thread (ea (+ static-space-start
-                            (ash vector-data-offset word-shift))))
+         #-sb-thread (ea boxed-region))
         ;; thread->alloc_region.end_addr
         (end-addr
          #+sb-thread (thread-slot-ea (1+ thread-alloc-region-slot))
-         #-sb-thread (ea (+ static-space-start
-                            (ash (1+ vector-data-offset) word-shift)))))
+         #-sb-thread (ea (+ boxed-region n-word-bytes))))
 
-    (cond ((or in-elsewhere
-               ;; large objects will never be made in a per-thread region
-               (and (integerp size)
-                    (>= size large-object-size)))
-           (%alloc-tramp node alloc-tn size lowtag))
+    (cond ((typep size `(integer , large-object-size))
+           ;; large objects will never be made in a per-thread region
+           (%alloc-tramp type node alloc-tn size lowtag))
           ((eql lowtag 0)
            (cond ((and (tn-p size) (location= size alloc-tn))
                   (inst mov temp-reg-tn free-pointer)
@@ -164,7 +186,7 @@
                                 temp-reg-tn)
                                (t
                                 size))))
-               (%alloc-tramp node alloc-tn size 0))
+               (%alloc-tramp type node alloc-tn size 0))
              (inst jmp DONE)))
           (t
            (inst mov temp-reg-tn free-pointer)
@@ -182,10 +204,10 @@
            (assemble (:elsewhere)
              (emit-label NOT-INLINE)
              (cond ((and (tn-p size) (location= size alloc-tn)) ; recover SIZE
-                    (inst sub alloc-tn free-pointer)
-                    (%alloc-tramp node temp-reg-tn alloc-tn 0))
+                    (inst sub alloc-tn temp-reg-tn)
+                    (%alloc-tramp type node temp-reg-tn alloc-tn 0))
                    (t ; SIZE is intact
-                    (%alloc-tramp node temp-reg-tn size 0)))
+                    (%alloc-tramp type node temp-reg-tn size 0)))
              (inst jmp DONE))))
     (values)))
 
@@ -196,12 +218,12 @@
                     &aux (bytes (pad-data-block size)))
   (let ((header (compute-object-header size widetag)))
     (cond (stack-allocate-p
-           (allocation result-tn bytes node t other-pointer-lowtag)
+           (allocation nil bytes other-pointer-lowtag node t result-tn)
            (storew header result-tn 0 other-pointer-lowtag))
           (t
            (instrument-alloc bytes node)
            (pseudo-atomic ()
-             (allocation result-tn bytes node nil 0)
+             (allocation nil bytes 0 node nil result-tn)
              (storew* header result-tn 0 0 t)
              (inst or :byte result-tn other-pointer-lowtag))))))
 
@@ -241,8 +263,8 @@
                (unless stack-allocate-p
                  (instrument-alloc size node))
                (pseudo-atomic (:elide-if stack-allocate-p)
-                (allocation res size node stack-allocate-p
-                            (if (= cons-cells 2) 0 list-pointer-lowtag))
+                (allocation 'list size (if (= cons-cells 2) 0 list-pointer-lowtag)
+                            node stack-allocate-p res)
                 (multiple-value-bind (last-base-reg lowtag car cdr)
                     (cond
                       ((= cons-cells 2)
@@ -350,7 +372,7 @@
       (let ((size (calc-size-in-bytes words result)))
         (instrument-alloc size node)
         (pseudo-atomic ()
-         (allocation result size node nil 0)
+         (allocation nil size 0 node nil result)
          (put-header result 0 type length t)
          (inst or :byte result other-pointer-lowtag)))))
 
@@ -448,8 +470,7 @@
                                (ash 1 (1+ (- word-shift n-fixnum-tag-bits)))))
                      ,answer)))
            (compute-end ()
-             `(let ((size (cond ((or (not (fixnump size))
-                                     (plausible-signed-imm32-operand-p size))
+             `(let ((size (cond ((typep size '(or (signed-byte 32) tn))
                                  size)
                                 (t
                                  (inst mov limit size)
@@ -500,7 +521,7 @@
              (and (sc-is element immediate) (eql (tn-value element) 0))))
         (instrument-alloc size node)
         (pseudo-atomic ()
-         (allocation result size node nil list-pointer-lowtag)
+         (allocation 'list size list-pointer-lowtag node nil result)
          (compute-end)
          (inst mov next result)
          (inst jmp entry)
@@ -543,7 +564,7 @@
      (unless stack-allocate-p
        (instrument-alloc bytes node))
      (pseudo-atomic (:elide-if stack-allocate-p)
-       (allocation result bytes node stack-allocate-p fun-pointer-lowtag)
+       (allocation nil bytes fun-pointer-lowtag node stack-allocate-p result)
        (storew* #-immobile-space header ; write the widetag and size
                 #+immobile-space        ; ... plus the layout pointer
                 (progn (inst mov temp header)
@@ -569,12 +590,6 @@
 
 ;;;; automatic allocators for primitive objects
 
-(define-vop (make-unbound-marker)
-  (:args)
-  (:results (result :scs (descriptor-reg any-reg)))
-  (:generator 1
-    (inst mov result unbound-marker-widetag)))
-
 (define-vop (make-funcallable-instance-tramp)
   (:args)
   (:results (result :scs (any-reg)))
@@ -605,10 +620,10 @@
            (t
             ;; If storing a header word, defer ORing in the lowtag until after
             ;; the header is written so that displacement can be 0.
-            (allocation result bytes node stack-allocate-p (if type 0 lowtag))
+            (allocation nil bytes (if type 0 lowtag) node stack-allocate-p result)
             (when type
               (let* ((widetag (if instancep instance-widetag type))
-                     (header (logior (ash (1- words) n-widetag-bits) widetag)))
+                     (header (logior (ash (1- words) (length-field-shift widetag)) widetag)))
                 (if (or #+compact-instance-header
                         (and (eq name '%make-structure-instance) stack-allocate-p))
                     ;; Write a :DWORD, not a :QWORD, because the high half will be
@@ -647,9 +662,9 @@
             (ea (* (1+ words) n-word-bytes) nil
                 extra (ash 1 (- word-shift n-fixnum-tag-bits))))
       (inst mov operand-size header bytes)
-      (inst shl operand-size header (- n-widetag-bits word-shift)) ; w+1 to length field
+      (inst shl operand-size header (- (length-field-shift type) word-shift)) ; w+1 to length field
       (inst lea operand-size header                    ; (w-1 << 8) | type
-            (ea (+ (ash -2 n-widetag-bits) type) header))
+            (ea (+ (ash -2 (length-field-shift type)) type) header))
       (inst and operand-size bytes (lognot lowtag-mask)))
       (cond (stack-allocate-p
              (stack-allocation result bytes lowtag)
@@ -657,7 +672,7 @@
             (t
              (instrument-alloc bytes node)
              (pseudo-atomic ()
-              (allocation result bytes node nil lowtag)
+              (allocation nil bytes lowtag node nil result)
               (storew header result 0 lowtag))))))
 
 (macrolet ((c-call (name)
@@ -697,7 +712,7 @@
    ;; which has that effect
    (inst and rsp-tn -16)
    (pseudo-atomic () (c-call "alloc_code_object"))
-   ;; C-RESULT is a tagged ptr. MOV doesn't need to be pseudo-atomic.
+   ;; C-RESULT is a tagged ptr. MOV doesn't need to be inside the PSEUDO-ATOMIC.
    (inst mov result c-result)))
 
 ) ; end MACROLET
