@@ -63,13 +63,17 @@
       (t
        (inst mov dst src)))))
 
-(defmacro make-ea-for-object-slot (ptr slot lowtag)
+(defun 32bit-move (dst src)
+  (unless (location= dst src)
+    (inst mov :dword dst src)))
+
+(defmacro object-slot-ea (ptr slot lowtag)
   `(ea (- (* ,slot n-word-bytes) ,lowtag) ,ptr))
 (defmacro tls-index-of (sym)
   `(ea (+ 4 (- other-pointer-lowtag)) ,sym))
 
 (defmacro loadw (value ptr &optional (slot 0) (lowtag 0))
-  `(inst mov ,value (make-ea-for-object-slot ,ptr ,slot ,lowtag)))
+  `(inst mov ,value (object-slot-ea ,ptr ,slot ,lowtag)))
 
 (defun storew (value ptr &optional (slot 0) (lowtag 0))
   (let* ((size (if (tn-p value)
@@ -85,10 +89,10 @@
            (inst mov :qword ea value)))))
 
 (defmacro pushw (ptr &optional (slot 0) (lowtag 0))
-  `(inst push (make-ea-for-object-slot ,ptr ,slot ,lowtag)))
+  `(inst push (object-slot-ea ,ptr ,slot ,lowtag)))
 
 (defmacro popw (ptr &optional (slot 0) (lowtag 0))
-  `(inst pop (make-ea-for-object-slot ,ptr ,slot ,lowtag)))
+  `(inst pop (object-slot-ea ,ptr ,slot ,lowtag)))
 
 
 ;;;; macros to generate useful values
@@ -97,10 +101,11 @@
   `(inst mov ,reg (+ nil-value (static-symbol-offset ,symbol))))
 
 ;; Return the effective address of the value slot of static SYMBOL.
-(defun static-symbol-value-ea (symbol)
+(defun static-symbol-value-ea (symbol &optional (byte 0))
    (ea (+ nil-value
           (static-symbol-offset symbol)
           (ash symbol-value-slot word-shift)
+          byte
           (- other-pointer-lowtag))))
 
 (defun thread-tls-ea (index)
@@ -114,6 +119,11 @@
   ;; explicit displacement of 0.  Using INDEX as base avoids the extra byte.
   (ea index thread-base-tn))
 
+;;; assert that alloc-region->free_pointer and ->end_addr can be accessed
+;;; using a single byte displacement from thread-base-tn
+(eval-when (:compile-toplevel)
+  (aver (<= (1+ thread-alloc-region-slot) 15)))
+
 (defun thread-slot-ea (slot-index)
   (ea (ash slot-index word-shift) thread-base-tn))
 
@@ -193,8 +203,7 @@
 
 #+sb-safepoint
 (defun emit-safepoint ()
-  (inst test :byte rax-tn (ea (- nil-value n-word-bytes other-pointer-lowtag
-                                 gc-safepoint-trap-offset))))
+  (inst test :byte rax-tn (ea (- static-space-start gc-safepoint-trap-offset))))
 
 (defmacro pseudo-atomic ((&key elide-if) &rest forms)
   #+sb-safepoint-strictly
@@ -224,6 +233,16 @@
 
 ;;;; indexed references
 
+(sb-xc:deftype load/store-index (scale lowtag min-offset
+                                 &optional (max-offset min-offset))
+  `(integer ,(- (truncate (+ (ash 1 16)
+                             (* min-offset sb-vm:n-word-bytes)
+                             (- lowtag))
+                          scale))
+            ,(truncate (- (+ (1- (ash 1 16)) lowtag)
+                          (* max-offset sb-vm:n-word-bytes))
+                       scale)))
+
 (defmacro define-full-compare-and-swap
     (name type offset lowtag scs el-type &optional translate)
   `(progn
