@@ -328,6 +328,20 @@ static void trace_object(lispobj* where)
 #endif
     case CODE_HEADER_WIDETAG:
         scan_to = code_header_words((struct code*)where);
+#ifdef LISP_FEATURE_UNTAGGED_FDEFNS
+        struct code* code = (struct code*)where;
+        lispobj* fdefns_start = code->constants + code_n_funs(code) * 4;
+        lispobj* fdefns_end  = fdefns_start + code_n_named_calls(code);
+        lispobj* limit = where + scan_to;
+        where = where + scan_from;
+        while (where < limit) {
+            lispobj word = *where;
+            if (where >= fdefns_start && where < fdefns_end) word |= OTHER_POINTER_LOWTAG;
+            gc_mark_obj(word);
+            ++where;
+        }
+        return;
+#endif
         break;
     case FDEFN_WIDETAG:
         gc_mark_obj(fdefn_callee_lispobj((struct fdefn*)where));
@@ -373,7 +387,7 @@ void execute_full_mark_phase()
     struct rusage before, after;
     getrusage(RUSAGE_SELF, &before);
 #endif
-    lispobj* where = (lispobj*)STATIC_SPACE_START;
+    lispobj* where = (lispobj*)STATIC_SPACE_OBJECTS_START;
     lispobj* end = static_space_free_pointer;
     while (where < end) {
         lispobj obj = compute_lispobj(where);
@@ -495,27 +509,16 @@ static uword_t sweep(lispobj* where, lispobj* end, uword_t arg)
 
     // TODO: consecutive dead objects on same page should be merged.
     for ( ; where < end ; where += nwords ) {
-        lispobj header = *where;
-        if (is_cons_half(header)) {
-            nwords = 2;
-            if (!cons_markedp((lispobj)where)) {
-                if (where[0] | where[1]) {
-               cons:
-                    gc_dcheck(!immobile_space_p((lispobj)where));
-                    NOTE_GARBAGE(page_table[find_page_index(where)].gen,
-                                 where, 2, zeroed,
-                                 where[0] = where[1] = 0);
-                }
-            }
-        } else {
-            nwords = sizetab[header_widetag(header)](where);
+        lispobj word = *where;
+        if (is_header(word)) {
+            nwords = sizetab[header_widetag(word)](where);
             lispobj markbit = MARK_BIT;
-            switch (header_widetag(header)) {
+            switch (header_widetag(word)) {
             case BIGNUM_WIDETAG: markbit = BIGNUM_MARK_BIT; break;
             case FDEFN_WIDETAG : markbit = FDEFN_MARK_BIT; break;
             }
-            if (header & markbit)
-                *where = header ^ markbit;
+            if (word & markbit)
+                *where = word ^ markbit;
             else {
                 // Turn the object into either a (0 . 0) cons
                 // or an unboxed filler depending on size.
@@ -535,6 +538,17 @@ static uword_t sweep(lispobj* where, lispobj* end, uword_t arg)
                     })
                 }
             }
+        } else {
+            nwords = 2;
+            if (!cons_markedp((lispobj)where)) {
+                if (where[0] | where[1]) {
+               cons:
+                    gc_dcheck(!immobile_space_p((lispobj)where));
+                    NOTE_GARBAGE(page_table[find_page_index(where)].gen,
+                                 where, 2, zeroed,
+                                 where[0] = where[1] = 0);
+                }
+            }
         }
     }
     return 0;
