@@ -50,6 +50,7 @@
 #include "forwarding-ptr.h"
 #include "var-io.h"
 #include "search.h"
+#include "murmur_hash.h"
 
 #ifdef LISP_FEATURE_SPARC
 #define LONG_FLOAT_SIZE 4
@@ -92,7 +93,15 @@ copy_object(lispobj object, sword_t nwords)
     return gc_general_copy_object(object, nwords, BOXED_PAGE_FLAG);
 }
 
+#ifdef LISP_FEATURE_PPC64
+// unevenly spaced pointer lowtags
+static void (*scav_ptr[16])(lispobj *where, lispobj object); /* forward decl */
+#define PTR_SCAVTAB_INDEX(ptr) (ptr & 15)
+#else
+// evenly spaced pointer lowtags
 static void (*scav_ptr[4])(lispobj *where, lispobj object); /* forward decl */
+#define PTR_SCAVTAB_INDEX(ptr) ((uint32_t)ptr>>(N_LOWTAG_BITS-2))&3
+#endif
 
 static inline void scav1(lispobj* object_ptr, lispobj object)
 {
@@ -122,7 +131,7 @@ static inline void scav1(lispobj* object_ptr, lispobj object)
     if (forwarding_pointer_p(ptr)) \
         *object_ptr = LOW_WORD(forwarding_pointer_value(ptr)); \
     else /* Scavenge that pointer. */ \
-        scav_ptr[(object>>(N_LOWTAG_BITS-2))&3](object_ptr, object); \
+        scav_ptr[PTR_SCAVTAB_INDEX(object)](object_ptr, object);  \
     }
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     page_index_t page;
@@ -304,7 +313,7 @@ trans_code(struct code *code)
     set_forwarding_pointer((lispobj *)code, l_new_code);
 
     struct code *new_code = (struct code *) native_pointer(l_new_code);
-    uword_t displacement = l_new_code - l_code;
+    sword_t displacement = l_new_code - l_code;
 
 #if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64 || \
     defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
@@ -428,7 +437,43 @@ static inline lispobj copy_instance(lispobj object)
 {
     // Object is an un-forwarded object in from_space
     lispobj header = *(lispobj*)(object - INSTANCE_POINTER_LOWTAG);
-    lispobj copy = copy_object(object, 1 + (instance_length(header)|1));
+    int original_length = instance_length(header);
+    lispobj copy;
+    // KLUDGE: reading both flags at once doesn't really work
+    // unless either we know what the opaque values are:
+    //  8 = stable-hash-required-flag
+    //  9 = hash-slot-present-flag
+    // Maybe the C compiler is sufficiently smart to turn a boolean expression
+    // involving "&&" into this simple expression?
+    if (((header >> 8) & 3) == 1) { // state = hashed + not moved
+        /* If odd, add 1 (making even). Rounding that to odd and then
+         * adding 1 for the header will effectively add 2 words.
+         * Otherwise, don't add anything because a padding slot exists */
+        int new_length = original_length + (original_length & 1);
+        copy = gc_copy_object_resizing(object, 1 + (new_length|1), BOXED_PAGE_FLAG,
+                                       1 + (original_length|1));
+        lispobj *base = native_pointer(copy);
+        /* store the old address as the hash value */
+#ifdef LISP_FEATURE_64_BIT
+        uint64_t hash = murmur3_fmix64(object);
+#else
+        uint32_t hash = murmur3_fmix32(object);
+#endif
+        hash = (hash << (N_FIXNUM_TAG_BITS+1)) >> 1;
+        base[original_length+1] = hash;
+        /* If the object was enlarged by 2 words, then clear the final word.
+         * Git rev 5f435a2b66 removed prezeroizing during GC, and that
+         * last word was not the target of a memcpy */
+        if (original_length & 1) base[original_length+2] = 0;
+        *base |= 1 << flag_HashSlotPresent;
+#if 0
+        printf("stable hash: obj=%p L=%d -> %p L=%d (recalc=%d)\n",
+               (void*)object, original_length, (void*)copy, new_length,
+               instance_length(*base));
+#endif
+    } else {
+        copy = copy_object(object, 1 + (original_length|1));
+    }
     set_forwarding_pointer(native_pointer(object), copy);
     return copy;
 }
@@ -441,16 +486,31 @@ scav_instance_pointer(lispobj *where, lispobj object)
     *where = copy;
 
     struct instance* node = (struct instance*)(copy - INSTANCE_POINTER_LOWTAG);
-    // Copy chain of lockfree list nodes to consecutive memory addresses,
-    // just like trans_list does.  A logically deleted node will break the chain,
-    // as its 'next' will not satisfy instancep(), but that's ok.
-    if (node->header & CUSTOM_GC_SCAVENGE_FLAG) {
-        while (instancep(object = node->slots[INSTANCE_DATA_START]) // node.next
-               && from_space_p(object)
-               && !forwarding_pointer_p(native_pointer(object))) {
-            copy = copy_instance(object);
-            node->slots[INSTANCE_DATA_START] = copy;
-            node = (struct instance*)(copy - INSTANCE_POINTER_LOWTAG);
+    lispobj layout = instance_layout((lispobj*)node);
+    if (layout) {
+        if (forwarding_pointer_p((lispobj*)LAYOUT(layout)))
+            layout = forwarding_pointer_value((lispobj*)LAYOUT(layout));
+        if (lockfree_list_node_layout_p(LAYOUT(layout))) {
+            // Copy chain of lockfree list nodes to consecutive memory addresses,
+            // just like trans_list does.  A logically deleted node will break the chain,
+            // as its 'next' will not satisfy instancep(), but that's ok.
+            while (instancep(object = node->slots[INSTANCE_DATA_START]) // node.next
+                   && from_space_p(object)
+                   && !forwarding_pointer_p(native_pointer(object))) {
+                copy = copy_instance(object);
+                node->slots[INSTANCE_DATA_START] = copy;
+                node = (struct instance*)(copy - INSTANCE_POINTER_LOWTAG);
+                // We don't have to stop upon seeing an instance with a different layout.
+                // The only other object in the 'next' chain could be *TAIL-ATOM* if we reach
+                // the end. It's possible that all of the tests in the 'while' loop are met
+                // by *TAIL-ATOM* though probably not, because it is in pseudo-static space
+                // which is never from_space. But even we iterate one more time, it's fine,
+                // despite tail-atom's layout not having the custom GC bit set - it has the
+                // 'next' pointer in the required slot. So even if we somehow copy it in
+                // this loop (vs when scavenging the symbol referencing it), the iteration
+                // after this will stop because forwarding_pointer_p will be true next time
+                // due to the fact that *TAIL-ATOM* is its own 'next' (behaving like NIL).
+            }
         }
     }
 
@@ -467,7 +527,7 @@ trans_list(lispobj object)
 {
     /* Copy 'object'. */
     struct cons *copy = (struct cons *)
-        gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG, ALLOC_QUICK);
+        gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG);
     lispobj new_list_pointer = make_lispobj(copy, LIST_POINTER_LOWTAG);
     copy->car = CONS(object)->car;
     /* Grab the cdr: set_forwarding_pointer will clobber it in GENCGC  */
@@ -484,7 +544,7 @@ trans_list(lispobj object)
         }
         /* Copy 'cdr'. */
         struct cons *cdr_copy = (struct cons*)
-            gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG, ALLOC_QUICK);
+            gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG);
         cdr_copy->car = ((struct cons*)native_cdr)->car;
         /* Grab the cdr before it is clobbered. */
         lispobj next = ((struct cons*)native_cdr)->cdr;
@@ -642,20 +702,18 @@ instance_scan(void (*proc)(lispobj*, sword_t, uword_t),
 static sword_t
 scav_instance(lispobj *where, lispobj header)
 {
-    lispobj lbitmap = make_fixnum(-1);
+    int nslots = instance_length(header) | 1;
+    if (!instance_layout(where)) return 1 + nslots;
 
-    if (instance_layout(where)) {
-        lispobj *layout = native_pointer(instance_layout(where));
+    lispobj *layout = native_pointer(instance_layout(where));
 #ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-        if (immobile_obj_gen_bits(layout) == from_space)
-            enliven_immobile_obj(layout, 1);
+    if (immobile_obj_gen_bits(layout) == from_space)
+        enliven_immobile_obj(layout, 1);
 #else
-        if (forwarding_pointer_p(layout))
-            layout = native_pointer(forwarding_pointer_value(layout));
+    if (forwarding_pointer_p(layout))
+        layout = native_pointer(forwarding_pointer_value(layout));
 #endif
-        lbitmap = ((struct layout*)layout)->bitmap;
-    }
-    sword_t nslots = instance_length(header) | 1;
+    lispobj lbitmap = ((struct layout*)layout)->bitmap;
     if (lbitmap == make_fixnum(-1)) {
         scavenge(where+1, nslots);
         return 1 + nslots;
@@ -664,9 +722,7 @@ scav_instance(lispobj *where, lispobj header)
     // pending deletion, 'next' will satisfy fixnump() but is in fact a pointer.
     // GC doesn't care too much about the deletion algorithm, but does have to
     // ensure liveness of the pointee, which may move unless pinned.
-    // One could imagine that the strategy is further determind by layout->_flags.
-    // A single bit suffices for now, but more generality is certainly possible.
-    if (header & CUSTOM_GC_SCAVENGE_FLAG) {
+    if (lockfree_list_node_layout_p((struct layout*)layout)) {
         struct instance* node = (struct instance*)where;
         lispobj next = node->slots[INSTANCE_DATA_START];
         if (fixnump(next) && next) { // ignore initially 0 heap words
@@ -688,7 +744,7 @@ scav_instance(lispobj *where, lispobj header)
                       where+1, nslots, lbitmap, 0);
     } else {
         sword_t bitmap = fixnum_value(lbitmap); // signed integer!
-        sword_t n = nslots;
+        int n = nslots;
         lispobj obj;
         for ( ; n-- ; bitmap >>= 1) {
             ++where;
@@ -699,16 +755,34 @@ scav_instance(lispobj *where, lispobj header)
     return 1 + nslots;
 }
 
+static sword_t size_instance(lispobj *where) {
+    return 1 + (instance_length(*where) | 1);
+}
+
 #ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
 static sword_t
 scav_funinstance(lispobj *where, lispobj header)
 {
-    // This works because the layout is in the header word of all instances,
-    // ordinary and funcallable, when compact headers are enabled.
-    // The trampoline slot in the funcallable-instance is raw, but can be
-    // scavenged, because it points to readonly space, never oldspace.
-    // (And for certain backends it looks like a fixnum, not a pointer)
-    return scav_instance(where, header);
+    // Do a similar thing as scav_instance but do not split into 3 cases
+    // based on whether the bitmap is a fixnum or a bignum or the special
+    // case of all tagged; it's always a fixnum, with at least 1 raw slot.
+    int nslots = SHORT_BOXED_NWORDS(header);
+    if (!instance_layout(where)) return 1 + nslots;
+
+    lispobj *layout = native_pointer(instance_layout(where));
+    if (immobile_obj_gen_bits(layout) == from_space)
+        enliven_immobile_obj(layout, 1);
+    lispobj lbitmap = ((struct layout*)layout)->bitmap;
+    gc_assert(fixnump(lbitmap));
+    sword_t bitmap = fixnum_value(lbitmap);
+    int n = nslots;
+    lispobj obj;
+    for ( ; n-- ; bitmap >>= 1) {
+        ++where;
+        if ((bitmap & 1) && is_lisp_pointer(obj = *where))
+            scav1(where, obj);
+    }
+    return 1 + nslots;
 }
 #endif
 
@@ -762,7 +836,7 @@ scav_fdefn(lispobj *where, lispobj __attribute__((unused)) object)
     lispobj obj = fdefn_callee_lispobj(fdefn);
     lispobj new = obj;
     scavenge(&new, 1);
-    if (new != obj) fdefn->raw_addr += (new - obj);
+    if (new != obj) fdefn->raw_addr += (sword_t)(new - obj);
     // Payload length is not computed from the header
     return FDEFN_SIZE;
 }
@@ -864,12 +938,11 @@ NWORDS(uword_t x, uword_t n_bits)
 
 DEF_SPECIALIZED_VECTOR(vector_nil, 0*length)
 DEF_SPECIALIZED_VECTOR(vector_bit, NWORDS(length,1))
-/* NOTE: strings contain one more element of data (a terminating '\0'
+/* NOTE: base strings contain one more element of data (a terminating '\0'
  * to help interface with C functions) than indicated by the length slot.
- * This is true even for UCS4 strings, despite that C APIs are unlikely
- * to have a convention that expects 4 zero bytes. */
+ * UCS4 strings do not get a terminator element */
 DEF_SPECIALIZED_VECTOR(base_string, NWORDS((length+1), 8))
-DEF_SPECIALIZED_VECTOR(character_string, NWORDS((length+1), 32))
+DEF_SPECIALIZED_VECTOR(character_string, NWORDS(length, 32))
 DEF_SCAV_TRANS_SIZE_UB(2)
 DEF_SCAV_TRANS_SIZE_UB(4)
 DEF_SCAV_TRANS_SIZE_UB(8)
@@ -1019,16 +1092,11 @@ static inline int pointer_survived_gc_yet(lispobj obj)
 /* Return the beginning of data in ARRAY (skipping the header and the
  * length) or NULL if it isn't an array of the specified widetag after
  * all. */
-static inline void *
-get_array_data (lispobj array, int widetag, sword_t *length)
+static inline void *get_array_data(lispobj array, int widetag)
 {
-    if (is_lisp_pointer(array) && widetag_of(native_pointer(array)) == widetag) {
-        if (length != NULL)
-            *length = fixnum_value(native_pointer(array)[1]);
+    if (is_lisp_pointer(array) && widetag_of(native_pointer(array)) == widetag)
         return &(VECTOR(array)->data[0]);
-    } else {
-        return NULL;
-    }
+    lose("bad type: %"OBJ_FMTX" should have widetag %x", array, widetag);
 }
 
 extern uword_t gc_private_cons(uword_t, uword_t);
@@ -1151,14 +1219,34 @@ void weakobj_init()
                      32 /* logical bin count */, 0 /* default range */);
 }
 
+static inline boolean stable_eql_hash_p(lispobj obj)
+{
+    return lowtag_of(obj) == OTHER_POINTER_LOWTAG
+        && widetag_of((lispobj*)(obj-OTHER_POINTER_LOWTAG)) <= SYMBOL_WIDETAG;
+}
+
+/* EQUAL and EQUALP tables always have hash vectors, so GC always knows
+ * for any given key whether it was hashed by address.
+ * EQ never has a hash vector (except if there is a user-defined hash function)
+ * but always hashes by the pointer bits (which could be an address or immediate).
+ * EQL never has a hash vector (same exception), but can hash some objects
+ * by their contents, not their address. This macro determines for a key whether
+ * its pointer bits force a rehash. In the case where we would call
+ * stable_eql_hash_p(), skip the call if 'rehash' is already 1.
+ */
+#define SHOULD_REHASH(oldkey, newkey, hashvec, hv_index) \
+  ((newkey != oldkey) && \
+   (!hashvec ? rehash || !eql_hashing || !stable_eql_hash_p(newkey) : \
+    hashvec[hv_index] == MAGIC_HASH_VECTOR_VALUE))
+
+
 /* Scavenge the "real" entries in the hash-table kv vector. The vector element
  * at index 0 bounds the scan. The element at length-1 (the hash table itself)
  * was scavenged already.
  *
  * We can disregard any entry in which both key and value are immediates.
  * This effectively ignores empty pairs, as well as makes fixnum -> fixnum table
- * more efficient. If the bitwise OR of two lispobjs satisfies is_lisp_pointer(),
- * then at least one is a pointer.
+ * more efficient.
  */
 #define SCAV_ENTRIES(entry_alivep, defer)                                      \
     boolean __attribute__((unused)) any_deferred = 0;                          \
@@ -1167,14 +1255,12 @@ void weakobj_init()
     unsigned i;                                                                \
     for (i = 1; i <= hwm; i++) {                                               \
         lispobj key = data[2*i], value = data[2*i+1];                          \
-        if (is_lisp_pointer(key|value)) {                                      \
+        if (at_least_one_pointer_p(key,value)) {                               \
           if (!entry_alivep) { defer; any_deferred = 1; } else {               \
             /* Scavenge the key and value. */                                  \
             scav_entry(&data[2*i]);                                            \
             /* mark the table for rehash if address-based key moves */         \
-            if (data[2*i] != key &&                                            \
-                (!hash_vector || hash_vector[i] == MAGIC_HASH_VECTOR_VALUE))   \
-                rehash = 1;                                                    \
+            if (SHOULD_REHASH(key, data[2*i], hashvals, i)) rehash = 1;        \
     }}}                                                                        \
     /* Though at least partly writable, vector element 1 could be on a write-protected page. */ \
     if (rehash) \
@@ -1195,18 +1281,21 @@ static void scan_nonweak_kv_vector(struct vector *kv_vector, void (*scav_entry)(
     // When growing a weak table, the KV vector is not created as weak initially;
     // its last element points to the hash-vector as for any strong KV vector.
     sword_t kv_length = fixnum_value(kv_vector->length);
-    lispobj table_or_hashes = data[kv_length-1];
-    lispobj lhash_vector;
-    if (instancep(table_or_hashes))
-        lhash_vector = ((struct hash_table*)native_pointer(table_or_hashes))->hash_vector;
-    else
-        lhash_vector = table_or_hashes;
-    sword_t hash_vector_length;
-    uint32_t *hash_vector = get_array_data(lhash_vector,
-                                           SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG,
-                                           &hash_vector_length);
-    if (hash_vector != NULL)
-        gc_assert(hash_vector_length == kv_length>>1);
+    lispobj kv_supplement = data[kv_length-1];
+    boolean eql_hashing = 0; // whether this table is an EQL table
+    if (instancep(kv_supplement)) {
+        struct hash_table* ht = (struct hash_table*)native_pointer(kv_supplement);
+        eql_hashing = hashtable_kind(ht) == 1;
+        kv_supplement = ht->hash_vector;
+    } else if (kv_supplement == T) { // EQL hashing on a non-weak table
+        eql_hashing = 1;
+        kv_supplement = NIL;
+    }
+    uint32_t *hashvals = 0;
+    if (kv_supplement != NIL) {
+        hashvals = get_array_data(kv_supplement, SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG);
+        gc_assert(2 * fixnum_value(VECTOR(kv_supplement)->length) + 1 == kv_length);
+    }
     SCAV_ENTRIES(1, );
 }
 
@@ -1214,46 +1303,34 @@ boolean scan_weak_hashtable(struct hash_table *hash_table,
                             int (*predicate)(lispobj,lispobj),
                             void (*scav_entry)(lispobj*))
 {
-    sword_t kv_length;
-    sword_t length;
-    sword_t next_vector_length;
-    sword_t hash_vector_length;
-
-    lispobj *data = get_array_data(hash_table->pairs,
-                                   SIMPLE_VECTOR_WIDETAG, &kv_length);
+    lispobj *data = get_array_data(hash_table->pairs, SIMPLE_VECTOR_WIDETAG);
     if (data == NULL)
         lose("invalid kv_vector %"OBJ_FMTX, hash_table->pairs);
+    sword_t kv_length = fixnum_value(VECTOR(hash_table->pairs)->length);
 
-    uint32_t *index_vector = get_array_data(hash_table->index_vector,
-                                            SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG,
-                                            &length);
-    if (index_vector == NULL)
-        lose("invalid index_vector %"OBJ_FMTX, hash_table->index_vector);
+    uint32_t *hashvals = 0;
+    if (hash_table->hash_vector != NIL) {
+        hashvals = get_array_data(hash_table->hash_vector,
+                                     SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG);
+        gc_assert(VECTOR(hash_table->hash_vector)->length ==
+                  VECTOR(hash_table->next_vector)->length);
+    }
 
-    if (get_array_data(hash_table->next_vector,
-                       SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG,
-                       &next_vector_length) == NULL)
-        lose("invalid next_vector %"OBJ_FMTX, hash_table->next_vector);
-
-    uint32_t *hash_vector = get_array_data(hash_table->hash_vector,
-                                           SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG,
-                                           &hash_vector_length);
-    if (hash_vector != NULL)
-        gc_assert(hash_vector_length == next_vector_length);
-
-     /* These lengths could be different as the index_vector can be a
-      * different length from the others, a larger index_vector could
-      * help reduce collisions. */
-    gc_assert(next_vector_length == kv_length >> 1);
+     /* next_vector and hash_vector have a 1:1 size relation.
+      * kv_vector is twice that plus the supplemental cell.
+      * The index vector length is arbitrary - it can be smaller or larger
+      * than the maximum number of table entries. */
+    gc_assert(2 * fixnum_value(VECTOR(hash_table->next_vector)->length) + 1
+              == kv_length);
 
     int weakness = hashtable_weakness(hash_table);
+    boolean eql_hashing = hashtable_kind(hash_table) == 1;
     /* Work through the KV vector. */
     SCAV_ENTRIES(predicate(key, value), add_kv_triggers(&data[2*i], weakness));
     if (!any_deferred && debug_weak_ht)
         fprintf(stderr,
                 "will skip rescan of weak ht: %d/%d items\n",
-                (int)KV_PAIRS_HIGH_WATER_MARK(data),
-                (int)(next_vector_length - 1));
+                (int)KV_PAIRS_HIGH_WATER_MARK(data), (int)(kv_length/2));
 
     return any_deferred;
 }
@@ -1264,8 +1341,12 @@ scav_vector (lispobj *where, lispobj header)
     sword_t length = fixnum_value(where[1]);
 
     /* SB-VM:VECTOR-HASHING-SUBTYPE is set for all hash tables in the
-     * Lisp HASH-TABLE code to indicate need for special GC support. */
-    if (is_vector_subtype(header, VectorNormal)) {
+     * Lisp HASH-TABLE code to indicate need for special GC support.
+     * But note that if the vector is a hashing vector that is neither
+     * weak nor contains any key hashed by its address, then it is basically
+     * a regular vector, except that GC needs only to look at cells below
+     * the high-water-mark, plus the SUPPLEMENT slot at the end */
+    if (vector_subtype(header) == 0) { // no special indicator bits set
  normal:
         scavenge(where + 2, length);
         goto done;
@@ -1277,7 +1358,7 @@ scav_vector (lispobj *where, lispobj header)
     }
 
     lispobj* data = where + 2;
-    // Verify that the rehash epoch is a fixnum
+    // Verify that the rehash stamp is a fixnum
     gc_assert(fixnump(data[1]));
 
     /* Scavenge element (length-1), which may be a hash-table structure. */
@@ -1351,8 +1432,9 @@ scav_vector (lispobj *where, lispobj header)
 }
 
 /* Walk through the chain whose first element is *FIRST and remove
- * dead weak entries. */
-static inline void
+ * dead weak entries.
+ * Return the new value for 'should rehash' */
+static inline boolean
 cull_weak_hash_table_bucket(struct hash_table *hash_table,
                             uint32_t bucket, uint32_t index,
                             lispobj *kv_vector,
@@ -1360,9 +1442,10 @@ cull_weak_hash_table_bucket(struct hash_table *hash_table,
                             int (*alivep_test)(lispobj,lispobj),
                             void (*fix_pointers)(lispobj[2]),
                             boolean save_culled_values,
-                            boolean *rehash)
+                            boolean rehash)
 {
     const lispobj empty_symbol = UNBOUND_MARKER_WIDETAG;
+    int eql_hashing = hashtable_kind(hash_table) == 1;
     for ( ; index ; index = next_vector[index] ) {
         lispobj key = kv_vector[2 * index];
         lispobj value = kv_vector[2 * index + 1];
@@ -1371,6 +1454,8 @@ cull_weak_hash_table_bucket(struct hash_table *hash_table,
         if (key == empty_symbol && value == empty_symbol) continue;
         // If the pair doesn't have both halves empty,
         // then it mustn't have either half empty.
+        // FIXME: this looks like a potential data race - do we definitely store
+        // the key and value before inserting into a chain? Probably.
         gc_assert(key != empty_symbol);
         gc_assert(value != empty_symbol);
         if (!alivep_test(key, value)) {
@@ -1379,7 +1464,7 @@ cull_weak_hash_table_bucket(struct hash_table *hash_table,
                 lispobj val = kv_vector[2 * index + 1];
                 gc_assert(!is_lisp_pointer(val));
                 struct cons *cons = (struct cons*)
-                  gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG, ALLOC_QUICK);
+                  gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG);
                 // Lisp code which manipulates the culled_values slot must use
                 // compare-and-swap, but C code need not, because GC runs in one
                 // thread and has stopped the Lisp world.
@@ -1402,14 +1487,14 @@ cull_weak_hash_table_bucket(struct hash_table *hash_table,
             struct cons *cons;
             if ((index & ~0x3FFF) | (bucket & ~0x3FFF)) { // large values
                 cons = (struct cons*)
-                  gc_general_alloc(2 * sizeof(struct cons), BOXED_PAGE_FLAG, ALLOC_QUICK);
+                  gc_general_alloc(2 * sizeof(struct cons), BOXED_PAGE_FLAG);
                 cons->car = make_lispobj(cons + 1, LIST_POINTER_LOWTAG);
                 cons[1].car = make_fixnum(index);  // which cell became free
                 cons[1].cdr = make_fixnum(bucket); // which chain was it in
                 if (!compacting_p()) gc_mark_obj(cons->car);
             } else { // small values
                 cons = (struct cons*)
-                  gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG, ALLOC_QUICK);
+                  gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG);
                 cons->car = ((index << 14) | bucket) << N_FIXNUM_TAG_BITS;
             }
             cons->cdr = hash_table->smashed_cells;
@@ -1423,12 +1508,12 @@ cull_weak_hash_table_bucket(struct hash_table *hash_table,
             if (fix_pointers) { // Follow FPs as necessary
                 lispobj key = kv_vector[2 * index];
                 fix_pointers(&kv_vector[2 * index]);
-                if (kv_vector[2 * index] != key &&
-                    (!hash_vector || hash_vector[index] == MAGIC_HASH_VECTOR_VALUE))
-                    *rehash = 1;
+                if (SHOULD_REHASH(key, kv_vector[2 * index], hash_vector, index))
+                    rehash = 1;
             }
         }
     }
+    return rehash;
 }
 
 static void
@@ -1436,27 +1521,34 @@ cull_weak_hash_table (struct hash_table *hash_table,
                       int (*alivep_test)(lispobj,lispobj),
                       void (*fix_pointers)(lispobj[2]))
 {
-    sword_t length = 0; /* prevent warning */
     sword_t i;
 
-    lispobj *kv_vector = get_array_data(hash_table->pairs,
-                                        SIMPLE_VECTOR_WIDETAG, NULL);
+    lispobj *kv_vector = get_array_data(hash_table->pairs, SIMPLE_VECTOR_WIDETAG);
     uint32_t *index_vector = get_array_data(hash_table->index_vector,
-                                            SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG,
-                                            &length);
+                                            SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG);
+    sword_t n_buckets = fixnum_value(VECTOR(hash_table->index_vector)->length);
     uint32_t *next_vector = get_array_data(hash_table->next_vector,
-                                           SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG,
-                                           NULL);
-    uint32_t *hash_vector = get_array_data(hash_table->hash_vector,
-                                           SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG, NULL);
+                                           SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG);
+    uint32_t *hash_vector = 0;
+    if (hash_table->hash_vector != NIL)
+        hash_vector = get_array_data(hash_table->hash_vector,
+                                     SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG);
 
     boolean rehash = 0;
     boolean save_culled_values = (hash_table->flags & MAKE_FIXNUM(4)) != 0;
-    for (i = 0; i < length; i++) {
-        cull_weak_hash_table_bucket(hash_table, i, index_vector[i],
-                                    kv_vector, next_vector, hash_vector,
-                                    alivep_test, fix_pointers,
-                                    save_culled_values, &rehash);
+    // I'm slightly confused as to why we can't (or don't) compute the
+    // 'should rehash' flag while scavenging the weak k/v vector.
+    // I believe the explanation is this: for weak-key-AND-value tables, the vector
+    // is never scavenged. It just ends up here after all other scavenging is done.
+    // We then need to fix the still-live pointers, which entails possibly setting the
+    // 'rehash' flag. It would not make sense to treat the other 3 flavors of
+    // weakness any differently.
+    for (i = 0; i < n_buckets; i++) {
+        if (cull_weak_hash_table_bucket(hash_table, i, index_vector[i],
+                                        kv_vector, next_vector, hash_vector,
+                                        alivep_test, fix_pointers,
+                                        save_culled_values, rehash))
+            rehash = 1;
     }
     /* If an EQ-based key has moved, mark the hash-table for rehash */
     if (rehash)
@@ -1557,6 +1649,30 @@ component_ptr_from_pc(char *pc)
     return NULL;
 }
 
+/// Return the name of the simple-fun containing 'pc',
+/// and if 'pfun' is non-null then store the base pointer
+/// to the function in *pfun. This is an easier-to-use interface than
+/// just returning the function, because accessing the name requires
+/// knowing the function's index into the entry-point vector.
+lispobj simple_fun_name_from_pc(char *pc, lispobj** pfun)
+{
+    struct code* code = (void*)component_ptr_from_pc(pc);
+    if (!code) return 0; // not in lisp heap
+    char *insts = code_text_start(code);
+    unsigned int* offsets = code_fun_table(code) - 1;
+    int i;
+    // Scanning backwards makes the stopping condition easy:
+    // the first function lower than 'pc' is the right answer.
+    for (i=code_n_funs(code)-1; i>=0; --i) {
+        struct simple_fun* fun = (void*)(insts + offsets[-i]);
+        if ((char*)fun < pc) {
+            if (pfun) *pfun = (lispobj*)fun;
+            return code->constants[i*CODE_SLOTS_PER_SIMPLE_FUN];
+        }
+    }
+    return 0; // oops, how did this happen?
+}
+
 /* Scan an area looking for an object which encloses the given pointer.
  * Return the object start on success, or NULL on failure. */
 lispobj *
@@ -1630,20 +1746,14 @@ properly_tagged_p_internal(lispobj pointer, lispobj *start_addr)
 {
     // If a headerless object, confirm that 'pointer' is a list pointer.
     // Given the precondition that the heap is in a valid state,
-    // it may be assumed that one check of is_cons_half() suffices;
-    // we don't need to check the other half.
-    lispobj header = *start_addr;
-    if (is_cons_half(header))
+    // it may be assumed that is_cons_half() is satisfied by both the
+    // car and cdr.
+    lispobj word = *start_addr;
+    if (!is_header(word))
         return make_lispobj(start_addr, LIST_POINTER_LOWTAG) == pointer;
 
-    // Because this heap object was not deemed to be a cons,
-    // it must be an object header. Don't need a check except when paranoid.
-    gc_dcheck(other_immediate_lowtag_p(header));
-
-    // The space of potential widetags has 64 elements, not 256,
-    // because of the constant low 2 bits.
-    int widetag = header_widetag(header);
-    int lowtag = lowtag_for_widetag[widetag>>2];
+    int widetag = header_widetag(word);
+    int lowtag = LOWTAG_FOR_WIDETAG(widetag);
     if (lowtag && make_lispobj(start_addr, lowtag) == pointer)
         return 1; // instant win
 
@@ -1741,7 +1851,7 @@ maybe_gc(os_context_t *context)
      */
 #if !(defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_SAFEPOINT))
     check_gc_signals_unblocked_or_lose(os_context_sigmask_addr(context));
-    unblock_gc_signals(0, 0);
+    unblock_gc_signals();
 #endif
     FSHOW((stderr, "/maybe_gc: calling SUB_GC\n"));
     /* FIXME: Nothing must go wrong during GC else we end up running
@@ -2271,3 +2381,8 @@ void gc_heapsort_uwords(heap array, int length)
         sift_down(array, 0, end);
     }
 }
+
+/// External function for calling from Lisp.
+page_index_t ext_lispobj_size(lispobj *addr) {
+    return OBJECT_SIZE(*addr,addr) * N_WORD_BYTES;
+}
