@@ -35,13 +35,11 @@ extern int foreign_function_call_active;
 extern os_vm_size_t dynamic_space_size;
 extern os_vm_size_t thread_control_stack_size;
 
-#if defined(LISP_FEATURE_RELOCATABLE_HEAP)
 #ifdef LISP_FEATURE_CHENEYGC
 extern uword_t DYNAMIC_0_SPACE_START, DYNAMIC_1_SPACE_START;
 #else
 extern uword_t DYNAMIC_SPACE_START;
 #endif
-#endif
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
 extern uword_t FIXEDOBJ_SPACE_START, VARYOBJ_SPACE_START;
 extern uword_t immobile_space_lower_bound, immobile_space_max_offset;
@@ -65,10 +63,6 @@ extern lispobj alloc_profile_data; // Lisp SIMPLE-VECTOR
 #endif
 extern char **ENVIRON;
 
-#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_GCC_TLS)
-extern pthread_key_t specials;
-#endif
-
 #if !defined(LISP_FEATURE_SB_THREAD)
 extern lispobj *current_control_stack_pointer;
 #endif
@@ -92,7 +86,9 @@ extern lispobj *current_binding_stack_pointer;
  *  moment, it is also used by the GENCGC, to hold the pseudo_atomic
  *  bits, and is tightly coupled to reg_ALLOC by the assembly
  *  routines. */
+#if !(defined LISP_FEATURE_ARM || defined LISP_FEATURE_ARM64 || defined LISP_FEATURE_RISCV)
 extern lispobj *dynamic_space_free_pointer;
+#endif
 extern lispobj *read_only_space_free_pointer;
 extern lispobj *static_space_free_pointer;
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
@@ -176,7 +172,9 @@ EXTERN(current_control_frame_pointer, POINTERSIZE)
 # if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64)
 EXTERN(current_binding_stack_pointer, POINTERSIZE)
 # endif
-# ifndef LISP_FEATURE_GENCGC
+// don't want an undefined C symbol for this in 'nm' output, it's confusing
+# if defined LISP_FEATURE_CHENEYGC && \
+  !(defined LISP_FEATURE_ARM || defined LISP_FEATURE_ARM64 || defined LISP_FEATURE_RISCV)
 EXTERN(dynamic_space_free_pointer, POINTERSIZE)
 # endif
 
