@@ -87,6 +87,12 @@
 # define REAL_SIGSET_SIZE_BYTES ((NSIG/8))
 #endif
 
+#ifdef LISP_FEATURE_NETBSD
+#define OS_SA_NODEFER 0
+#else
+#define OS_SA_NODEFER SA_NODEFER
+#endif
+
 static inline void
 sigcopyset(sigset_t *new, sigset_t *old)
 {
@@ -165,7 +171,7 @@ static void sigmask_logandc(sigset_t *dest, const sigset_t *source)
 
 /* Foreign code may want to start some threads on its own.
  * Non-targetted, truly asynchronous signals can be delivered to
- * basically any thread, but invoking Lisp handlers in such foregign
+ * basically any thread, but invoking Lisp handlers in such foreign
  * threads is really bad, so let's resignal it.
  *
  * This should at least bring attention to the problem, but it cannot
@@ -217,7 +223,7 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
 }
 
 #if INSTALL_SIG_MEMORY_FAULT_HANDLER && defined(THREAD_SANITIZER)
-/* Under TSAN, every signal blocks every other signal regardless of the
+/* Under TSAN, any delivered signal blocks all other signals regardless of the
  * 'sa_mask' given to sigaction(). This is courtesy of an interceptor -
  * https://github.com/llvm-mirror/compiler-rt/blob/bcc227ee4af1ef3e63033b35dcb1d5627a3b2941/lib/tsan/rtl/tsan_interceptors.cc#L1972
  *
@@ -228,11 +234,6 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
  * blocked SIGSEGV exactly as if the specified disposition were SIG_DFL,
  * which results in process termination and a core dump.
  *
- * It doesn't work to route all our signals through 'unblock_me_trampoline',
- * because that only unblocks the specific signal that was just delivered,
- * to work around the problem of SA_NODEFER not working. (Which says that
- * a signal should not be blocked within in its own handler; it says nothing
- * about all other signals.)
  * Our trick is to unblock SIGSEGV early in every handler,
  * so not to face sudden death if it happens to invoke Lisp.
  */
@@ -249,12 +250,10 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
  *
  * interrupt_handle_now_handler
  * maybe_now_maybe_later
- * unblock_me_trampoline
  * low_level_handle_now_handler
  * low_level_maybe_now_maybe_later
- * low_level_unblock_me_trampoline
  *
- * This gives us a single point of control (or six) over errno, fp
+ * This gives us a single point of control (or four) over errno, fp
  * control word, and fixing up signal context on sparc.
  *
  * The SPARC/Linux platform doesn't quite do signals the way we want
@@ -373,14 +372,7 @@ sigaddset_deferrable(sigset_t *s)
     sigaddset(s, SIGWINCH);
 }
 
-void
-sigaddset_blockable(sigset_t *sigset)
-{
-    sigaddset_deferrable(sigset);
-    sigaddset_gc(sigset);
-}
-
-void
+static void
 sigaddset_gc(sigset_t __attribute__((unused)) *sigset)
 {
 #ifdef THREADS_USING_GCSIGNAL
@@ -388,6 +380,13 @@ sigaddset_gc(sigset_t __attribute__((unused)) *sigset)
 #endif
 }
 
+void
+sigaddset_blockable(sigset_t *sigset)
+{
+    sigaddset_deferrable(sigset);
+    sigaddset_gc(sigset);
+}
+
 /* initialized in interrupt_init */
 sigset_t deferrable_sigset;
 sigset_t blockable_sigset;
@@ -1430,7 +1429,7 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
              (lispobj)fixnum_value(thread->state));
     }
 
-    set_thread_state(thread,STATE_STOPPED);
+    set_thread_state(thread, STATE_STOPPED, 0);
     FSHOW_SIGNAL((stderr,"suspended\n"));
 
     /* While waiting for gc to finish occupy ourselves with zeroing
@@ -1837,80 +1836,7 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
     else return 0;
 }
 
-/*
- * noise to install handlers
- */
-
 #ifndef LISP_FEATURE_WIN32
-/* In Linux 2.4 synchronous signals (sigtrap & co) can be delivered if
- * they are blocked, in Linux 2.6 the default handler is invoked
- * instead that usually coredumps. One might hastily think that adding
- * SA_NODEFER helps, but until ~2.6.13 if SA_NODEFER is specified then
- * the whole sa_mask is ignored and instead of not adding the signal
- * in question to the mask. That means if it's not blockable the
- * signal must be unblocked at the beginning of signal handlers.
- *
- * It turns out that NetBSD's SA_NODEFER doesn't DTRT in a different
- * way: if SA_NODEFER is set and the signal is in sa_mask, the signal
- * will be unblocked in the sigmask during the signal handler.  -- RMK
- * X-mas day, 2005
- */
-static volatile int sigaction_nodefer_works = -1;
-
-#define SA_NODEFER_TEST_BLOCK_SIGNAL SIGABRT
-#define SA_NODEFER_TEST_KILL_SIGNAL SIGUSR1
-
-static void
-sigaction_nodefer_test_handler(int signal,
-                               siginfo_t __attribute__((unused)) *info,
-                               void __attribute__((unused)) *void_context)
-{
-    sigset_t current;
-    int i;
-    get_current_sigmask(&current);
-    /* There should be exactly two blocked signals: the two we added
-     * to sa_mask when setting up the handler.  NetBSD doesn't block
-     * the signal we're handling when SA_NODEFER is set; Linux before
-     * 2.6.13 or so also doesn't block the other signal when
-     * SA_NODEFER is set. */
-    for(i = 1; i <= MAX_SIGNUM; i++)
-        if (sigismember(&current, i) !=
-            (((i == SA_NODEFER_TEST_BLOCK_SIGNAL) || (i == signal)) ? 1 : 0)) {
-            FSHOW_SIGNAL((stderr, "SA_NODEFER doesn't work, signal %d\n", i));
-            sigaction_nodefer_works = 0;
-        }
-    if (sigaction_nodefer_works == -1)
-        sigaction_nodefer_works = 1;
-}
-
-void set_sigaction_nodefer_works() { sigaction_nodefer_works = 1; }
-
-static void
-see_if_sigaction_nodefer_works(void)
-{
-    if (sigaction_nodefer_works > 0) return; // good
-    struct sigaction sa, old_sa;
-
-    sa.sa_flags = SA_SIGINFO | SA_NODEFER;
-    sa.sa_sigaction = sigaction_nodefer_test_handler;
-    sigemptyset(&sa.sa_mask);
-    sigaddset(&sa.sa_mask, SA_NODEFER_TEST_BLOCK_SIGNAL);
-    sigaddset(&sa.sa_mask, SA_NODEFER_TEST_KILL_SIGNAL);
-    sigaction(SA_NODEFER_TEST_KILL_SIGNAL, &sa, &old_sa);
-    /* Make sure no signals are blocked. */
-    {
-        sigset_t empty;
-        sigemptyset(&empty);
-        thread_sigmask(SIG_SETMASK, &empty, 0);
-    }
-    kill(getpid(), SA_NODEFER_TEST_KILL_SIGNAL);
-    while (sigaction_nodefer_works == -1);
-    sigaction(SA_NODEFER_TEST_KILL_SIGNAL, &old_sa, NULL);
-}
-
-#undef SA_NODEFER_TEST_BLOCK_SIGNAL
-#undef SA_NODEFER_TEST_KILL_SIGNAL
-
 extern void restore_sbcl_signals () {
     int signal;
     for (signal = 0; signal < NSIG; signal++) {
@@ -2004,32 +1930,6 @@ lost:
 }
 #endif
 
-static void
-unblock_me_trampoline(int signal, siginfo_t *info, void *void_context)
-{
-    SAVE_ERRNO(signal,context,void_context);
-    sigset_t unblock;
-
-    sigemptyset(&unblock);
-    sigaddset(&unblock, signal);
-    thread_sigmask(SIG_UNBLOCK, &unblock, 0);
-    interrupt_handle_now(signal, info, context);
-    RESTORE_ERRNO;
-}
-
-static void
-low_level_unblock_me_trampoline(int signal, siginfo_t *info, void *void_context)
-{
-    SAVE_ERRNO(signal,context,void_context);
-    sigset_t unblock;
-
-    sigemptyset(&unblock);
-    sigaddset(&unblock, signal);
-    thread_sigmask(SIG_UNBLOCK, &unblock, 0);
-    (*interrupt_low_level_handlers[signal])(signal, info, context);
-    RESTORE_ERRNO;
-}
-
 static void
 low_level_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 {
@@ -2052,9 +1952,6 @@ undoably_install_low_level_interrupt_handler (int signal,
         sa.sa_sigaction = (void (*)(int, siginfo_t*, void*))handler;
     else if (sigismember(&deferrable_sigset,signal))
         sa.sa_sigaction = low_level_maybe_now_maybe_later;
-    else if (!sigaction_nodefer_works &&
-             !sigismember(&blockable_sigset, signal))
-        sa.sa_sigaction = low_level_unblock_me_trampoline;
     else
         sa.sa_sigaction = low_level_handle_now_handler;
 
@@ -2068,8 +1965,7 @@ undoably_install_low_level_interrupt_handler (int signal,
 #endif
 
     sa.sa_mask = blockable_sigset;
-    sa.sa_flags = SA_SIGINFO | SA_RESTART
-        | (sigaction_nodefer_works ? SA_NODEFER : 0);
+    sa.sa_flags = SA_SIGINFO | SA_RESTART | OS_SA_NODEFER;
 #if defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
     if(signal==SIG_MEMORY_FAULT) {
         sa.sa_flags |= SA_ONSTACK;
@@ -2115,15 +2011,11 @@ install_handler(int signal, void handler(int, siginfo_t*, os_context_t*),
 #endif
         else if (sigismember(&deferrable_sigset, signal))
             sa.sa_sigaction = maybe_now_maybe_later;
-        else if (!sigaction_nodefer_works &&
-                 !sigismember(&blockable_sigset, signal))
-            sa.sa_sigaction = unblock_me_trampoline;
         else
             sa.sa_sigaction = interrupt_handle_now_handler;
 
         sa.sa_mask = blockable_sigset;
-        sa.sa_flags = SA_SIGINFO | SA_RESTART |
-            (sigaction_nodefer_works ? SA_NODEFER : 0);
+        sa.sa_flags = SA_SIGINFO | SA_RESTART | OS_SA_NODEFER;
         sigaction(signal, &sa, NULL);
     }
 
@@ -2183,9 +2075,6 @@ interrupt_init(void)
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     int i;
     SHOW("entering interrupt_init()");
-#ifndef LISP_FEATURE_WIN32
-    see_if_sigaction_nodefer_works();
-#endif
     sigemptyset(&deferrable_sigset);
     sigemptyset(&blockable_sigset);
     sigemptyset(&gc_sigset);
