@@ -37,7 +37,8 @@
 #include "thread.h"
 #include "genesis/static-symbols.h"
 #include "genesis/primitive-objects.h"
-
+#include "genesis/gc-tables.h"
+#include "gc-internal.h"
 
 
 /* When we need to do command input, we use this stream, which is not
@@ -53,22 +54,23 @@
 static FILE *ldb_in = 0;
 static int ldb_in_fd = -1;
 
-typedef void cmd(char **ptr);
+typedef int cmd(char **ptr);
 
-static cmd dump_cmd, print_cmd, quit_cmd, help_cmd;
-static cmd flush_cmd, search_cmd, regs_cmd, exit_cmd;
-static cmd print_context_cmd;
+static cmd call_cmd, dump_cmd, print_cmd, quit_cmd, help_cmd;
+static cmd flush_cmd, regs_cmd, exit_cmd;
+static cmd print_context_cmd, pte_cmd, search_cmd;
 static cmd backtrace_cmd, purify_cmd, catchers_cmd;
 static cmd grab_sigs_cmd;
 static cmd kill_cmd;
 
 static struct cmd {
     char *cmd, *help;
-    void (*fn)(char **ptr);
+    int (*fn)(char **ptr);
 } supported_cmds[] = {
     {"help", "Display this help information.", help_cmd},
     {"?", "(an alias for help)", help_cmd},
     {"backtrace", "Backtrace up to N frames.", backtrace_cmd},
+    {"call", "Call FUNCTION with ARG1, ARG2, ...", call_cmd},
     {"catchers", "Print a list of all the active catchers.", catchers_cmd},
     {"context", "Print interrupt context number I.", print_context_cmd},
     {"dump", "Dump memory starting at ADDRESS for COUNT words.", dump_cmd},
@@ -84,10 +86,10 @@ static struct cmd {
     {"purify", "Purify. (Caveat purifier!)", purify_cmd},
     {"print", "Print object at ADDRESS.", print_cmd},
     {"p", "(an alias for print)", print_cmd},
+    {"pte", "Page table entry for address", pte_cmd},
     {"quit", "Quit.", quit_cmd},
     {"regs", "Display current Lisp registers.", regs_cmd},
-    {"search", "Search for TYPE starting at ADDRESS for a max of COUNT words.", search_cmd},
-    {"s", "(an alias for search)", search_cmd},
+    {"search", "Search heap for object.", search_cmd},
     {NULL, NULL, NULL}
 };
 
@@ -102,7 +104,7 @@ visible(unsigned char c)
         return c;
 }
 
-static void
+static int NO_SANITIZE_MEMORY
 dump_cmd(char **ptr)
 {
     static char *lastaddr = 0;
@@ -110,22 +112,26 @@ dump_cmd(char **ptr)
 
     char *addr = lastaddr;
     int count = lastcount, displacement;
-    int force = 0;
+    int force = 0, decode = 0;
 
     if (more_p(ptr)) {
+        // you can't both "force" and "decode" - only one or the other,
+        // or neither
         if (!strncmp(*ptr, "-f ", 3)) {
           force = 1;
           *ptr += 3;
+        } else if (!strncmp(*ptr, "-d ", 3)) {
+          decode = 1;
+          *ptr += 3;
         }
-        addr = parse_addr(ptr, !force);
+        if (!parse_addr(ptr, !force, &addr)) return 0;
 
-        if (more_p(ptr))
-            count = parse_number(ptr);
+        if (more_p(ptr) && !parse_number(ptr, &count)) return 0;
     }
 
     if (count == 0) {
         printf("COUNT must be non-zero.\n");
-        return;
+        return 0;
     }
 
     lastcount = count;
@@ -137,6 +143,13 @@ dump_cmd(char **ptr)
         count = -count;
     }
 
+    boolean aligned = ((uword_t)addr & LOWTAG_MASK) == 0;
+    if (decode && (!aligned || displacement < 0)) {
+        printf("Sorry, can only decode if aligned and stepping forward\n");
+        decode = 0;
+    }
+    lispobj* next_object = decode ? (lispobj*)addr : 0;
+
     while (count-- > 0) {
 #ifndef LISP_FEATURE_ALPHA
         printf("%p: ", (os_vm_address_t) addr);
@@ -152,7 +165,7 @@ dump_cmd(char **ptr)
             unsigned char *cptr = (unsigned char *)addr;
 
 #if N_WORD_BYTES == 8
-            printf("0x%016lx | %c%c%c%c%c%c%c%c\n",
+            printf("0x%016lx | %c%c%c%c%c%c%c%c",
                    lptr[0],
                    visible(cptr[0]), visible(cptr[1]),
                    visible(cptr[2]), visible(cptr[3]),
@@ -163,12 +176,36 @@ dump_cmd(char **ptr)
             printf("0x%08lx   0x%04x 0x%04x   "
                    "0x%02x 0x%02x 0x%02x 0x%02x    "
                    "%c%c"
-                   "%c%c\n",
+                   "%c%c",
                    lptr[0], sptr[0], sptr[1],
                    cptr[0], cptr[1], cptr[2], cptr[3],
                    visible(cptr[0]), visible(cptr[1]),
                    visible(cptr[2]), visible(cptr[3]));
 #endif
+#ifdef LISP_FEATURE_GENCGC
+            if (aligned) {
+                lispobj ptr = *(lispobj*)addr;
+                int gen;
+                if (is_lisp_pointer(ptr) && gc_managed_heap_space_p(ptr)
+                    && (gen = gc_gen_of(ptr, 99)) != 99) // say that static is 99
+                    if (gen != 99) printf(" | %d", gen);
+            }
+#endif
+            if (decode && addr == (char*)next_object) {
+                lispobj word = *(lispobj*)addr;
+                // ensure validity of widetag because crashing with
+                // "no size function" would be worse than doing nothing
+                if (word != 0 && !is_lisp_pointer(word)
+                    && valid_widetag_p(header_widetag(word))) {
+                    printf(" %s", widetag_names[header_widetag(word)>>2]);
+                    next_object += sizetab[header_widetag(word)](next_object);
+                } else if (!is_header(word)) {
+                    next_object += CONS_SIZE;
+                } else { // disable decoder if weirdness observed
+                    decode = 0;
+                }
+            }
+            printf("\n");
         }
         else
             printf("invalid Lisp-level address\n");
@@ -177,26 +214,38 @@ dump_cmd(char **ptr)
     }
 
     lastaddr = addr;
+    return 0;
 }
 
-static void
+static int
 print_cmd(char **ptr)
 {
-    lispobj obj = parse_lispobj(ptr);
-
-    print(obj);
+    lispobj obj;
+    if (parse_lispobj(ptr, &obj)) print(obj);
+    return 0;
 }
 
-static void
+static int
+pte_cmd(char **ptr)
+{
+    extern void gc_show_pte(lispobj);
+    lispobj obj;
+    if (parse_lispobj(ptr, &obj)) gc_show_pte(obj);
+    return 0;
+}
+
+static int
 kill_cmd(char **ptr)
 {
 #ifndef LISP_FEATURE_WIN32
-    kill(getpid(), parse_number(ptr));
+    int sig;
+    if (parse_number(ptr, &sig)) kill(getpid(), sig);
 #endif
+    return 0;
 }
 
-static void
-regs_cmd(char **ptr)
+static int
+regs_cmd(char __attribute__((unused)) **ptr)
 {
     struct thread __attribute__((unused)) *thread=arch_os_get_current_thread();
 
@@ -229,81 +278,90 @@ regs_cmd(char **ptr)
 #ifndef LISP_FEATURE_GENCGC
     printf("TRIGGER\t=\t%p\n", (void*)current_auto_gc_trigger);
 #endif
+    return 0;
 }
 
-static void
-search_cmd(char **ptr)
+static int
+call_cmd(char **ptr)
 {
-    static int lastval = 0, lastcount = 0;
-    static lispobj *start = 0, *end = 0;
-    int val, count;
-    lispobj *addr, obj;
+    lispobj thing;
+    parse_lispobj(ptr, &thing);
+    lispobj function, args[3];
+    lispobj result = NIL;
 
-    if (more_p(ptr)) {
-        val = parse_number(ptr);
-        if (val < 0 || val > 0xff) {
-            printf("can only search for single bytes\n");
-            return;
-        }
-        if (more_p(ptr)) {
-            addr = native_pointer((uword_t)parse_addr(ptr, 1));
-            if (more_p(ptr)) {
-                count = parse_number(ptr);
-            }
-            else {
-                /* Specified value and address, but no count. Only one. */
-                count = -1;
-            }
-        }
-        else {
-            /* Specified a value, but no address, so search same range. */
-            addr = start;
-            count = lastcount;
+    int numargs;
+
+    if (lowtag_of(thing) == OTHER_POINTER_LOWTAG) {
+        lispobj *obj = native_pointer(thing);
+        switch (widetag_of(obj)) {
+          case SYMBOL_WIDETAG:
+              function = symbol_function(obj);
+              if (function == NIL) {
+                  printf("Symbol 0x%08lx is undefined.\n", (long unsigned)thing);
+                  return 0;
+              }
+              break;
+          case FDEFN_WIDETAG:
+              function = FDEFN(thing)->fun;
+              if (function == NIL) {
+                  printf("Fdefn 0x%08lx is undefined.\n", (long unsigned)thing);
+                  return 0;
+              }
+              break;
+          default:
+              printf("0x%08lx is not a function pointer, symbol, "
+                     "or fdefn object.\n",
+                     (long unsigned)thing);
+              return 0;
         }
     }
-    else {
-        /* Specified nothing, search again for val. */
-        val = lastval;
-        addr = end;
-        count = lastcount;
+    else if (lowtag_of(thing) != FUN_POINTER_LOWTAG) {
+        printf("0x%08lx is not a function pointer, symbol, or fdefn object.\n",
+               (long unsigned)thing);
+        return 0;
     }
+    else
+        function = thing;
 
-    lastval = val;
-    start = end = addr;
-    lastcount = count;
-
-    printf("searching for 0x%x at %p\n", val, (void*)(uword_t)end);
-
-    while (search_for_type(val, &end, &count)) {
-        printf("found 0x%x at %p:\n", val, (void*)(uword_t)end);
-        obj = *end;
-        addr = end;
-        end += 2;
-        if (widetag_of(obj) == SIMPLE_FUN_WIDETAG) {
-            print((uword_t)addr | FUN_POINTER_LOWTAG);
-        } else if (other_immediate_lowtag_p(obj)) {
-            print((lispobj)addr | OTHER_POINTER_LOWTAG);
-        } else {
-            print((lispobj)addr);
-        } if (count == -1) {
-            return;
+    numargs = 0;
+    while (more_p(ptr)) {
+        if (numargs >= 3) {
+            printf("too many arguments (no more than 3 supported)\n");
+            return 0;
         }
+        parse_lispobj(ptr, &args[numargs++]);
     }
+
+    switch (numargs) {
+      case 0:
+          result = funcall0(function);
+          break;
+      case 1:
+          result = funcall1(function, args[0]);
+          break;
+      case 2:
+          result = funcall2(function, args[0], args[1]);
+          break;
+      case 3:
+          result = funcall3(function, args[0], args[1], args[2]);
+          break;
+      default:
+          lose("unsupported arg count made it past validity check?!");
+    }
+
+    print(result);
+    return 0;
 }
 
-/* (There used to be call_cmd() here, to call known-at-cold-init-time
- * Lisp functions from ldb, but it bitrotted and was deleted in
- * sbcl-0.7.5.1. See older CVS versions if you want to resuscitate
- * it.) */
-
-static void
-flush_cmd(char **ptr)
+static int
+flush_cmd(char __attribute__((unused)) **ptr)
 {
     flush_vars();
+    return 0;
 }
 
-static void
-quit_cmd(char **ptr)
+static int
+quit_cmd(char __attribute__((unused)) **ptr)
 {
     char buf[10];
 
@@ -316,30 +374,31 @@ quit_cmd(char **ptr)
         printf("\nUnable to read response, assuming y.\n");
         exit(1);
     }
+    return 0;
 }
 
-static void
-help_cmd(char **ptr)
+static int
+help_cmd(char __attribute__((unused)) **ptr)
 {
     struct cmd *cmd;
 
     for (cmd = supported_cmds; cmd->cmd != NULL; cmd++)
         if (cmd->help != NULL)
             printf("%s\t%s\n", cmd->cmd, cmd->help);
+    return 0;
 }
 
-static int done;
-
-static void
-exit_cmd(char **ptr)
+static int
+exit_cmd(char __attribute__((unused)) **ptr)
 {
-    done = 1;
+    return 1; // 'done' flag
 }
 
-static void
-purify_cmd(char **ptr)
+static int
+purify_cmd(char __attribute__((unused)) **ptr)
 {
     purify(NIL, NIL);
+    return 0;
 }
 
 static void
@@ -349,21 +408,20 @@ print_context(os_context_t *context)
 
     for (i = 0; i < NREGS; i++) {
         printf("%s:\t", lisp_register_names[i]);
-#if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
-        brief_print((lispobj)(*os_context_register_addr(context, i*2)));
-#else
         brief_print((lispobj)(*os_context_register_addr(context,i)));
-#endif
+
     }
 #if defined(LISP_FEATURE_DARWIN) && defined(LISP_FEATURE_PPC)
     printf("DAR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 41)));
     printf("DSISR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 42)));
 #endif
+#ifndef REG_PC
     printf("PC:\t\t  0x%08lx\n",
            (unsigned long)(*os_context_pc_addr(context)));
+#endif
 }
 
-static void
+static int
 print_context_cmd(char **ptr)
 {
     int free_ici;
@@ -374,7 +432,7 @@ print_context_cmd(char **ptr)
     if (more_p(ptr)) {
         int index;
 
-        index = parse_number(ptr);
+        if (!parse_number(ptr, &index)) return 0;
 
         if ((index >= 0) && (index < free_ici)) {
             printf("There are %d interrupt contexts.\n", free_ici);
@@ -393,25 +451,39 @@ print_context_cmd(char **ptr)
             print_context(nth_interrupt_context(free_ici - 1, thread));
         }
     }
+    return 0;
 }
 
-static void
+static int
 backtrace_cmd(char **ptr)
 {
     void lisp_backtrace(int frames);
     int n;
 
-    if (more_p(ptr))
-        n = parse_number(ptr);
-    else
+    if (more_p(ptr)) {
+        if (!parse_number(ptr, &n)) return 0;
+    } else
         n = 100;
 
     printf("Backtrace:\n");
     lisp_backtrace(n);
+    return 0;
 }
 
-static void
-catchers_cmd(char **ptr)
+static int search_cmd(char **ptr)
+{
+    char *addr;
+    if (!parse_addr(ptr, 1, &addr)) return 0;
+    lispobj *obj = search_all_gc_spaces((void*)addr);
+    if(obj)
+        printf("#x%"OBJ_FMTX"\n", compute_lispobj(obj));
+    else
+        printf("Not found\n");
+    return 0;
+}
+
+static int
+catchers_cmd(char __attribute__((unused)) **ptr)
 {
     struct catch_block *catch = (struct catch_block *)
         read_TLS(CURRENT_CATCH_BLOCK, arch_os_get_current_thread());
@@ -436,15 +508,17 @@ catchers_cmd(char **ptr)
             catch = catch->previous_catch;
         }
     }
+    return 0;
 }
 
-static void
-grab_sigs_cmd(char **ptr)
+static int
+grab_sigs_cmd(char __attribute__((unused)) **ptr)
 {
     extern void sigint_init(void);
 
     printf("Grabbing signals.\n");
     sigint_init();
+    return 0;
 }
 
 static void
@@ -468,7 +542,7 @@ sub_monitor(void)
         ldb_in_fd = fileno(ldb_in);
     }
 
-    while (!done) {
+    while (1) {
         printf("ldb> ");
         fflush(stdout);
         line = fgets(buf, sizeof(buf), ldb_in);
@@ -499,7 +573,8 @@ sub_monitor(void)
             printf("unknown command: ``%s''\n", token);
         else {
             reset_printer();
-            (*found->fn)(&ptr);
+            int done = (*found->fn)(&ptr);
+            if (done) return;
         }
     }
 }
@@ -509,7 +584,7 @@ ldb_monitor()
 {
     jmp_buf oldbuf;
 
-    bcopy(curbuf, oldbuf, sizeof(oldbuf));
+    memcpy(oldbuf, curbuf, sizeof(oldbuf));
 
     printf("Welcome to LDB, a low-level debugger for the Lisp runtime environment.\n");
 
@@ -517,9 +592,7 @@ ldb_monitor()
 
     sub_monitor();
 
-    done = 0;
-
-    bcopy(oldbuf, curbuf, sizeof(curbuf));
+    memcpy(curbuf, oldbuf, sizeof(curbuf));
 }
 
 void
