@@ -463,7 +463,7 @@ static boolean tagged_slot_p(struct layout *layout, int slot_index)
 static void print_struct(lispobj obj)
 {
     struct instance *instance = (struct instance *)native_pointer(obj);
-    unsigned int i;
+    short int i;
     char buffer[16];
     lispobj layout = instance_layout(native_pointer(obj));
     print_obj("type: ", layout);
@@ -511,7 +511,7 @@ void show_lstring(struct vector * string, int quotes, FILE *s)
   if (quotes) putc('"', s);
 }
 
-static void brief_otherptr(lispobj obj)
+static void brief_fun_or_otherptr(lispobj obj)
 {
     extern void safely_show_lstring(struct vector*, int, FILE*);
     lispobj *ptr, header;
@@ -569,7 +569,7 @@ static void print_slots(char **slots, int count, lispobj *ptr)
     }
 }
 
-static lispobj symbol_function(lispobj* symbol)
+lispobj symbol_function(lispobj* symbol)
 {
     lispobj info = ((struct symbol*)symbol)->info;
     if (listp(info))
@@ -589,16 +589,14 @@ static lispobj symbol_function(lispobj* symbol)
     return NIL;
 }
 
-static void print_otherptr(lispobj obj)
+static void print_fun_or_otherptr(lispobj obj)
 {
 #ifndef LISP_FEATURE_ALPHA
     lispobj *ptr;
     unsigned long header;
-    unsigned long length;
 #else
     u32 *ptr;
     u32 header;
-    u32 length;
 #endif
     int count, type, index;
     char buffer[16];
@@ -610,7 +608,6 @@ static void print_otherptr(lispobj obj)
     }
 
     header = *ptr++;
-    length = fixnum_value(*ptr);
     count = HeaderValue(header);
     type = header_widetag(header);
 
@@ -621,8 +618,6 @@ static void print_otherptr(lispobj obj)
         return;
     }
 
-    if (unprintable_array_types[type/8] & (1<<(type % 8)))
-        return;
     switch (type) {
     case BIGNUM_WIDETAG:
         ptr += count;
@@ -652,6 +647,14 @@ static void print_otherptr(lispobj obj)
         print_slots(symbol_slots, count & 0xFF, ptr);
         if (symbol_function(ptr-1) != NIL)
             print_obj("fun: ", symbol_function(ptr-1));
+#ifdef LISP_FEATURE_SB_THREAD
+        int tlsindex = tls_index_of((struct symbol*)(ptr-1));
+        struct thread*th = arch_os_get_current_thread();
+        if (th != 0 && tlsindex != 0) {
+            lispobj v = *(lispobj*)(tlsindex + (char*)th);
+            print_obj("tlsval: ", v);
+        }
+#endif
         break;
 
 #if N_WORD_BITS == 32
@@ -717,6 +720,8 @@ static void print_otherptr(lispobj obj)
 
     case SIMPLE_VECTOR_WIDETAG:
         NEWLINE_OR_RETURN;
+        {
+        long length = fixnum_value(*ptr);
         printf("length = %ld", length);
         ptr++;
         index = 0;
@@ -724,12 +729,13 @@ static void print_otherptr(lispobj obj)
             sprintf(buffer, "%d: ", index++);
             print_obj(buffer, *ptr++);
         }
+        }
         break;
 
     // FIXME: This case looks unreachable. print_struct() does it
     case INSTANCE_WIDETAG:
         NEWLINE_OR_RETURN;
-        count &= SHORT_HEADER_MAX_WORDS;
+        count = instance_length(header);
         printf("length = %ld", (long) count);
         index = 0;
         while (count-- > 0) {
@@ -798,34 +804,35 @@ static void print_otherptr(lispobj obj)
         print_obj("entry: ", fdefn_callee_lispobj((struct fdefn*)(ptr-1)));
         break;
 
+    // Make certain vectors printable from C for when all hell breaks lose
+    case SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG:
+        NEWLINE_OR_RETURN;
+        {
+        long length = fixnum_value(*ptr);
+        uint32_t * data = (uint32_t*)(ptr + 1);
+        long i;
+        printf("#(");
+        for (i=0; i<length; ++i) {
+            printf("%s%d", i>0?" ":"", data[i]);
+            if(i==255 && length>256) { printf(" ..."); break; }
+        }
+        printf(")");
+        }
+        break;
     default:
         NEWLINE_OR_RETURN;
-        printf("Unknown header object?");
+        if (type >= SIMPLE_ARRAY_UNSIGNED_BYTE_2_WIDETAG &&
+            type <= SIMPLE_BIT_VECTOR_WIDETAG)
+            printf("length = %ld", (long)fixnum_value(*ptr));
+        else
+            printf("Unknown header object?");
         break;
     }
 }
 
 static void print_obj(char *prefix, lispobj obj)
 {
-#ifdef LISP_FEATURE_64_BIT
-    static void (*verbose_fns[])(lispobj obj)
-        = {print_fixnum, print_otherimm, print_fixnum, print_struct,
-           print_fixnum, print_otherimm, print_fixnum, print_list,
-           print_fixnum, print_otherimm, print_fixnum, print_otherptr,
-           print_fixnum, print_otherimm, print_fixnum, print_otherptr};
-    static void (*brief_fns[])(lispobj obj)
-        = {brief_fixnum, brief_otherimm, brief_fixnum, brief_struct,
-           brief_fixnum, brief_otherimm, brief_fixnum, brief_list,
-           brief_fixnum, brief_otherimm, brief_fixnum, brief_otherptr,
-           brief_fixnum, brief_otherimm, brief_fixnum, brief_otherptr};
-#else
-    static void (*verbose_fns[])(lispobj obj)
-        = {print_fixnum, print_struct, print_otherimm, print_list,
-           print_fixnum, print_otherptr, print_otherimm, print_otherptr};
-    static void (*brief_fns[])(lispobj obj)
-        = {brief_fixnum, brief_struct, brief_otherimm, brief_list,
-           brief_fixnum, brief_otherptr, brief_otherimm, brief_otherptr};
-#endif
+#include "genesis/print.inc"
     int type = lowtag_of(obj);
     struct var *var = lookup_by_obj(obj);
     char buffer[256];
@@ -855,7 +862,7 @@ static void print_obj(char *prefix, lispobj obj)
         printf("%s0x%08lx: ", prefix, (unsigned long) obj);
         if (cur_depth < brief_depth) {
             fputs(lowtag_names[type], stdout);
-            fns = verbose_fns;
+            fns = print_fns;
         }
         else
             fns = brief_fns;
@@ -922,10 +929,7 @@ struct vector * symbol_name(lispobj * sym)
     sym = native_pointer(forwarding_pointer_value(sym));
   if (lowtag_of(((struct symbol*)sym)->name) != OTHER_POINTER_LOWTAG)
       return NULL;
-  lispobj * name = native_pointer(((struct symbol*)sym)->name);
-  if (forwarding_pointer_p(name))
-      name = native_pointer(forwarding_pointer_value(name));
-  return (struct vector*)name;
+  return VECTOR(follow_maybe_fp(((struct symbol*)sym)->name));
 }
 struct vector * classoid_name(lispobj * classoid)
 {
