@@ -44,13 +44,13 @@
 #include "interr.h"             /* for lose() */
 #include "alloc.h"
 #include "gc-internal.h"
-#include "pseudo-atomic.h"
+#include "getallocptr.h"
 #include "interrupt.h"
 #include "lispregs.h"
 
 #ifdef LISP_FEATURE_SB_THREAD
 
-#ifdef LISP_FEATURE_OPENBSD
+#if defined LISP_FEATURE_OPENBSD || defined LISP_FEATURE_FREEBSD || defined LISP_FEATURE_DRAGONFLY
 #include <pthread_np.h>
 #endif
 
@@ -61,7 +61,7 @@
 #if defined(LISP_FEATURE_WIN32) || defined(OS_THREAD_STACK)
 # define IMMEDIATE_POST_MORTEM
 #else
-static struct thread *postmortem_thread;
+static __attribute__((unused)) struct thread *postmortem_thread;
 #endif
 
 #endif
@@ -72,13 +72,12 @@ struct thread *all_threads;
 #ifdef LISP_FEATURE_SB_THREAD
 pthread_mutex_t all_threads_lock = PTHREAD_MUTEX_INITIALIZER;
 
-static pthread_mutex_t create_thread_lock = PTHREAD_MUTEX_INITIALIZER;
+static __attribute__((unused)) pthread_mutex_t create_thread_lock = PTHREAD_MUTEX_INITIALIZER;
 
 #ifdef LISP_FEATURE_GCC_TLS
 __thread struct thread *current_thread;
-__thread int is_lisp_thread;
 #else
-pthread_key_t lisp_thread = 0;
+pthread_key_t specials = 0;
 #endif
 #endif
 
@@ -127,11 +126,14 @@ thread_state(struct thread *thread)
 }
 
 void
-set_thread_state(struct thread *thread, lispobj state)
+set_thread_state(struct thread *thread, lispobj state,
+                 boolean signals_already_blocked) // for foreign thread
 {
     int i, waitcount = 0;
     sigset_t old;
-    block_blockable_signals(&old);
+    // If we've already masked the blockable signals we can avoid two syscalls here.
+    if (!signals_already_blocked)
+        block_blockable_signals(&old);
     os_sem_wait(thread->state_sem, "set_thread_state");
     if (thread->state != state) {
         if ((STATE_STOPPED==state) ||
@@ -151,7 +153,8 @@ set_thread_state(struct thread *thread, lispobj state)
         thread->state = state;
     }
     os_sem_post(thread->state_sem, "set_thread_state");
-    thread_sigmask(SIG_SETMASK, &old, NULL);
+    if (!signals_already_blocked)
+        thread_sigmask(SIG_SETMASK, &old, NULL);
 }
 
 void
@@ -188,30 +191,48 @@ wait_for_thread_state_change(struct thread *thread, lispobj state)
 #endif /* sb-safepoint */
 #endif /* sb-thread */
 
-static int
-initial_thread_trampoline(struct thread *th)
-{
-    lispobj function;
+#ifdef __linux__
+// gettid() was added in glibc 2.30 but we support older glibc
+static int sb_GetTID() { return syscall(SYS_gettid); }
+#else
+#define sb_GetTID() 0
+#endif
+
+// Only a single 'attributes' object is used if #+pauseless-threadstart.
+// This is ok because creation is synchronized by *MAKE-THREAD-LOCK*.
+#ifdef LISP_FEATURE_SB_THREAD
+pthread_attr_t new_lisp_thread_attr;
+#define init_shared_attr_object() (pthread_attr_init(&new_lisp_thread_attr)==0)
+#else
+#define init_shared_attr_object() (1)
+#endif
+struct thread *alloc_thread_struct(void*,lispobj);
+
+void create_main_lisp_thread(lispobj function) {
+    struct thread *th = alloc_thread_struct(0, NO_TLS_VALUE_MARKER_WIDETAG);
+    if (!th || arch_os_thread_init(th)==0 || !init_shared_attr_object())
+        lose("can't create initial thread");
+#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_GCC_TLS)
+    pthread_key_create(&specials, 0);
+#endif
 #if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
     lispobj *args = NULL;
 #endif
 #ifdef LISP_FEATURE_SB_THREAD
 # ifdef LISP_FEATURE_GCC_TLS
-    is_lisp_thread = 1;
+    current_thread = th;
 # else
-    pthread_setspecific(lisp_thread, (void *)1);
+    pthread_setspecific(specials, th);
 # endif
 #endif
 #if defined THREADS_USING_GCSIGNAL && \
-    (defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64 || defined LISP_FEATURE_ARM64)
+    (defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64 || defined LISP_FEATURE_ARM64 || defined LISP_FEATURE_RISCV)
     /* SIG_STOP_FOR_GC defaults to blocked on PPC? */
-    unblock_gc_signals(0,0);
+    unblock_gc_signals();
 #endif
-    function = th->no_tls_value_marker;
-    th->no_tls_value_marker = NO_TLS_VALUE_MARKER_WIDETAG;
-    if(arch_os_thread_init(th)==0) return 1;
     link_thread(th);
-    th->os_thread=thread_self();
+    th->os_kernel_tid = sb_GetTID();
+    th->os_thread = thread_self();
 #ifndef LISP_FEATURE_WIN32
     protect_control_stack_hard_guard_page(1, NULL);
 #endif
@@ -231,9 +252,9 @@ initial_thread_trampoline(struct thread *th)
      * call_into_lisp_first_time will put the new stack in the middle
      * of the current stack */
 #if !defined(LISP_FEATURE_WIN32) && (defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
-    return call_into_lisp_first_time(function,args,0);
+    call_into_lisp_first_time(function,args,0);
 #else
-    return funcall0(function);
+    funcall0(function);
 #endif
 }
 
@@ -248,7 +269,7 @@ initial_thread_trampoline(struct thread *th)
  * not anymore, now that we properly call pthread_attr_destroy before
  * freeing the stack. */
 
-static void free_thread_struct(struct thread *th)
+void free_thread_struct(struct thread *th)
 {
 #if defined(LISP_FEATURE_WIN32)
     os_invalidate_free((os_vm_address_t) th->os_address, THREAD_STRUCT_SIZE);
@@ -270,7 +291,7 @@ schedule_thread_post_mortem(struct thread *corpse)
     free_thread_struct(corpse);
 }
 
-# else
+# elif !defined LISP_FEATURE_PAUSELESS_THREADSTART
 
 static void
 perform_thread_post_mortem(struct thread *post_mortem)
@@ -346,25 +367,34 @@ schedule_thread_post_mortem(struct thread *corpse)
 static void
 init_new_thread(struct thread *th,
                 init_thread_data __attribute__((unused)) *scribble,
-                int guardp)
+                int guardp,
+                int retain_all_threads_lock)
 {
     int lock_ret;
 
 #ifdef LISP_FEATURE_GCC_TLS
-    is_lisp_thread = 1;
+    current_thread = th;
 #else
-    pthread_setspecific(lisp_thread, (void *)1);
+    pthread_setspecific(specials, th);
 #endif
     if(arch_os_thread_init(th)==0) {
         /* FIXME: handle error */
-        lose("arch_os_thread_init failed\n");
+        lose("arch_os_thread_init failed");
     }
 
     th->os_thread=thread_self();
-    if (guardp)
+
+#define GUARD_CONTROL_STACK 1
+#define GUARD_BINDING_STACK 2
+#define GUARD_ALIEN_STACK   4
+
+    if (guardp & GUARD_CONTROL_STACK)
         protect_control_stack_guard_page(1, NULL);
-    protect_binding_stack_guard_page(1, NULL);
-    protect_alien_stack_guard_page(1, NULL);
+    if (guardp & GUARD_BINDING_STACK)
+        protect_binding_stack_guard_page(1, NULL);
+    if (guardp & GUARD_ALIEN_STACK)
+        protect_alien_stack_guard_page(1, NULL);
+
     /* Since GC can only know about this thread from the all_threads
      * list and we're just adding this thread to it, there is no
      * danger of deadlocking even with SIG_STOP_FOR_GC blocked (which
@@ -375,8 +405,10 @@ init_new_thread(struct thread *th,
     lock_ret = pthread_mutex_lock(&all_threads_lock);
     gc_assert(lock_ret == 0);
     link_thread(th);
-    lock_ret = pthread_mutex_unlock(&all_threads_lock);
-    gc_assert(lock_ret == 0);
+    if (!retain_all_threads_lock) {
+        lock_ret = pthread_mutex_unlock(&all_threads_lock);
+        gc_assert(lock_ret == 0);
+    }
 
     /* Kludge: Changed the order of some steps between the safepoint/
      * non-safepoint versions of this code.  Can we unify this more?
@@ -399,6 +431,7 @@ undo_init_new_thread(struct thread *th,
      * non-safepoint versions of this code.  Can we unify this more?
      */
 #ifdef LISP_FEATURE_SB_SAFEPOINT
+
     block_blockable_signals(0);
     ensure_region_closed(&th->alloc_region, BOXED_PAGE_FLAG);
 #if defined(LISP_FEATURE_SB_SAFEPOINT_STRICTLY) && !defined(LISP_FEATURE_WIN32)
@@ -410,16 +443,25 @@ undo_init_new_thread(struct thread *th,
     unlink_thread(th);
     lock_ret = pthread_mutex_unlock(&all_threads_lock);
     gc_assert(lock_ret == 0);
+
 #else
+
     /* Block GC */
     block_blockable_signals(0);
-    set_thread_state(th, STATE_DEAD);
+    /* This state change serves to "acknowledge" any stop-the-world
+     * signal received while the STOP_FOR_GC signal is blocked */
+    set_thread_state(th, STATE_DEAD, 1);
 
     /* SIG_STOP_FOR_GC is blocked and GC might be waiting for this
-     * thread, but since we are already dead it won't wait long. */
+     * thread, but since we are either exiting lisp code as a lisp
+     * thread that is dying, or exiting lisp code to return to
+     * former status as a C thread, it won't wait long. */
     lock_ret = pthread_mutex_lock(&all_threads_lock);
     gc_assert(lock_ret == 0);
 
+    /* FIXME: this nests the free_pages_lock inside the all_threads_lock.
+     * There's no reason for that, so closing of regions should be done
+     * sooner to eliminate an ordering constraint. */
     ensure_region_closed(&th->alloc_region, BOXED_PAGE_FLAG);
 #if defined(LISP_FEATURE_SB_SAFEPOINT_STRICTLY) && !defined(LISP_FEATURE_WIN32)
     ensure_region_closed(&th->sprof_alloc_region, BOXED_PAGE_FLAG);
@@ -427,6 +469,7 @@ undo_init_new_thread(struct thread *th,
     unlink_thread(th);
     pthread_mutex_unlock(&all_threads_lock);
     gc_assert(lock_ret == 0);
+
 #endif
 
     arch_os_thread_cleanup(th);
@@ -455,6 +498,13 @@ undo_init_new_thread(struct thread *th,
     /* Undo the association of the current pthread to its `struct thread',
      * such that we can call arch_os_get_current_thread() later in this
      * thread and cleanly get back NULL. */
+    /* FIXME: this seems to be prone to causing lossage of signals because
+     * until setting current_thread to NULL, lisp_thread_p() returns 1,
+     * so maybe_resignal_to_lisp_thread() would consider this thread ok to
+     * receive a signal if delivered to a non-lisp thread. But then we just
+     * block everything and disappear. It's essentially the same problem as
+     * the GC pending signal that has to be explicitly discarded, which is
+     * the right disposition of that signal, but not signals in general */
 #ifdef LISP_FEATURE_GCC_TLS
     current_thread = NULL;
 #else
@@ -469,7 +519,56 @@ undo_init_new_thread(struct thread *th,
  */
 void* new_thread_trampoline(void* arg)
 {
+#ifdef LISP_FEATURE_PAUSELESS_THREADSTART
+
+    // 'arg' is an untagged pointer to an instance of SB-THREAD:THREAD
+    // which is currently pinned via *STARTING-THREADS*
+    // In that structure, the STARTUP-INFO slot holds a simple-vector
+    // which is pinned, and element 0 of the vector is also pinned.
+    struct thread_instance *lispthread = arg;
+    struct thread* th = (void*)lispthread->primitive_thread; // 'lispthread' is pinned
+    struct vector* startup_info = VECTOR(lispthread->startup_info);
+    gc_assert(header_widetag(startup_info->header) == SIMPLE_VECTOR_WIDETAG);
+    lispobj startfun = startup_info->data[0]; // 'startup_info' is pinned
+    gc_assert(functionp(startfun));
+    // The lisp thread instance stores the stack end for one purpose only, for
+    // SB-EXT:STACK-ALLOCATED-P to quickly check whether an object is on _any_
+    // thread's stack using only the all-threads tree and not acquiring a lock
+    // on the primitive thread. This is the exactly-calculated end.
+    lispthread->stack_end = (lispobj)th->control_stack_end;
+#if defined LISP_FEATURE_C_STACK_IS_CONTROL_STACK && !defined ADDRESS_SANITIZER
+    // ... but GC can benefit from knowing that the _effective_ end of
+    // the ambiguous root range is not as high as the accessible end.
+    // Nothing at a higher address than &arg needs to be scanned for ambiguous roots.
+    // For x86 + linux this optimization skips over about 800 words in the stack scan,
+    // and for x86-64 it skip about 550 words as observed via:
+    // fprintf(stderr, "%d non-lisp stack words\n",
+    //                 (int)((lispobj*)th->control_stack_end - (lispobj*)&arg));
+    // I'm not sure why ADDRESS_SANITIZER doesn't allow this optimization.
+    // Both of these assertions failed for me with the sanitizer enabled:
+    //    gc_assert(th->control_stack_start <= (lispobj*)&arg
+    //              && (lispobj*)&arg <= th->control_stack_end);
+    //    gc_assert(th->control_stack_start <= (lispobj*)&startup_info
+    //              && (lispobj*)&startup_info <= th->control_stack_end);
+    // It must subvert the "&" and "*" operators in a way that only it understands,
+    // while the stack pointer register is unperturbed.
+    // So why is it OK to take '&raise' in gencgc for the current thread?
+    th->control_stack_end = (lispobj*)&arg;
+#endif
+    th->os_kernel_tid = sb_GetTID();
+    init_new_thread(th, 0, 0, 0);
+    // Passing the untagged pointer ensures 2 things:
+    // - that the pinning mechanism works as designed, and not just by accident.
+    // - that the initial stack does not contain a lisp pointer after it is not needed.
+    //   (a regression test asserts that not even a THREAD instance is on the stack)
+    long result = funcall1(startfun, (lispobj)lispthread); // both pinned
+    // Close the GC region and unlink from all_threads
+    undo_init_new_thread(th, 0);
+
+#else // !PAUSELESS_THREADSTART
+
     struct thread *th = (struct thread *)arg;
+    th->os_kernel_tid = sb_GetTID();
     int result;
     init_thread_data scribble;
 
@@ -481,12 +580,15 @@ void* new_thread_trampoline(void* arg)
 
     lispobj function = th->no_tls_value_marker;
     th->no_tls_value_marker = NO_TLS_VALUE_MARKER_WIDETAG;
-    init_new_thread(th, &scribble, 1);
+    init_new_thread(th, &scribble,
+                    GUARD_CONTROL_STACK|GUARD_BINDING_STACK|GUARD_ALIEN_STACK,
+                    0);
     result = funcall0(function);
     undo_init_new_thread(th, &scribble);
 
 #ifndef OS_THREAD_STACK
     schedule_thread_post_mortem(th);
+#endif
 #endif
 
     FSHOW((stderr,"/exiting thread %p\n", thread_self()));
@@ -510,64 +612,110 @@ void* new_thread_trampoline_switch_stack(void* arg) {
 }
 #endif
 
-static struct thread *create_thread_struct(lispobj);
+static struct thread* recyclebin_threads;
+static pthread_mutex_t recyclebin_lock = PTHREAD_MUTEX_INITIALIZER;
+static struct thread* get_recyclebin_item()
+{
+    struct thread* result = 0;
+    int rc;
+    rc = pthread_mutex_lock(&recyclebin_lock);
+    gc_assert(!rc);
+    if (recyclebin_threads) {
+        result = recyclebin_threads;
+        recyclebin_threads = result->next;
+    }
+    pthread_mutex_unlock(&recyclebin_lock);
+    return result ? result->os_address : 0;
+}
+static void put_recyclebin_item(struct thread* th)
+{
+    int rc;
+    rc = pthread_mutex_lock(&recyclebin_lock);
+    gc_assert(!rc);
+    th->next = recyclebin_threads;
+    recyclebin_threads = th;
+    pthread_mutex_unlock(&recyclebin_lock);
+}
+void empty_thread_recyclebin()
+{
+    if (!recyclebin_threads) return;
+    sigset_t old;
+    block_deferrable_signals(&old);
+    int rc = pthread_mutex_trylock(&recyclebin_lock);
+    if (!rc) { // no big deal if already locked (recursive GC?)
+        struct thread* this = recyclebin_threads;
+        while (this) {
+            struct thread* next = this->next;
+            free_thread_struct(this);
+            this = next;
+        }
+        recyclebin_threads = 0;
+        pthread_mutex_unlock(&recyclebin_lock);
+    }
+    thread_sigmask(SIG_SETMASK, &old, 0);
+}
 
 void
 attach_os_thread(init_thread_data *scribble)
 {
-    os_thread_t os = pthread_self();
-    odxprint(misc, "attach_os_thread: attaching to %p", os);
-
-    struct thread *th = create_thread_struct(NO_TLS_VALUE_MARKER_WIDETAG);
     block_deferrable_signals(&scribble->oldset);
+    void* recycled_memory = get_recyclebin_item();
+    struct thread *th = alloc_thread_struct(recycled_memory,
+                                            NO_TLS_VALUE_MARKER_WIDETAG);
 
 #ifndef LISP_FEATURE_SB_SAFEPOINT
     /* new-lisp-thread-trampoline doesn't like when the GC signal is blocked */
     /* FIXME: could be done using a single call to pthread_sigmask
        together with locking the deferrable signals above. */
-    unblock_gc_signals(0, 0);
+    unblock_gc_signals();
 #endif
 
+    th->os_kernel_tid = sb_GetTID();
+    th->os_thread = pthread_self();
+
 #if !defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
     /* On windows, arch_os_thread_init will take care of finding the
      * stack. */
     void *stack_addr;
     size_t stack_size;
-#ifdef LISP_FEATURE_OPENBSD
+# ifdef LISP_FEATURE_OPENBSD
     stack_t stack;
-    pthread_stackseg_np(os, &stack);
+    pthread_stackseg_np(th->os_thread, &stack);
     stack_size = stack.ss_size;
     stack_addr = (void*)((size_t)stack.ss_sp - stack_size);
-#elif defined LISP_FEATURE_SUNOS
-  stack_t stack;
-  thr_stksegment(&stack);
-  stack_size = stack.ss_size;
-  stack_addr = (void*)((size_t)stack.ss_sp - stack_size);
-#elif defined(LISP_FEATURE_DARWIN)
-    stack_size = pthread_get_stacksize_np(os);
-    stack_addr = (char*)pthread_get_stackaddr_np(os) - stack_size;
-#else
+# elif defined LISP_FEATURE_SUNOS
+    stack_t stack;
+    thr_stksegment(&stack);
+    stack_size = stack.ss_size;
+    stack_addr = (void*)((size_t)stack.ss_sp - stack_size);
+# elif defined(LISP_FEATURE_DARWIN)
+    stack_size = pthread_get_stacksize_np(th->os_thread);
+    stack_addr = (char*)pthread_get_stackaddr_np(th->os_thread) - stack_size;
+# else
     pthread_attr_t attr;
-#ifdef LISP_FEATURE_FREEBSD
-    pthread_attr_get_np(os, &attr);
-#else
+#   if defined LISP_FEATURE_FREEBSD || defined LISP_FEATURE_DRAGONFLY
+    pthread_attr_get_np(th->os_thread, &attr);
+#   else
     int pthread_getattr_np(pthread_t, pthread_attr_t *);
-    pthread_getattr_np(os, &attr);
-#endif
+    pthread_getattr_np(th->os_thread, &attr);
+#   endif
     pthread_attr_getstack(&attr, &stack_addr, &stack_size);
     pthread_attr_destroy(&attr);
-#endif
-
+# endif
     th->control_stack_start = stack_addr;
     th->control_stack_end = (void *) (((uintptr_t) stack_addr) + stack_size);
 #endif
 
-    init_new_thread(th, scribble, 0);
-
-    uword_t stacksize
-        = (uword_t) th->control_stack_end - (uword_t) th->control_stack_start;
-    odxprint(misc, "attach_os_thread: attached %p as %p (0x%lx bytes stack)",
-             os, th, (long) stacksize);
+#ifdef LISP_FEATURE_SB_SAFEPOINT
+    const int retain_lock = 0;
+#else
+    const int retain_lock = 1;
+#endif
+    init_new_thread(th, scribble,
+                    /* recycled memory already had mprotect() done,
+                     * so avoid 2 syscalls when possible */
+                    recycled_memory ? 0 : GUARD_BINDING_STACK|GUARD_ALIEN_STACK,
+                    retain_lock);
 }
 
 void
@@ -578,14 +726,31 @@ detach_os_thread(init_thread_data *scribble)
 
     undo_init_new_thread(th, scribble);
 
-    odxprint(misc, "deattach_os_thread: detached");
-#ifdef LISP_FEATURE_GCC_TLS
-    is_lisp_thread = 0;
-#else
-    pthread_setspecific(lisp_thread, (void *)0);
+    /* We have to clear a STOP_FOR_GC signal if pending. Consider:
+     *  - on entry to undo_init_new_thread, we block all signals
+     *  - simultaneously some other thread decides that it needs to initiate a GC
+     *  - that thread observes that this thread exists in all_threads and sends
+     *    STOP_FOR_GC, so it becomes pending but undeliverable in this thread
+     *  - immediately after blocking signals, we change state to DEAD,
+     *    which allows the GCing thread to ignore this thread
+     *    (it sees the state change criterion as having been satisfied)
+     *  - the GCing thread releases the all_threads lock
+     *  - this thread acquires the lock and removes itself from all_threads,
+     *    and indicates that it is no longer a lisp thread
+     *  - but STOP_FOR_GC is pending because it was in the blocked set.
+     * Bad things happen unless we clear the pending GC signal.
+     */
+#ifndef LISP_FEATURE_SB_SAFEPOINT
+    sigset_t pending;
+    sigpending(&pending);
+    if (sigismember(&pending, SIG_STOP_FOR_GC)) {
+        int sig, rc;
+        rc = sigwait(&gc_sigset, &sig);
+        gc_assert(rc == 0 && sig == SIG_STOP_FOR_GC);
+    }
 #endif
+    put_recyclebin_item(th);
     thread_sigmask(SIG_SETMASK, &scribble->oldset, 0);
-    free_thread_struct(th);
 }
 
 #if defined(LISP_FEATURE_X86_64) && !defined(LISP_FEATURE_WIN32)
@@ -640,6 +805,14 @@ callback_wrapper_trampoline(
 #endif
     }
 }
+
+// Balance out the mutex_lock in attach_os_thread()
+void release_all_threads_lock()
+{
+    if (pthread_mutex_unlock(&all_threads_lock))
+        lose("ENTER-FOREIGN-CALLBACK bug");
+}
+
 #endif /* LISP_FEATURE_SB_THREAD */
 
 /* this is called from any other thread to create the new one, and
@@ -685,8 +858,8 @@ callback_wrapper_trampoline(
  * On sb-safepoint builds one page before the thread base is used for the foreign calls safepoint.
  */
 
-static struct thread *
-create_thread_struct(lispobj start_routine) {
+struct thread *
+alloc_thread_struct(void* spaces, lispobj start_routine) {
 #if defined(LISP_FEATURE_SB_THREAD) || defined(LISP_FEATURE_WIN32)
     unsigned int i;
 #endif
@@ -699,9 +872,18 @@ create_thread_struct(lispobj start_routine) {
      * on the alignment passed from os_validate, since that might
      * assume the current (e.g. 4k) pagesize, while we calculate with
      * the biggest (e.g. 64k) pagesize allowed by the ABI. */
-    void *spaces = os_validate(IS_THREAD_STRUCT, NULL, THREAD_STRUCT_SIZE);
-    if(!spaces)
-        return NULL;
+    boolean zeroize_stack = 0;
+    if (spaces) {
+        // If reusing memory from a previously exited thread, start by removing
+        // some old junk from the stack. This is imperfect since we only clear a little
+        // at the top, but doing so enables diagnosing some garbage-retention issues
+        // using a fine-toothed comb. It would not be possible at all to diagnose
+        // if any newly started thread could refer a dead thread's heap objects.
+        zeroize_stack = 1;
+    } else {
+        spaces = os_validate(MOVABLE|IS_THREAD_STRUCT, NULL, THREAD_STRUCT_SIZE);
+        if (!spaces) return NULL;
+    }
     /* Aligning up is safe as THREAD_STRUCT_SIZE has
      * THREAD_ALIGNMENT_BYTES padding. */
     char *aligned_spaces = PTR_ALIGN_UP(spaces, THREAD_ALIGNMENT_BYTES);
@@ -719,9 +901,13 @@ create_thread_struct(lispobj start_routine) {
     lispobj* tls = (lispobj*)th;
     for(i = 0; i < (unsigned int)(dynamic_values_bytes/N_WORD_BYTES); i++)
         tls[i] = NO_TLS_VALUE_MARKER_WIDETAG;
+    th->lisp_thread = 0; // force it to be always-thread-local, of course
     th->tls_size = dynamic_values_bytes;
 #endif
     uword_t* __attribute__((__unused__)) constants = (uword_t*)th;
+#if defined LISP_FEATURE_X86_64 && defined LISP_FEATURE_LINUX
+    constants[THREAD_MSAN_XOR_CONSTANT_SLOT] = 0x500000000000;
+#endif
 #ifdef LISP_FEATURE_GENCGC
 #ifdef THREAD_VARYOBJ_CARD_MARKS_SLOT
     extern unsigned int* varyobj_page_touched_bits;
@@ -739,6 +925,33 @@ create_thread_struct(lispobj start_routine) {
     th->binding_stack_start=
         (lispobj*)((char*)th->control_stack_start+thread_control_stack_size);
     th->control_stack_end = th->binding_stack_start;
+
+    if (zeroize_stack) {
+#if GENCGC_IS_PRECISE
+    /* Clear the entire control stack. Without this I was able to induce a GC failure
+     * in a test which hammered on thread creation for hours. The control stack is
+     * scavenged before the heap, so a stale word could point to the start (or middle)
+     * of an object using a bad lowtag, for whatever object formerly was there.
+     * Then a wrong transport function would be called and (if it worked at all) would
+     * place a wrongly tagged FP into a word that might not be the base of an object.
+     * Assume for simplicity (as is true) that stacks grow upward if GENCGC_IS_PRECISE.
+     * This could just call scrub_thread_control_stack but the comment there says that
+     * it's a lame algorithm and only mostly right - it stops after (1<<12) words
+     * and checks if the next is nonzero, looping again if it isn't.
+     * There's no reason not to be exactly right here instead of probably right */
+        memset((char*)th->control_stack_start, 0,
+               // take off 2 pages because of the soft and hard guard pages
+               thread_control_stack_size - 2*os_vm_page_size);
+#else
+    /* This is a little wasteful of cycles to pre-zero the pthread overhead (which in glibc
+     * resides at the highest stack addresses) comprising about 5kb, below which is the lisp
+     * stack. We don't need to zeroize above the lisp stack end, but we don't know exactly
+     * where that will be.  Zeroizing more than necessary is conservative, and helps ensure
+     * that garbage retention from reused stacks does not pose a huge problem. */
+        memset((char*)th->control_stack_end - 16384, 0, 16384);
+#endif
+    }
+
     th->control_stack_guard_page_protected = T;
     th->alien_stack_start=
         (lispobj*)((char*)th->binding_stack_start+BINDING_STACK_SIZE);
@@ -843,27 +1056,6 @@ create_thread_struct(lispobj start_routine) {
 #  define INITIALIZE_TLS(sym,val) SYMBOL(sym)->value = val
 #endif
 #include "genesis/thread-init.inc"
-#ifdef LISP_FEATURE_SB_THREAD
-    /* Each initial binding is a cons whose car is a symbol evaluated as if
-     * by SYMBOL-GLOBAL-VALUE (unsafely), and whose cdr is the target symbol.
-     * In particular, we will obligingly assign the unbound-marker.
-     * An atom implies NIL for the value. */
-    struct vector* tls_init = VECTOR(SYMBOL(THREAD_INITIAL_BINDINGS)->value);
-    for (i = 0; i < tls_init->length; i += make_fixnum(1)) {
-        lispobj binding = tls_init->data[fixnum_value(i)];
-        lispobj value = NIL;
-        if (listp(binding)) {
-            lispobj val_form = CONS(binding)->car;
-            value = fixnump(val_form) ? val_form : SYMBOL(val_form)->value;
-            binding = CONS(binding)->cdr;
-        }
-        struct symbol* sym = SYMBOL(binding);
-        write_TLS_index(tls_index_of(sym), value, th, sym);
-    }
-    /* If a symbol assigned above had a TLS index of 0, then it'll
-     * mess up th->no_tls_value_marker. Fail now if that happened. */
-    gc_assert(th->no_tls_value_marker == NO_TLS_VALUE_MARKER_WIDETAG);
-#endif
     th->no_tls_value_marker = start_routine;
 
 #if defined(LISP_FEATURE_WIN32)
@@ -877,16 +1069,6 @@ create_thread_struct(lispobj start_routine) {
     return th;
 }
 
-void create_initial_thread(lispobj initial_function) {
-    struct thread *th = create_thread_struct(initial_function);
-#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_GCC_TLS)
-    pthread_key_create(&lisp_thread, 0);
-#endif
-    if(th) {
-        initial_thread_trampoline(th); /* no return */
-    } else lose("can't create initial thread\n");
-}
-
 #ifdef LISP_FEATURE_SB_THREAD
 
 #ifndef __USE_XOPEN2K
@@ -894,6 +1076,8 @@ extern int pthread_attr_setstack (pthread_attr_t *__attr, void *__stackaddr,
                                   size_t __stacksize);
 #endif
 
+#ifndef LISP_FEATURE_PAUSELESS_THREADSTART
+
 /* Call pthread_create() and return 1 for success, 0 for failure */
 boolean create_os_thread(struct thread *th,os_thread_t *kid_tid)
 {
@@ -911,7 +1095,7 @@ boolean create_os_thread(struct thread *th,os_thread_t *kid_tid)
     pthread_attr_t attr;
     if (pthread_attr_init(&attr) == 0) {
 
-    /* See perform_thread_post_mortem for at least one reason why this lock is neccessary */
+    /* See perform_thread_post_mortem for at least one reason why this lock is necessary */
         retcode = pthread_mutex_lock(&create_thread_lock);
         gc_assert(retcode == 0);
 
@@ -923,13 +1107,19 @@ boolean create_os_thread(struct thread *th,os_thread_t *kid_tid)
             (retcode = pthread_create(kid_tid, &attr, new_thread_trampoline_switch_stack, th))
 #else
 
-# if defined(LISP_FEATURE_WIN32)
+# ifdef LISP_FEATURE_WIN32
             pthread_attr_setstacksize(&attr, thread_control_stack_size) ||
 # elif defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
             pthread_attr_setstack(&attr, th->control_stack_start, thread_control_stack_size) ||
 # else
             pthread_attr_setstack(&attr, th->alien_stack_start, ALIEN_STACK_SIZE) ||
 # endif
+# ifdef LISP_FEATURE_NETBSD
+            /* Even though the manpage says pthread_attr_setstack
+               would override the guard page, it's no longer true. */
+            pthread_attr_setguardsize(&attr, 0) ||
+# endif
+
             (retcode = pthread_create(kid_tid, &attr, new_thread_trampoline, th))
 #endif
             ) {
@@ -952,13 +1142,13 @@ os_thread_t create_thread(lispobj start_routine) {
 
     /* Must defend against async unwinds. */
     if (read_TLS(INTERRUPTS_ENABLED, thread) != NIL)
-        lose("create_thread is not safe when interrupts are enabled.\n");
+        lose("create_thread is not safe when interrupts are enabled.");
 
     /* Assuming that a fresh thread struct has no lisp objects in it,
      * linking it to all_threads can be left to the thread itself
      * without fear of gc lossage. 'start_routine' violates this
      * assumption and must stay pinned until the child starts up. */
-    th = create_thread_struct(start_routine);
+    th = alloc_thread_struct(0, start_routine);
     if (th && !create_os_thread(th, &kid_tid)) {
         free_thread_struct(th);
         kid_tid = 0;
@@ -966,6 +1156,8 @@ os_thread_t create_thread(lispobj start_routine) {
     return kid_tid;
 }
 
+#endif
+
 /* stopping the world is a two-stage process.  From this thread we signal
  * all the others with SIG_STOP_FOR_GC.  The handler for this signal does
  * the usual pseudo-atomic checks (we don't want to stop a thread while
@@ -983,15 +1175,22 @@ void gc_stop_the_world()
 {
     struct thread *p,*th=arch_os_get_current_thread();
     int status, lock_ret;
+    // There is no create_thread lock if pauseless start is enabled.
+    // And wouldn't the right fix for FreeBSD be to inhibit the stop-for-GC signal
+    // rather than acquire a lock? And why exactly there a deadlock ?
+    // That we don't endeavor to find these things out leads to never-ending
+    // accretion of dubious code that we'll not know when to remove.
+#ifndef LISP_FEATURE_PAUSELESS_THREADSTART
     /* KLUDGE: Stopping the thread during pthread_create() causes deadlock
      * on FreeBSD. */
     FSHOW_SIGNAL((stderr,"/gc_stop_the_world:waiting on create_thread_lock\n"));
     lock_ret = pthread_mutex_lock(&create_thread_lock);
     gc_assert(lock_ret == 0);
     FSHOW_SIGNAL((stderr,"/gc_stop_the_world:got create_thread_lock\n"));
+#endif
     FSHOW_SIGNAL((stderr,"/gc_stop_the_world:waiting on lock\n"));
     /* keep threads from starting while the world is stopped. */
-    lock_ret = pthread_mutex_lock(&all_threads_lock);      \
+    lock_ret = pthread_mutex_lock(&all_threads_lock);
     gc_assert(lock_ret == 0);
 
     FSHOW_SIGNAL((stderr,"/gc_stop_the_world:got lock\n"));
@@ -1037,9 +1236,8 @@ void gc_start_the_world()
     struct thread *p,*th=arch_os_get_current_thread();
     int lock_ret;
     /* if a resumed thread creates a new thread before we're done with
-     * this loop, the new thread will get consed on the front of
-     * all_threads, but it won't have been stopped so won't need
-     * restarting */
+     * this loop, the new thread will be suspended waiting to acquire
+     * the all_threads lock */
     FSHOW_SIGNAL((stderr,"/gc_start_the_world:begin\n"));
     for(p=all_threads;p;p=p->next) {
         gc_assert(p->os_thread!=0);
@@ -1052,16 +1250,17 @@ void gc_start_the_world()
                 }
                 FSHOW_SIGNAL((stderr, "/gc_start_the_world: resuming %lu\n",
                               p->os_thread));
-                set_thread_state(p, STATE_RUNNING);
+                set_thread_state(p, STATE_RUNNING, 0);
             }
         }
     }
 
     lock_ret = pthread_mutex_unlock(&all_threads_lock);
     gc_assert(lock_ret == 0);
+#ifndef LISP_FEATURE_PAUSELESS_THREADSTART
     lock_ret = pthread_mutex_unlock(&create_thread_lock);
     gc_assert(lock_ret == 0);
-
+#endif
 
     FSHOW_SIGNAL((stderr,"/gc_start_the_world:end\n"));
 }
@@ -1079,6 +1278,14 @@ thread_yield()
 #endif
 }
 
+// The code is extremely trashy and a good candidate for removal/cleanup
+// but I don't understand this feature combination well enough to do so.
+// The part of it we need to keep is the GC state interaction, though
+// as can plainly be seen from the "good" case of SB-THREAD::INTERRUPT-THREAD
+// there is a perfectly fine way of detecting validity of a pthread ID,
+// and even the link below to Ulrich Drepper's blog post says exactly what
+// to do. So why does this have anything to do with the all_threads_lock???
+#if defined LISP_FEATURE_SB_THRUPTION || defined LISP_FEATURE_SB_SAFEPOINT
 int
 wake_thread(os_thread_t os_thread)
 {
@@ -1146,7 +1353,7 @@ kill_safely(os_thread_t os_thread, int signal)
             if (thread->os_thread == os_thread) {
                 int status = pthread_kill(os_thread, signal);
                 if (status)
-                    lose("kill_safely: pthread_kill failed with %d\n", status);
+                    lose("kill_safely: pthread_kill failed with %d", status);
 #if defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_SB_THRUPTION)
                 wake_thread_win32(thread);
 #endif
@@ -1164,7 +1371,7 @@ kill_safely(os_thread_t os_thread, int signal)
 #else
         int status;
         if (os_thread != 0)
-            lose("kill_safely: who do you want to kill? %d?\n", os_thread);
+            lose("kill_safely: who do you want to kill? %d?", os_thread);
         /* Dubious (as in don't know why it works) workaround for the
          * signal sometimes not being generated on darwin. */
 #ifdef LISP_FEATURE_DARWIN
@@ -1180,9 +1387,10 @@ kill_safely(os_thread_t os_thread, int signal)
         if (status == 0) {
             return 0;
         } else {
-            lose("cannot raise signal %d, %d %s\n",
+            lose("cannot raise signal %d, %d %s",
                  signal, status, strerror(errno));
         }
 #endif
     }
 }
+#endif
