@@ -1,4 +1,3 @@
-
 #include "sbcl.h"
 #include "arch.h"
 #include "runtime.h"
@@ -15,9 +14,11 @@
 #include "genesis/layout.h"
 #include "genesis/package.h"
 #include "genesis/vector.h"
-#include "pseudo-atomic.h" // for get_alloc_pointer()
+#include "getallocptr.h" // for get_alloc_pointer()
 #include "search.h"
 #include "genesis/avlnode.h"
+#include "genesis/sap.h"
+#include "genesis/thread-instance.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -29,6 +30,25 @@
 #include <sys/resource.h> // for getrusage()
 #endif
 
+#ifndef TRACEROOT_USE_ABSL_HASHMAP
+#define TRACEROOT_USE_ABSL_HASHMAP 0
+#endif
+
+#if TRACEROOT_USE_ABSL_HASHMAP
+extern void* new_absl_hashmap(int);
+extern void absl_hashmap_destroy(void*);
+extern unsigned absl_hashmap_get(void*,lispobj);
+extern unsigned* absl_hashmap_get_ref(void*,lispobj);
+extern int absl_hashmap_size(void*,int*);
+typedef void* inverted_heap_t;
+#define inverted_heap_get(graph, key) absl_hashmap_get(graph, key)
+#define inverted_heap_get_ref(graph, key) absl_hashmap_get_ref(graph, key)
+#else
+typedef struct hopscotch_table* inverted_heap_t;
+#define inverted_heap_get(graph, key) hopscotch_get(graph, key, 0)
+#define inverted_heap_get_ref(graph, key) hopscotch_get_ref(graph, key)
+#endif
+
 int heap_trace_verbose = 0;
 
 extern generation_index_t gencgc_oldest_gen_to_gc;
@@ -60,9 +80,10 @@ struct scan_state {
     long n_immediates;
     long n_pointers;
     int record_ptrs;
-    // A hashtable mapping each object to a list of objects pointing to it
-    struct hopscotch_table inverted_heap;
+    // A hashmap from object to list of objects pointing to it
+    struct hopscotch_table* inverted_heap;
     struct scratchpad scratchpad;
+    int keep_leaves;
 };
 
 static int traceroot_gen_of(lispobj obj) {
@@ -115,7 +136,7 @@ static void add_to_layer(lispobj* obj, int wordindex,
 
 /// If 'obj' is a simple-fun, return its code component,
 /// otherwise return obj directly.
-static lispobj canonical_obj(lispobj obj)
+static inline lispobj canonical_obj(lispobj obj)
 {
     if (functionp(obj) && widetag_of(FUNCTION(obj)) == SIMPLE_FUN_WIDETAG)
         return fun_code_tagged(FUNCTION(obj));
@@ -131,9 +152,9 @@ static int find_ref(lispobj* source, lispobj target)
     lispobj layout, bitmap;
     int scan_limit, i;
 
-    lispobj header = *source;
-    if (is_cons_half(header)) {
-        check_ptr(0, header);
+    lispobj word = *source;
+    if (!is_header(word)) {
+        check_ptr(0, source[0]);
         check_ptr(1, source[1]);
         return -1;
     }
@@ -153,11 +174,11 @@ static int find_ref(lispobj* source, lispobj target)
         bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
         for(i=1; i<scan_limit; ++i)
             if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(i, source[i]);
-        // FIXME: check CUSTOM_GC_SCAVENGE in the header
+        // FIXME: check lockfree_list_node_p() also
         return -1;
 #if FUN_SELF_FIXNUM_TAGGED
     case CLOSURE_WIDETAG:
-        check_ptr(1, ((struct closure*)source)->fun - FUN_RAW_ADDR_OFFSET);
+        check_ptr(1, fun_taggedptr_from_self(((struct closure*)source)->fun));
         break;
 #endif
     case CODE_HEADER_WIDETAG:
@@ -174,7 +195,6 @@ static int find_ref(lispobj* source, lispobj target)
 #undef check_ptr
 
 enum ref_kind { HEAP, CONTROL_STACK, BINDING_STACK, TLS };
-char *ref_kind_name[4] = {"heap","C stack","bindings","TLS"};
 
 static inline int interestingp(lispobj ptr, struct hopscotch_table* targets)
 {
@@ -256,13 +276,14 @@ deduce_thread(void (*context_scanner)(), uword_t pointer, char** pc)
 
 /* KNOWN BUG: stack reference to pinned large object or immobile object
  * won't be found in pins hashtable */
-static lispobj examine_stacks(struct hopscotch_table* targets,
-                              void (*context_scanner)(),
-                              int n_pins, lispobj* pins,
-                              enum ref_kind *root_kind,
-                              struct thread** root_thread,
-                              char** thread_pc,
-                              lispobj *tls_index)
+/* Also: should take 'struct lisp_thread**' instead of 'struct thread**' */
+static lispobj examine_threads(struct hopscotch_table* targets,
+                               void (*context_scanner)(),
+                               int n_pins, lispobj* pins,
+                               enum ref_kind *root_kind,
+                               struct thread** root_thread,
+                               char** thread_pc,
+                               lispobj *tls_index)
 {
     struct thread *th;
 
@@ -271,7 +292,7 @@ static lispobj examine_stacks(struct hopscotch_table* targets,
 #ifdef LISP_FEATURE_SB_THREAD
         // Examine thread-local storage
         *root_kind = TLS;
-        where = (lispobj*)(th+1);
+        where = &th->lisp_thread;
         end   = (lispobj*)((char*)th + SymbolValue(FREE_TLS_INDEX,0));
         for( ; where < end ; ++where)
             if (interestingp(*where, targets)) {
@@ -408,18 +429,6 @@ static inline lispobj decode_pointer(uint32_t encoding)
         return encoding; // Literal pointer
 }
 
-static struct simple_fun* simple_fun_from_pc(char* pc)
-{
-    struct code* code = (struct code*)component_ptr_from_pc(pc);
-    if (!code) return 0;
-    struct simple_fun* prev_fun = 0;
-    for_each_simple_fun(i, fun, code, 1, {
-        if (pc < (char*)fun) break;
-        prev_fun = fun;
-    })
-    return prev_fun;
-}
-
 static void maybe_show_object_name(lispobj obj, FILE* stream)
 {
     extern void safely_show_lstring(lispobj* string, int quotes, FILE *s);
@@ -451,15 +460,36 @@ static boolean root_p(lispobj ptr, int criterion)
                            : gencgc_oldest_gen_to_gc));
 }
 
+// Make a cons (or 'liststar2')
+static lispobj mkcons(lispobj car, lispobj cdr)
+{
+    struct cons *cons = (struct cons*)
+        gc_general_alloc(sizeof(struct cons), BOXED_PAGE_FLAG);
+    cons->car = car;
+    cons->cdr = cdr;
+    return make_lispobj(cons, LIST_POINTER_LOWTAG);
+}
+static lispobj liststar3(lispobj x, lispobj y, lispobj z) {
+  return mkcons(x, mkcons(y, z));
+}
+static lispobj make_sap(char* value)
+{
+    struct sap *sap = (struct sap*)
+        gc_general_alloc(sizeof(struct sap), BOXED_PAGE_FLAG);
+    sap->header = (1<<N_WIDETAG_BITS) | SAP_WIDETAG;
+    sap->pointer = value;
+    return make_lispobj(sap, OTHER_POINTER_LOWTAG);
+}
+
 /// Find any shortest path to 'object' starting at a tenured object or a thread stack.
-/// Return 1 if a path was found, 0 if not.
-static int trace1(lispobj object,
-                  struct hopscotch_table* targets,
-                  struct hopscotch_table* visited,
-                  struct hopscotch_table* inverted_heap,
-                  struct scratchpad* scratchpad,
-                  int n_pins, lispobj* pins, void (*context_scanner)(),
-                   int criterion)
+/// Return the path as a list, or 0 if a path could not be found.
+static lispobj trace1(lispobj object,
+                      struct hopscotch_table* targets,
+                      struct hopscotch_table* visited,
+                      inverted_heap_t graph,
+                      struct scratchpad* scratchpad,
+                      int n_pins, lispobj* pins, void (*context_scanner)(),
+                      int criterion)
 {
     struct node* anchor = 0;
     lispobj thread_ref;
@@ -474,9 +504,9 @@ static int trace1(lispobj object,
     int layer_capacity = 0;
 
     hopscotch_put(targets, object, 1);
-    while ((thread_ref = examine_stacks(targets, context_scanner, n_pins, pins,
-                                        &root_kind, &root_thread, &thread_pc,
-                                        &tls_index)) == 0) {
+    while ((thread_ref = examine_threads(targets, context_scanner, n_pins, pins,
+                                         &root_kind, &root_thread, &thread_pc,
+                                         &tls_index)) == 0) {
         // TODO: preallocate layers to avoid possibility of malloc deadlock
         struct layer* layer = (struct layer*)malloc(sizeof (struct layer));
         layer->nodes   = 0;
@@ -487,7 +517,7 @@ static int trace1(lispobj object,
         if (heap_trace_verbose)
             printf("Next layer: Looking for %d object(s)\n", targets->count);
         for_each_hopscotch_key(i, target, (*targets)) {
-            uint32_t list = hopscotch_get(inverted_heap, target, 0);
+            uint32_t list = inverted_heap_get(graph, target);
             if (heap_trace_verbose>1) {
                 uint32_t list1 = list;
                 fprintf(stderr, "target=%p srcs=", (void*)target);
@@ -513,8 +543,10 @@ static int trace1(lispobj object,
                     continue;
                 int wordindex = find_ref((lispobj*)ptr, target);
                 if (wordindex == -1) {
-                    fprintf(stderr, "Strange: no ref from %p to %p\n",
-                            (void*)ptr, (void*)target);
+                    if (heap_trace_verbose) {
+                        fprintf(stderr, "Strange: no ref from %p to %p\n",
+                                (void*)ptr, (void*)target);
+                    }
                     continue;
                 }
                 hopscotch_insert(visited, ptr, 1);
@@ -522,21 +554,25 @@ static int trace1(lispobj object,
                              top_layer, &layer_capacity);
                 // Stop if the object at 'ptr' is tenured.
                 if (root_p(ptr, criterion)) {
-                    fprintf(stderr, "Stopping at %p: tenured\n", (void*)ptr);
+                    if (heap_trace_verbose) {
+                        fprintf(stderr, "Stopping at %p: tenured\n", (void*)ptr);
+                    }
                     anchor = &top_layer->nodes[top_layer->count-1];
                 }
             }
         }
         if (!top_layer->count) {
-            fprintf(stderr, "Failure tracing from %p. Current targets:\n", (void*)object);
-            for_each_hopscotch_key(i, target, (*targets)) {
-                fprintf(stderr, "%p", (void*)target);
-                fprintf(stderr, "(g%d,", traceroot_gen_of(target));
-                fputs(classify_obj(target), stderr);
-                maybe_show_object_name(target, stderr);
-                fprintf(stderr,") ");
+            if (heap_trace_verbose) {
+                fprintf(stderr, "Failure tracing from %p. Current targets:\n", (void*)object);
+                for_each_hopscotch_key(i, target, (*targets)) {
+                    fprintf(stderr, "%p", (void*)target);
+                    fprintf(stderr, "(g%d,", traceroot_gen_of(target));
+                    fputs(classify_obj(target), stderr);
+                    maybe_show_object_name(target, stderr);
+                    fprintf(stderr,") ");
+                }
+                putc('\n', stderr);
             }
-            putc('\n', stderr);
             free_graph(top_layer);
             return 0;
         }
@@ -555,12 +591,10 @@ static int trace1(lispobj object,
         }
     }
 
-    FILE *file = stdout;
+    lispobj path_node = NIL;
     if (thread_ref) {
-        struct vector* lisp_thread_name(os_thread_t os_thread);
-        extern void show_lstring(struct vector*, int, FILE*);
-        struct vector* thread_name;
   #if 0
+        char *ref_kind_name[4] = {"heap","C stack","bindings","TLS"};
         fprintf(stderr,
                 "%s pointed to by %s: %p\n",
                 top_layer ? "Indirectly" : "Directly",
@@ -574,55 +608,29 @@ static int trace1(lispobj object,
             anchor = find_node(top_layer, thread_ref);
             gc_assert(anchor);
         }
-        putc('{', file);
-        if (!root_thread)
-            fprintf(file, "(unknown-thread)");
-        else if ((thread_name = lisp_thread_name(root_thread->os_thread)) != 0)
-            show_lstring(thread_name, 1, file);
-        else
-            fprintf(file, "thread=%p", root_thread);
-        fprintf(file, ":%s:", ref_kind_name[root_kind]);
         if (root_kind==BINDING_STACK || root_kind==TLS) {
-#ifdef LISP_FEATURE_SB_THREAD
-            lispobj* symbol = (lispobj*)
-              lisp_symbol_from_tls_index(tls_index);
-#else
-            lispobj* symbol = native_pointer(tls_index);
-#endif
-            if (symbol)
-                show_lstring(symbol_name(symbol), 0, file);
-            else
-                fprintf(file, "%"OBJ_FMTX, tls_index);
+            path_node = liststar3(make_fixnum(root_kind),
+                                  (lispobj)root_thread,
+                                  tls_index);
         } else {
-            struct simple_fun* fun = simple_fun_from_pc(thread_pc);
-            if (fun) {
-                fprintf(file, "fun=%p", (void*)make_lispobj(fun, FUN_POINTER_LOWTAG));
-                lispobj fun_name = simple_fun_name(fun);
-                if (is_lisp_pointer(fun_name) &&
-                    widetag_of(native_pointer(fun_name)) == SYMBOL_WIDETAG) {
-                    fprintf(file, "=");
-                    show_lstring(VECTOR(SYMBOL(fun_name)->name), 0, file);
-                }
-            } else if (thread_pc)
-                fprintf(file, "pc=%p", thread_pc);
+            path_node = liststar3(make_fixnum(root_kind),
+                                  (lispobj)root_thread,
+                                  make_sap(thread_pc));
         }
-        fprintf(file, "}->");
     } else { // Stopped at (pseudo)static object
-        fprintf(stderr, "Anchor object is @ %p. word[%d]\n",
-                native_pointer(anchor->object), anchor->wordindex);
+        if (heap_trace_verbose) {
+            fprintf(stderr, "Anchor object is @ %p. word[%d]\n",
+                    native_pointer(anchor->object), anchor->wordindex);
+        }
+        path_node = liststar3(0, 0, 0);
     }
 
+    lispobj path = mkcons(path_node, NIL);
     target = thread_ref;
     while (top_layer) {
         struct node next = *anchor;
         lispobj ptr = next.object;
-        if (ptr <= STATIC_SPACE_END)
-            fprintf(file, "(static,");
-        else
-            fprintf(file, "(g%d,", traceroot_gen_of(ptr));
-        fputs(classify_obj(ptr), file);
-        maybe_show_object_name(ptr, file);
-        fprintf(file, ")#x%"OBJ_FMTX"[%d]->", ptr, next.wordindex);
+        path = mkcons(mkcons(ptr, make_fixnum(next.wordindex)), path);
         target = native_pointer(ptr)[next.wordindex];
         // Special-case a few combinations of <type,wordindex>
         switch (next.wordindex) {
@@ -633,7 +641,7 @@ static int trace1(lispobj object,
 #if FUN_SELF_FIXNUM_TAGGED
         case 1:
             if (functionp(ptr) && widetag_of(native_pointer(ptr)) == CLOSURE_WIDETAG)
-                target -= FUN_RAW_ADDR_OFFSET;
+                target = fun_taggedptr_from_self(target);
             break;
 #endif
         case 3:
@@ -654,38 +662,43 @@ static int trace1(lispobj object,
             gc_assert(object == target);
         }
     }
-    fprintf(file, "#x%"OBJ_FMTX".\n", target);
-    fflush(file);
-    return 1;
+    return path;
 }
 
-static void record_ptr(lispobj* source, lispobj target,
-                       struct scan_state* ss)
+// Add 'source' to the list of objects keyed by 'target' in the inverted heap.
+// Note that 'source' has no lowtag, and 'target' does.
+// Pointer compression is used: the linked list of source objects
+// is built using offsets into the scratchpad rather than absolute addresses.
+// Return 1 if and only if 'target' was actually added to the graph.
+static boolean record_ptr(lispobj* source, lispobj target,
+                          struct scan_state* ss)
 {
-    // Add 'source' to the list of objects keyed by 'target' in the inverted heap.
-    // Note that 'source' has no lowtag, and 'target' does.
-    // Pointer compression occurs here as well: the linked list of source objects
-    // is built using offsets into the scratchpad rather than absolute addresses.
+    if (!ss->keep_leaves) { // expected case
+        if (!listp(target) &&
+            leaf_obj_widetag_p(widetag_of(native_pointer(target)))) return 0;
+    }
     target = canonical_obj(target);
     uint32_t* new_cell = (uint32_t*)ss->scratchpad.free;
     uint32_t* next = new_cell + 2;
     gc_assert((char*)next <= ss->scratchpad.end);
     ss->scratchpad.free = (char*)next;
+    if (ss->scratchpad.free > ss->scratchpad.end) lose("undersized scratchpad");
     new_cell[0] = encode_pointer((lispobj)source);
-    new_cell[1] = hopscotch_get(&ss->inverted_heap, target, 0);
-    hopscotch_put(&ss->inverted_heap, target,
-                  (sword_t)((char*)new_cell - ss->scratchpad.base));
+    uint32_t* valref = inverted_heap_get_ref(ss->inverted_heap, target);
+    new_cell[1] = *valref;
+    *valref = (uint32_t)((char*)new_cell - ss->scratchpad.base);
+    return 1;
 }
 
-#define relevant_ptr_p(x) find_page_index(x)>=0||immobile_space_p((lispobj)x)
+#define relevant_ptr_p(x) (find_page_index((void*)(x))>=0||immobile_space_p((lispobj)(x)))
 
-#define check_ptr(ptr) { \
-    ++n_scanned_words; \
-    if (!is_lisp_pointer(ptr)) ++n_immediates; \
-    else if (relevant_ptr_p((void*)(ptr))) { \
-      ++n_pointers; \
-      if (record_ptrs) record_ptr(where,ptr,ss); \
-    }}
+#define COUNT_POINTER(x) { ++n_scanned_words; \
+      if (!is_lisp_pointer(x)) ++n_immediates; \
+      else if (relevant_ptr_p(x)) ++n_pointers; }
+
+#define check_ptr(x) { \
+    if (count_only) COUNT_POINTER(x) \
+    else if (is_lisp_pointer(x) && relevant_ptr_p(x)) record_ptr(where,x,ss); }
 
 static uword_t build_refs(lispobj* where, lispobj* end,
                           struct scan_state* ss)
@@ -695,17 +708,17 @@ static uword_t build_refs(lispobj* where, lispobj* end,
     uword_t n_objects = 0, n_scanned_words = 0,
             n_immediates = 0, n_pointers = 0;
 
-    boolean record_ptrs = ss->record_ptrs;
+    boolean count_only = !ss->record_ptrs;
     for ( ; where < end ; where += nwords ) {
         ++n_objects;
-        lispobj header = *where;
-        if (is_cons_half(header)) {
+        lispobj word = *where;
+        if (!is_header(word)) {
             nwords = 2;
-            check_ptr(header);
+            check_ptr(where[0]);
             check_ptr(where[1]);
             continue;
         }
-        int widetag = widetag_of(where);
+        int widetag = header_widetag(word);
         nwords = scan_limit = sizetab[widetag](where);
         switch (widetag) {
         case INSTANCE_WIDETAG:
@@ -717,15 +730,15 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             check_ptr(layout);
             // Partially initialized instance can't have nonzero words yet
             bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
-            // FIXME: check CUSTOM_GC_SCAVENGE in the header
             // If no raw slots, just scan without use of the bitmap.
+            // FIXME: check lockfree_list_node_p() also
             if (bitmap == make_fixnum(-1)) break;
             for(i=1; i<scan_limit; ++i)
                 if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(where[i]);
             continue;
 #if FUN_SELF_FIXNUM_TAGGED
         case CLOSURE_WIDETAG:
-            check_ptr(((struct closure*)where)->fun - FUN_RAW_ADDR_OFFSET);
+            check_ptr(fun_taggedptr_from_self(((struct closure*)where)->fun));
             break;
 #endif
         case CODE_HEADER_WIDETAG:
@@ -740,18 +753,24 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             // a path from the root only if it does not involve whichever
             // object is definitely weak. This fails on weak key-OR-value
             // tables since we can't decide whether to allow the entry.
-            if (is_vector_subtype(*where, VectorValidHashing)) {
-                lispobj lhash_table = where[2];
+            // Disregard the address-sensitivity bit when checking whether
+            // this is a weak hash-table vector.
+            if ((vector_subtype(*where) & ~subtype_VectorAddrHashing)
+                == subtype_VectorHashing + subtype_VectorWeak) {
+                lispobj* data = where + 2;
+                int kv_vector_length = fixnum_value(where[1]);
+                lispobj lhash_table = data[kv_vector_length-1];
                 gc_assert(instancep(lhash_table));
                 struct hash_table* hash_table =
                   (struct hash_table *)native_pointer(lhash_table);
                 int weakness = hashtable_weakness(hash_table);
-                if (hashtable_weakp(hash_table) &&
-                    (weakness == 1 || weakness == 2)) { // 1=key, 2=value
-                    // Skip the first 4 words which are:
-                    //  header, length, vector->table backpointer, rehash-p
-                    int start = (weakness == 1) ? 5 : 4;
-                    for(i=start; i<scan_limit; i+=2) check_ptr(where[i]);
+                int high_water_mark = fixnum_value(data[0]);
+                // FIXME: this is supposed to do what for weak-AND or weak-OR ?
+                if (weakness == 1 || weakness == 2) { // 1=key, 2=value
+                    // Skip the first 2 data words
+                    int start = (weakness == 1) ? 3 : 2;
+                    int end = 2 * (high_water_mark + 1);
+                    for(i=start; i<end; i+=2) check_ptr(data[i]);
                     continue;
                 }
             }
@@ -759,8 +778,8 @@ static uword_t build_refs(lispobj* where, lispobj* end,
                 continue;
             break;
         default:
-            if (!(other_immediate_lowtag_p(widetag) && lowtag_for_widetag[widetag>>2]))
-              lose("Unknown widetag %x\n", widetag);
+            if (!(other_immediate_lowtag_p(widetag) && LOWTAG_FOR_WIDETAG(widetag)))
+              lose("Unknown widetag %x", widetag);
             // Skip irrelevant objects.
             if (leaf_obj_widetag_p(widetag) ||
                 (widetag == WEAK_POINTER_WIDETAG) || /* do not follow! */
@@ -769,9 +788,26 @@ static uword_t build_refs(lispobj* where, lispobj* end,
                 (widetag == RATIO_WIDETAG))
                 continue;
         }
-        for(i=1; i<scan_limit; ++i) check_ptr(where[i]);
+        if (widetag == SIMPLE_VECTOR_WIDETAG && ss->record_ptrs) {
+            // Try to eliminate some duplicate edges in the reversed graph.
+            // This is only a heuristic and will not eliminate all duplicate edges.
+            // It helps for vectors which get initialized like #(#:FOO #:FOO ...)
+            // by storing only one backpointer to #:FOO.
+            // It is not particularly helpful for other objects.
+            lispobj prev_interesting_ptr = 0;
+            for(i=1; i<scan_limit; ++i) {
+                lispobj pointer = where[i];
+                if (is_lisp_pointer(pointer)
+                    && relevant_ptr_p(pointer)
+                    && pointer != prev_interesting_ptr
+                    && record_ptr(where,pointer,ss))
+                    prev_interesting_ptr = pointer;
+            }
+        } else {
+            for(i=1; i<scan_limit; ++i) check_ptr(where[i]);
+        }
     }
-    if (!record_ptrs) { // just count them
+    if (count_only) {
         ss->n_objects += n_objects;
         ss->n_scanned_words += n_scanned_words;
         ss->n_immediates += n_immediates;
@@ -781,36 +817,54 @@ static uword_t build_refs(lispobj* where, lispobj* end,
 }
 #undef check_ptr
 
+#define show_tally(b,a) /* "before" and "after" */   \
+  if(heap_trace_verbose && !ss->record_ptrs) \
+    fprintf(stderr, "%ld objs, %ld ptrs, %ld immediates\n", \
+            a->n_objects - b.n_objects, a->n_pointers - b.n_pointers, \
+            (a->n_scanned_words - a->n_pointers) - (b.n_scanned_words - b.n_pointers))
+
 static void scan_spaces(struct scan_state* ss)
 {
-    build_refs((lispobj*)STATIC_SPACE_START, static_space_free_pointer, ss);
+    struct scan_state old = *ss;
+    build_refs((lispobj*)STATIC_SPACE_OBJECTS_START, static_space_free_pointer, ss);
+    show_tally(old, ss);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    build_refs((lispobj*)FIXEDOBJ_SPACE_START, fixedobj_free_pointer, ss);
-    build_refs((lispobj*)VARYOBJ_SPACE_START, varyobj_free_pointer, ss);
+    old = *ss; build_refs((lispobj*)FIXEDOBJ_SPACE_START, fixedobj_free_pointer, ss);
+    show_tally(old, ss);
+    old = *ss; build_refs((lispobj*)VARYOBJ_SPACE_START, varyobj_free_pointer, ss);
+    show_tally(old, ss);
 #endif
+    old = *ss;
     walk_generation((uword_t(*)(lispobj*,lispobj*,uword_t))build_refs,
                     -1, (uword_t)ss);
+    show_tally(old, ss);
 }
 
 #define HASH_FUNCTION HOPSCOTCH_HASH_FUN_MIX
 
-static void compute_heap_inverse(struct hopscotch_table* inverted_heap,
-                                 struct scratchpad* scratchpad)
+static void* compute_heap_inverse(boolean keep_leaves,
+                                  struct scratchpad* scratchpad)
 {
     struct scan_state ss;
     memset(&ss, 0, sizeof ss);
-    fprintf(stderr, "Pass 1: Counting heap objects... ");
+    ss.keep_leaves = keep_leaves;
+    if (heap_trace_verbose) fprintf(stderr, "Pass 1: Counting heap objects...\n");
     scan_spaces(&ss);
-    fprintf(stderr, "%ld objs, %ld ptrs, %ld immediates\n",
-            ss.n_objects, ss.n_pointers,
-            ss.n_scanned_words - ss.n_pointers);
     // Guess at the initial size of ~ .5 million objects.
     int size = 1<<19; // flsl(tot_n_objects); this would work if you have it
     while (ss.n_objects > size) size <<= 1;
-    fprintf(stderr, "Pass 2: Inverting heap. Initial size=%d objects\n", size);
-    hopscotch_create(&ss.inverted_heap, HASH_FUNCTION,
+    if (heap_trace_verbose) {
+        fprintf(stderr, "Pass 2: Inverting heap. Initial size=%d objects\n", size);
+    }
+#if TRACEROOT_USE_ABSL_HASHMAP
+    void* inverted_heap = new_absl_hashmap(size);
+    ss.inverted_heap = inverted_heap;
+#else
+    ss.inverted_heap = malloc(sizeof(struct hopscotch_table));
+    hopscotch_create(ss.inverted_heap, HASH_FUNCTION,
                      4, // XXX: half the word size if 64-bit
                      size /* initial size */, 0 /* default hop range */);
+#endif
     // Add one pointer due to inability to use the first
     // two words of the scratchpad.
     uword_t scratchpad_min_size = (1 + ss.n_pointers) * 2 * sizeof (uint32_t);
@@ -820,39 +874,68 @@ static void compute_heap_inverse(struct hopscotch_table* inverted_heap,
     gc_assert(ss.scratchpad.base);
     ss.scratchpad.free = ss.scratchpad.base + 2 * sizeof(uint32_t);
     ss.scratchpad.end  = ss.scratchpad.base + scratchpad_size;
-    fprintf(stderr, "Scratchpad: %lu bytes\n", (long unsigned)scratchpad_size);
+    if (heap_trace_verbose) {
+        fprintf(stderr, "Scratchpad: %lu bytes\n", (long unsigned)scratchpad_size);
+    }
 #if HAVE_GETRUSAGE
     struct rusage before, after;
     getrusage(RUSAGE_SELF, &before);
 #endif
     ss.record_ptrs = 1;
     scan_spaces(&ss);
-    *inverted_heap = ss.inverted_heap;
     *scratchpad = ss.scratchpad;
 #if HAVE_GETRUSAGE
     getrusage(RUSAGE_SELF, &after);
     // We're done building the necessary structure. Show some memory stats.
-#define timediff(b,a,field) \
-    ((a.field.tv_sec-b.field.tv_sec)*1000000+(a.field.tv_usec-b.field.tv_usec))
-    fprintf(stderr,
-            "Inverted heap: ct=%d, cap=%d, LF=%f ET=%ld+%ld sys+usr\n",
-            inverted_heap->count,
-            1+hopscotch_max_key_index(*inverted_heap),
-            100*(float)inverted_heap->count / (1+hopscotch_max_key_index(*inverted_heap)),
-            timediff(before, after, ru_stime),
-            timediff(before, after, ru_utime));
+    if (heap_trace_verbose) {
+        int count;
+        int capacity;
+#if TRACEROOT_USE_ABSL_HASHMAP
+        count = absl_hashmap_size(ss.inverted_heap, &capacity);
+#else
+        count = ss.inverted_heap->count;
+        capacity = 1+hopscotch_max_key_index(*ss.inverted_heap);
 #endif
+#define timediff(b,a,field) \
+        ((a.field.tv_sec-b.field.tv_sec)*1000000+(a.field.tv_usec-b.field.tv_usec))
+        float stime = timediff(before, after, ru_stime);
+        float utime = timediff(before, after, ru_utime);
+        fprintf(stderr,
+                "Inverted heap: ct=%d, cap=%d, LF=%f ET=%f+%f sys+usr=%f\n",
+                count, capacity, 100*(float)count / (capacity>0?capacity:1),
+                stime/1000000, utime/1000000, (stime+utime)/1000000);
+    }
+#endif
+    return ss.inverted_heap;
 };
 
+/* Return true if the user wants to find a leaf object.
+ * If not, then we can omit all leaf objects from the inverted heap
+ * because no leaf object can point to anything */
+static boolean finding_leaf_p(lispobj weak_pointers)
+{
+    do {
+        lispobj car = CONS(weak_pointers)->car;
+        lispobj value = ((struct weak_pointer*)native_pointer(car))->value;
+        weak_pointers = CONS(weak_pointers)->cdr;
+        if (is_lisp_pointer(value)
+            && !listp(value)
+            && leaf_obj_widetag_p(widetag_of(native_pointer(value)))) return 1;
+    } while (weak_pointers != NIL);
+    return 0; // this is the expected (and optimal) case
+}
+
 /* Find any shortest path from a thread or tenured object
  * to each of the specified objects.
  */
 static int trace_paths(void (*context_scanner)(),
-                       lispobj weak_pointers, int n_pins, lispobj* pins,
+                       lispobj weak_pointers, // list of inputs
+                       lispobj paths, // vector of outputs
+                       int n_pins, lispobj* pins,
                        int criterion)
 {
     int i;
-    struct hopscotch_table inverted_heap;
+    void* inverted_heap;
     struct scratchpad scratchpad;
     // A hashset of all objects in the reverse reachability graph so far
     struct hopscotch_table visited;  // *Without* lowtag
@@ -866,10 +949,13 @@ static int trace_paths(void (*context_scanner)(),
           fprintf(stderr, " %p%s", (void*)pins[i],
                   ((i%8)==7||i==n_pins-1)?"\n":"");
     }
-    compute_heap_inverse(&inverted_heap, &scratchpad);
+    inverted_heap = compute_heap_inverse(finding_leaf_p(weak_pointers),
+                                         &scratchpad);
     hopscotch_create(&visited, HASH_FUNCTION, 0, 32, 0);
     hopscotch_create(&targets, HASH_FUNCTION, 0, 32, 0);
+    i = 0;
     do {
+        gc_assert(make_fixnum(i) <= VECTOR(paths)->length);
         lispobj car = CONS(weak_pointers)->car;
         lispobj value = ((struct weak_pointer*)native_pointer(car))->value;
         weak_pointers = CONS(weak_pointers)->cdr;
@@ -878,14 +964,22 @@ static int trace_paths(void (*context_scanner)(),
                 fprintf(stderr, "Target=%p (%s)\n", (void*)value, classify_obj(value));
             hopscotch_reset(&visited);
             hopscotch_reset(&targets);
-            n_found += trace1(canonical_obj(value),
-                              &targets, &visited,
-                              &inverted_heap, &scratchpad,
-                              n_pins, pins, context_scanner, criterion);
+            lispobj path = trace1(canonical_obj(value),
+                                  &targets, &visited,
+                                  inverted_heap, &scratchpad,
+                                  n_pins, pins, context_scanner, criterion);
+            if ((VECTOR(paths)->data[i] = path) != 0) ++n_found;
         }
+        ++i;
     } while (weak_pointers != NIL);
+    ensure_region_closed(&boxed_region, BOXED_PAGE_FLAG);
     os_invalidate(scratchpad.base, scratchpad.end-scratchpad.base);
-    hopscotch_destroy(&inverted_heap);
+#if TRACEROOT_USE_ABSL_HASHMAP
+    absl_hashmap_destroy(inverted_heap);
+#else
+    hopscotch_destroy(inverted_heap);
+    free(inverted_heap);
+#endif
     hopscotch_destroy(&visited);
     hopscotch_destroy(&targets);
     return n_found;
@@ -898,9 +992,12 @@ int gc_prove_liveness(void(*context_scanner)(),
                       int n_pins, uword_t* pins,
                       int criterion)
 {
-    int n_watched = 0, n_live = 0, n_bad = 0, n_imm = 0;
+    int n_watched = 0, n_live = 0, n_bad = 0, n_imm = 0, n_paths = 0;
+    lispobj input = CONS(objects)->car,
+            output = CONS(objects)->cdr,
+            paths = CONS(output)->cdr;
     lispobj list;
-    for (list = objects ; list != NIL && listp(list) ; list = CONS(list)->cdr) {
+    for (list = input ; list != NIL && listp(list) ; list = CONS(list)->cdr) {
         ++n_watched;
         lispobj car = CONS(list)->car;
         if ((lowtag_of(car) != OTHER_POINTER_LOWTAG ||
@@ -914,17 +1011,26 @@ int gc_prove_liveness(void(*context_scanner)(),
         else if (wpval != UNBOUND_MARKER_WIDETAG)
             ++n_imm;
     }
-    if (!listp(list) || n_bad) {
-        fprintf(stderr, "; Bad value in liveness tracker\n");
+    if (!listp(list) || n_bad || lowtag_of(paths) != OTHER_POINTER_LOWTAG
+        || widetag_of(native_pointer(paths)) != SIMPLE_VECTOR_WIDETAG) {
+        if (heap_trace_verbose) {
+            fprintf(stderr, "; Bad value in liveness tracker\n");
+        }
+        CONS(output)->car = make_fixnum(-1);
         return -1;
     }
-    fprintf(stderr, "; Liveness tracking: %d/%d live watched objects",
-            n_live, n_watched);
-    if (n_imm)
-        fprintf(stderr, " (ignored %d non-pointers)", n_imm);
-    putc('\n', stderr);
-    if (!n_live)
+    if (heap_trace_verbose) {
+        fprintf(stderr, "; Liveness tracking: %d/%d live watched objects",
+                n_live, n_watched);
+
+        if (n_imm)
+            fprintf(stderr, " (ignored %d non-pointers)", n_imm);
+        putc('\n', stderr);
+    }
+    if (!n_live) {
+        CONS(output)->car = make_fixnum(0);
         return 0;
+    }
     // Put back lowtags on pinned objects, since wipe_nonpinned_words() removed
     // them. But first test whether lowtags were already repaired
     // in case prove_liveness() is called after gc_prove_liveness().
@@ -934,7 +1040,10 @@ int gc_prove_liveness(void(*context_scanner)(),
             pins[i] = compute_lispobj((lispobj*)pins[i]);
         }
     }
-    return trace_paths(context_scanner, objects, n_pins, (lispobj*)pins, criterion);
+    n_paths = trace_paths(context_scanner, input, paths,
+                          n_pins, (lispobj*)pins, criterion);
+    CONS(output)->car = make_fixnum(n_paths);
+    return n_paths;
 }
 
 /* This should be called inside WITHOUT-GCING so that the set
@@ -947,37 +1056,19 @@ int prove_liveness(lispobj objects, int criterion)
     return gc_prove_liveness(0, objects, gc_n_stack_pins, pinned_objects.keys, criterion);
 }
 
-// These are slot offsets in (DEFSTRUCT THREAD),
-// not the C structure defined in genesis/thread.h
-#define LISP_THREAD_NAME_SLOT INSTANCE_DATA_START+0
-#define LISP_THREAD_OS_THREAD_SLOT INSTANCE_DATA_START+3
-
-/* Perform exhaustive search on *ALL-THREADS* looking for a specific thread.
- * Efficiency is not a concern.
- */
-static struct instance* find_thread(os_thread_t os_thread, lispobj tree)
-{
-    if (tree == NIL) return 0;
-    struct avlnode* node = (struct avlnode*)native_pointer(tree);
-    struct instance* lisp_thread = (struct instance*)native_pointer(node->data);
-    if ((os_thread_t)lisp_thread->slots[LISP_THREAD_OS_THREAD_SLOT] == os_thread)
-        return lisp_thread;
-    if ((lisp_thread = find_thread(os_thread, node->left)) != NULL)
-        return lisp_thread;
-    if ((lisp_thread = find_thread(os_thread, node->right)) != NULL)
-        return lisp_thread;
-    return 0;
-}
-
+#ifdef LISP_FEATURE_SB_THREAD
+/// Given a pthread identifier, return the lisp thread's string name.
+/// Caveat emptor: This doesn't acquire the all_threads_lock.
+/// Ironically perhaps, 'traceroot' no longer uses this, but it's nice to have.
 struct vector* lisp_thread_name(os_thread_t os_thread)
 {
-    static unsigned int hint;
-    lispobj* sym = find_symbol("*ALL-THREADS*", find_package("SB-THREAD"), &hint);
-    if (sym) {
-        struct instance* lisp_thread =
-            find_thread(os_thread, ((struct symbol*)sym)->value);
-        if (lisp_thread)
-            return VECTOR(lisp_thread->slots[LISP_THREAD_NAME_SLOT]);
+    struct thread* th;
+    for_each_thread (th) {
+        if (pthread_equal(os_thread, th->os_thread)) {
+            struct thread_instance *lispthread = (void*)native_pointer(th->lisp_thread);
+            return VECTOR(lispthread->name);
+        }
     }
     return 0;
 }
+#endif
