@@ -25,7 +25,7 @@
 #include "interr.h"
 #include "breakpoint.h"
 #include "thread.h"
-#include "pseudo-atomic.h"
+#include "getallocptr.h"
 #include "unaligned.h"
 #include "search.h"
 #include "globals.h" // for asm_routines_start,end
@@ -35,7 +35,8 @@
 #include "forwarding-ptr.h"
 #include "core.h"
 
-#define INT3_INST 0xcc
+#define INT3_INST 0xCC
+#define INTO_INST 0xCE
 #define UD2_INST 0x0b0f
 #define BREAKPOINT_WIDTH 1
 
@@ -243,7 +244,11 @@ unsigned int
 arch_install_breakpoint(void *pc)
 {
     unsigned int result = *(unsigned int*)pc;
+#ifdef LISP_FEATURE_INT4_BREAKPOINTS
+    *(char*)pc = INTO_INST;
+#else
     *(char*)pc = INT3_INST;
+#endif
     *((char*)pc+1) = trap_Breakpoint;           /* Lisp trap code */
     return result;
 }
@@ -387,21 +392,21 @@ void
 sigill_handler(int __attribute__((unused)) signal,
                siginfo_t __attribute__((unused)) *siginfo,
                os_context_t *context) {
+    unsigned char* pc = (void*)*os_context_pc_addr(context);
 #ifndef LISP_FEATURE_MACH_EXCEPTION_HANDLER
-    if (*((unsigned short *)*os_context_pc_addr(context)) == UD2_INST) {
+    if (*(unsigned short *)pc == UD2_INST) {
         *os_context_pc_addr(context) += 2;
         return sigtrap_handler(signal, siginfo, context);
     }
-# ifdef LISP_FEATURE_X86_64 // handle INTO
-    if (*((unsigned char *)*os_context_pc_addr(context)) == 0xCE) {
+    // Interrupt if overflow (INTO) raises SIGILL in 64-bit mode
+    if (*(unsigned char *)pc == INTO_INST) {
         *os_context_pc_addr(context) += 1;
         return sigtrap_handler(signal, siginfo, context);
     }
-# endif
 #endif
 
     fake_foreign_function_call(context);
-    lose("Unhandled SIGILL at %p.", (void*)*os_context_pc_addr(context));
+    lose("Unhandled SIGILL at %p.", pc);
 }
 
 #ifdef X86_64_SIGFPE_FIXUP
@@ -484,10 +489,10 @@ arch_install_interrupt_handlers()
     SHOW("returning from arch_install_interrupt_handlers()");
 }
 
-#ifdef LISP_FEATURE_LINKAGE_TABLE
 void
-arch_write_linkage_table_entry(char *reloc_addr, void *target_addr, int datap)
+arch_write_linkage_table_entry(int index, void *target_addr, int datap)
 {
+    char *reloc_addr = (char*)LINKAGE_TABLE_SPACE_START + index * LINKAGE_TABLE_ENTRY_SIZE;
     if (datap) {
         *(uword_t *)reloc_addr = (uword_t)target_addr;
         return;
@@ -498,7 +503,6 @@ arch_write_linkage_table_entry(char *reloc_addr, void *target_addr, int datap)
     reloc_addr[6] = 0x66; reloc_addr[7] = 0x90; /* 2-byte NOP */
     *(void**)(reloc_addr+8) = target_addr;
 }
-#endif
 
 /* These setup and check *both* the sse2 and x87 FPUs. While lisp code
    only uses the sse2 FPU, other code (such as libc) may use the x87 FPU.
