@@ -101,17 +101,15 @@ int arch_os_thread_init(struct thread *thread) {
   __asm__ __volatile__ ("mov %0, %%fs" : : "r"(sel));
 
   thread->tls_cookie = sel;
-  pthread_setspecific(specials,thread);
-
 #endif
 
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
     /* Signal handlers are run on the control stack, so if it is exhausted
      * we had better use an alternate stack for whatever signal tells us
      * we've exhausted it */
-    sigstack.ss_sp=((void *) thread)+dynamic_values_bytes;
-    sigstack.ss_flags=0;
-    sigstack.ss_size = 32*SIGSTKSZ;
+    sigstack.ss_sp    = calc_altstack_base(thread);
+    sigstack.ss_flags = 0;
+    sigstack.ss_size  = calc_altstack_size(thread);
     sigaltstack(&sigstack,0);
 #endif
      return 1;                   /* success */
@@ -131,7 +129,7 @@ int arch_os_thread_cleanup(struct thread *thread) {
 
     thread_mutex_lock(&modify_ldt_lock);
     if (sysi86(SI86DSCR, &delete) < 0) {
-      lose("Couldn't remove segment\n");
+      lose("Couldn't remove segment");
     }
     thread_mutex_unlock(&modify_ldt_lock);
 #endif
