@@ -821,3 +821,36 @@
     (((1- (ash 1 32))) 32)
     (((1- (ash 1 48))) 48)
     (((1- (ash 1 54))) 54)))
+
+(with-test (:name :complex-rational-eql)
+  ;; ensure no funny business with constant sharing so that we're forced
+  ;; to do the most general form or EQL on two (complex rational)s
+  (eql (read-from-string "#c(1/2 1/3)") (read-from-string "#c(1/2 1/3)")))
+
+(with-test (:name :gcd-derive-type)
+  (checked-compile-and-assert
+   ()
+   '(lambda (a)
+     (declare (type (integer -142 -29) a))
+     (eql (gcd (setq a -29) a) 1))
+   ((-33) nil)))
+
+;;; Test that LOGIOR and LOGXOR on PPC64 can correctly perform the constant 2nd-arg
+;;; vop variant on a 32-bit immediate using op + op-shifted instructions.
+;;; (Probably we should refactor large parts of the test suite by the CPU
+;;; architecture so that you can really see clearly the areas of concern
+;;; for each backend, but I'm not about to embark on that now)
+(with-test (:name :ppc64-logxor-32-bit-const)
+  (let ((f (compile nil
+                    '(lambda (x)
+                      (declare (fixnum x))
+                      ;; The asm code went wrong only if the result was not in the same
+                      ;; register as the source, so make sure that X is live throughout
+                      ;; the test so that each result is in a different register.
+                      (let ((a (logxor x #x9516A7))
+                            (b (logior x #x2531b4))
+                            (c (ash x -1)))
+                        (list a b c x))))))
+    (let ((result (funcall f 0)))
+      (assert (equal result
+                     '(#x9516A7 #x2531b4 0 0))))))
