@@ -101,8 +101,7 @@
 ;;; (I don't know what platforms it passes on, but at least these two it does)
 (with-test (:name :repeatably-count-allocated-objects
             :skipped-on (or (not (or :x86 :x86-64))
-                            :interpreter)
-            :fails-on (not :sb-thread))
+                            :interpreter))
   (let ((a (make-array 5)))
     (dotimes (i (length a))
       (setf (aref a i) (count-dynamic-space-objects))
@@ -168,9 +167,15 @@
 
 #+immobile-space
 (with-test (:name :generation-of-fdefn)
+  ;; generation-of broke when fdefns stopped storing a generation in word 0
   (assert (= (sb-kernel:generation-of (sb-kernel::find-fdefn 'car))
              sb-vm:+pseudo-static-generation+)))
 
+(with-test (:name :static-fdefn-space)
+  (sb-int:dovector (name sb-vm:+static-fdefns+)
+    (assert (eq (sb-ext:heap-allocated-p (sb-kernel::find-fdefn name))
+                (or #+immobile-code :immobile :static)))))
+
 ;;; SB-EXT:GENERATION-* accessors returned bogus values for generation > 0
 (with-test (:name :bug-529014 :skipped-on (not :gencgc))
   (loop for i from 0 to sb-vm:+pseudo-static-generation+
@@ -214,12 +219,13 @@
                   :skipped-on :cheneygc)
   ;; The interpreters (both sb-eval and sb-fasteval) special-case
   ;; WITH-PINNED-OBJECTS as a "special form", because the x86oid
-  ;; version of WITH-PINNED-OBJECTS uses black magic that isn't
-  ;; supportable outside of the compiler.  The non-x86oid versions of
-  ;; WITH-PINNED-OBJECTS don't use black magic, but are overridden
-  ;; anyway.  But the special-case logic was, historically broken, and
-  ;; this affects all gencgc targets (cheneygc isn't affected because
-  ;; cheneygc WITH-PINNED-OBJECTS devolves to WITHOUT-GC>ING).
+  ;; version of WITH-PINNED-OBJECTS uses special functionality that
+  ;; isn't supportable outside of the compiler.  The non-x86oid
+  ;; versions of WITH-PINNED-OBJECTS don't use this special
+  ;; functionality, but are overridden anyway.  But the special-case
+  ;; logic was, historically broken, and this affects all gencgc
+  ;; targets (cheneygc isn't affected because cheneygc
+  ;; WITH-PINNED-OBJECTS devolves to WITHOUT-GCING).
   ;;
   ;; Our basic approach is to allocate some kind of object and stuff
   ;; it where it doesn't need to be on the control stack.  We then pin
@@ -317,7 +323,6 @@
              (assert (logbitp 4 type)))))))
    :all))
 
-#+64-bit ; code-serialno not defined unless 64-bit
 (with-test (:name :unique-code-serialno)
   (let ((a (make-array 100000 :element-type 'bit :initial-element 0)))
     (sb-vm:map-allocated-objects
@@ -419,3 +424,14 @@
     (sb-thread:join-thread worker-thread)
     (setq working nil)
     (sb-thread:join-thread gc-thread)))
+
+(with-test (:name :no-conses-on-large-object-pages)
+  (let* ((fun (checked-compile '(lambda (&rest params) params)))
+         (list (make-list #+gencgc (/ sb-vm:large-object-size
+                                      (sb-vm::primitive-object-size '(1))
+                                      1/2)
+                          #-gencgc 16384))
+         (rest (apply fun list)))
+    (sb-sys:with-pinned-objects (rest)
+      (sb-ext:gc :full t)
+      (assert (and (equal list rest) t)))))
