@@ -14,6 +14,7 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
+
 ;;;; Pathname accessors
 
 (with-test (:name (pathname :accessors :stream-not-associated-to-file type-error))
@@ -444,7 +445,8 @@
 ;;; ensure print-read consistency (or print-not-readable-error) on
 ;;; pathnames:
 (with-test (:name :print/read-consistency)
-  (dolist (p (list (make-pathname :name "foo" :type "txt" :version :newest)
+  (dolist (p (list (make-pathname :name ".")
+                   (make-pathname :name "foo" :type "txt" :version :newest)
                    (make-pathname :name "foo" :type "txt" :version 1)
                    (make-pathname :name "foo" :type ".txt")
                    (make-pathname :name "foo." :type "txt")
@@ -452,30 +454,32 @@
                    (make-pathname :name "^" :type "txt")
                    (make-pathname :name "foo*" :type "txt")
                    (make-pathname :name "foo[" :type "txt")
+                   (make-pathname :name "foo.bar" :type "txt")
+                   (make-pathname :name "foo" :type "txt.gz")
+                   (make-pathname :name "foo.bar" :type "txt.gz")
+                   (make-pathname :name "foo.bar")
                    (parse-namestring "SCRATCH:FOO.TXT.1")
                    (parse-namestring "SCRATCH:FOO.TXT.NEWEST")
                    (parse-namestring "SCRATCH:FOO.TXT")))
-    (handler-case
-        (let* ((*print-readably* t)
-               (new (read-from-string (format nil "~S" p))))
-          (unless (equal new p)
-            (let ((*print-readably* nil))
-              (error "oops: host:~S device:~S dir:~S version:~S~% ->~%~
-                             host:~S device:~S dir:~S version:~S"
-                     (pathname-host p) (pathname-device p)
-                     (pathname-directory p) (pathname-version p)
-                     (pathname-host new) (pathname-device new)
-                     (pathname-directory new) (pathname-version new)))))
-      (print-not-readable ()
-        nil))))
+    (let* ((*print-readably* t)
+           (new (read-from-string (format nil "~S" p))))
+      (unless (equal new p)
+        (let ((*print-readably* nil))
+          (error ":host ~S :device ~S :directory ~S :name ~S :type ~S :version ~S~@
+                  -> :host ~S :device ~S :directory ~S :name ~S :type ~S :version ~S"
+                 (pathname-host p) (pathname-device p)
+                 (pathname-directory p) (pathname-name p)
+                 (pathname-type p) (pathname-version p)
+                 (pathname-host new) (pathname-device new)
+                 (pathname-directory new) (pathname-name new)
+                 (pathname-type new) (pathname-version new)))))))
 
 ;;; BUG 330: "PARSE-NAMESTRING should accept namestrings as the
 ;;; default argument" ...and streams as well
 (with-test (:name (parse-namestring stream))
   (assert (equal (parse-namestring "foo" nil "/")
                  (parse-namestring "foo" nil #P"/")))
-  (let ((test "parse-namestring-test.tmp"))
-    (unwind-protect
+  (with-scratch-file (test "tmp")
         (with-open-file (f test :direction :output)
           ;; FIXME: This test is a bit flaky, since we only check that
           ;; no error is signalled. The dilemma here is "what is the
@@ -483,9 +487,7 @@
           ;; directory". Currently (0.8.10.73) we get #P"foo" here (as
           ;; opposed to eg. #P"/path/to/current/foo"), which is
           ;; possibly mildly surprising but probably conformant.
-          (assert (parse-namestring "foo" nil f)))
-      (when (probe-file test)
-        (delete-file test)))))
+          (assert (parse-namestring "foo" nil f)))))
 
 ;;; ENOUGH-NAMESTRING should probably not fail when the namestring in
 ;;; question has a :RELATIVE pathname.
@@ -530,6 +532,59 @@
             (test "*E[ab]" "*E[ab]")
             (test "*EE"    "*EE")))
 
+(with-test (:name (namestring :escape-dot))
+  (labels ((prepare (namestring)
+             #-win32 (substitute #\\ #\E namestring)
+             #+win32 (substitute #\^ #\E namestring))
+           (test (expected &rest args)
+             (let* ((expected (prepare expected))
+                    (args (loop for (key value) on args by #'cddr
+                                collect key collect (prepare value)))
+                    (pathname (pathname (apply #'make-pathname args))))
+               (assert (string= expected (namestring pathname)))
+               (assert (equal (parse-namestring expected) pathname)))))
+    (test "."                :name ".")
+    (test "foo"              :name "foo")
+    (test ".foo"             :name ".foo")
+    (test "fooE.baz"         :name "foo.baz")
+    (test "fooEE"            :name "fooE")
+    (test "fooEEEE"          :name "fooEE")
+
+    (test "..bar"            :name "."       :type "bar")
+    (test "foo.bar"          :name "foo"     :type "bar")
+    (test ".foo.bar"         :name ".foo"    :type "bar")
+    (test "foo.baz.bar"      :name "foo.baz" :type "bar")
+    (test "fooEE.bar"        :name "fooE"    :type "bar")
+    (test "fooEEEE.bar"      :name "fooEE"   :type "bar")
+
+    (test "..E.bar"          :name "."       :type ".bar")
+    (test "foo.E.bar"        :name "foo"     :type ".bar")
+    (test ".foo.E.bar"       :name ".foo"    :type ".bar")
+    (test "foo.baz.E.bar"    :name "foo.baz" :type ".bar")
+    (test "fooEE.E.bar"      :name "fooE"    :type ".bar")
+    (test "fooEEEE.E.bar"    :name "fooEE"   :type ".bar")
+
+    (test "..barE.fez"       :name "."       :type "bar.fez")
+    (test "foo.barE.fez"     :name "foo"     :type "bar.fez")
+    (test ".foo.barE.fez"    :name ".foo"    :type "bar.fez")
+    (test "foo.baz.barE.fez" :name "foo.baz" :type "bar.fez")
+    (test "fooEE.barE.fez"   :name "fooE"    :type "bar.fez")
+    (test "fooEEEE.barE.fez" :name "fooEE"   :type "bar.fez")
+
+    (test "..barE."          :name "."       :type "bar.")
+    (test "foo.barE."        :name "foo"     :type "bar.")
+    (test ".foo.barE."       :name ".foo"    :type "bar.")
+    (test "foo.baz.barE."    :name "foo.baz" :type "bar.")
+    (test "fooEE.barE."      :name "fooE"    :type "bar.")
+    (test "fooEEEE.barE."    :name "fooEE"   :type "bar.")
+
+    (test "..EEbar"          :name "."       :type "Ebar")
+    (test "foo.EEbar"        :name "foo"     :type "Ebar")
+    (test ".foo.EEbar"       :name ".foo"    :type "Ebar")
+    (test "foo.baz.EEbar"    :name "foo.baz" :type "Ebar")
+    (test "fooEE.EEbar"      :name "fooE"    :type "Ebar")
+    (test "fooEEEE.EEbar"    :name "fooEE"   :type "Ebar")))
+
 ;;; Printing of pathnames; see CLHS 22.1.3.1. This section was started
 ;;; to confirm that pathnames are printed as their namestrings under
 ;;; :escape nil :readably nil.
@@ -567,10 +622,8 @@
   (flet ((test (&rest initargs)
            (let ((pathname (apply #'make-pathname initargs)))
              (assert-error (namestring pathname) file-error))))
-    (test :name "foo.")
     (test :name "")
-    (test :type "foo")
-    (test :name "foo" :type ".bar")))
+    (test :type "foo")))
 
 ;;; reported by James Y Knight on sbcl-devel 2006-05-17
 (with-test (:name :merge-back)
@@ -678,11 +731,16 @@
   #|""         |#     "C:\\tmp\\"   "C:\\tmp\\"   "C:\\tmp\\."  "C:\\tmp\\.a"
   #|"a"        |#     "C:\\tmp\\a"  "C:\\tmp\\a"  "C:\\tmp\\a." "C:\\tmp\\a.a"))))
 
+(defun scratch-dir-truename ()
+  (with-scratch-file (name)
+    (truename (make-pathname :directory (pathname-directory name)))))
+
 (with-test (:name (delete-file logical-pathname))
   (setf (logical-pathname-translations "SB-TEST")
         (list (list "**;*.*.*" (make-pathname :name :wild
                                               :type :wild
-                                              :defaults (truename ".")))))
+                                              :defaults
+                                              (scratch-dir-truename)))))
   (let ((test (pathname "SB-TEST:delete-logical-pathname.tmp")))
     (assert (typep test 'logical-pathname))
     (with-open-file (f test :direction :output)
@@ -699,9 +757,8 @@
                  (read-from-string "#p\"\\\\\\\\\""))))
 
 (with-test (:name :load-logical-pathname-translations)
-  (let* ((cwd (truename "."))
-         (foo (merge-pathnames "llpnt-foo.translations" cwd))
-         (bar (merge-pathnames "llpnt-bar.translations" cwd))
+  (let* ((foo (scratch-file-name "translations"))
+         (bar (scratch-file-name "translations"))
          (translations (logical-pathname-translations "SYS")))
     (unwind-protect
          (progn
@@ -715,8 +772,8 @@
                                         :directory '(:absolute "my" "bar")
                                         :name :wild :type "lisp"))) f))
            (setf (logical-pathname-translations "SYS")
-                 (list* (list "SITE;LLPNT-FOO.TRANSLATIONS.NEWEST" foo)
-                        (list "SITE;LLPNT-BAR.TRANSLATIONS.NEWEST" bar)
+                 (list* (list "SITE;LLPNT-FOO.TRANSLATIONS.NEWEST" (truename foo))
+                        (list "SITE;LLPNT-BAR.TRANSLATIONS.NEWEST" (truename bar))
                         translations))
            (assert (load-logical-pathname-translations "LLPNT-FOO"))
            (assert (load-logical-pathname-translations "LLPNT-BAR"))
@@ -822,3 +879,29 @@
                            :displaced-to string
                            :displaced-index-offset 1)))
     (assert (equal (parse-namestring disp) #p"SYS:ABC.LISP"))))
+
+;;; LOGICAL-PATHAME was missing the specialized pathname comparator
+;;; function in SB_KERNEL::LAYOUT-EQUALP-IMPL
+(with-test (:name :logical-pathname-equalp-method)
+  (let ((s "#p\"sys:contrib/f[1-9].txt\""))
+    (let ((a (read-from-string s)))
+      ;; result shouldn't depend on the pathnames being EQ
+      (clrhash sb-impl::*pathnames*)
+      (let ((b (read-from-string s)))
+        (assert (not (eq a b)))
+        (equalp a b)))))
+
+;;; After the change which added address-based hashing to all INSTANCE types,
+;;; someone forgot to ensure that address-based hashes were not used on
+;;; PATTERN instances which can comprise the subcomponents of a pathname.
+;;; So make sure the hash is based on the string contents of the pattern.
+;;; Read pathnames from strings to guarantee that some magic effect of reading
+;;; pathnames doesn't coalesce them. To be especially certain, clear the
+;;; SB-IMPL::*PATHNAMES* table between the read-from-string calls.
+(with-test (:name :wild-pathnames-string-based-hash)
+  (let ((string "#p\"file[0-9].txt\""))
+    (let ((a (read-from-string string)))
+      (clrhash (clrhash sb-impl::*pathnames*))
+      (let ((b (read-from-string string)))
+        (assert (not (eq a b)))
+        (assert (eql (sxhash a) (sxhash b)))))))
