@@ -119,19 +119,20 @@
 (with-test (:name :tagged-slot-iterator-macro)
   ;; on 32-bit, the logical length is 14, which means 15 words (with header),
   ;; but slot index 14 (word index 15) exists after padding to 16 memory words.
+  ;; It is allowed to hold a fixnum (or any non-pointer) but naught else.
   #-64-bit (progn (assert (= (sb-kernel:%instance-length *afoo*) 14))
-                  (setf (sb-kernel:%instance-ref *afoo* 14) 'magic))
+                  (setf (sb-kernel:%instance-ref *afoo* 14) #xdead))
   ;; on 64-bit, the logical length is 10, which means 11 words (with header),
   ;; but slot index 10 (word index 11) exists after padding to 12 memory words.
   #+64-bit (progn (assert (= (sb-kernel:%instance-length *afoo*) 10))
-                  (setf (sb-kernel:%instance-ref *afoo* 10) 'magic))
+                  (setf (sb-kernel:%instance-ref *afoo* 10) #xdead))
 
   (let (l)
     (sb-kernel:do-instance-tagged-slot (i *afoo*)
       (push `(,i ,(sb-kernel:%instance-ref *afoo* i)) l))
     (assert (equalp (nreverse l)
-                    #-64-bit `((3 aaay) (9 bee) (13 cee) (14 magic))
-                    #+64-bit `((2 aaay) (6 bee) (9 cee) (10 magic))))))
+                    #-64-bit `((3 aaay) (9 bee) (13 cee) (14 #xdead))
+                    #+64-bit `((2 aaay) (6 bee) (9 cee) (10 #xdead))))))
 
 (defvar *anotherfoo* (make-foo1))
 
@@ -146,24 +147,35 @@
   (w 0 :type sb-ext:word)
   b
   (cdf #c(0d0 0d0) :type (complex double-float))
-  c)
+  c
+  (sword -1 :type (integer #.(1- most-negative-fixnum) 100)))
 (sb-kernel:define-structure-slot-addressor
  foo-w-ptr :structure foo :slot w)
 (sb-kernel:define-structure-slot-addressor
  foo-cdf-ptr :structure foo :slot cdf)
+(sb-kernel:define-structure-slot-addressor
+ foo-sword-ptr :structure foo :slot sword)
 
 (with-test (:name :define-structure-slot-addressor)
   (let* ((word (logand sb-ext:most-positive-word #xfeedbad))
          (re 4.2d58)
          (im 8.93d-10)
-         (thing (make-foo :cdf (complex re im) :w word)))
+         (thing (make-foo :cdf (complex re im) :w word :sword -9)))
      (sb-sys:with-pinned-objects (thing)
       (assert (= word (sb-sys:sap-ref-word
                        (sb-sys:int-sap (foo-w-ptr thing)) 0)))
       (assert (= re (sb-sys:sap-ref-double
                      (sb-sys:int-sap (foo-cdf-ptr thing)) 0)))
       (assert (= im (sb-sys:sap-ref-double
-                     (sb-sys:int-sap (foo-cdf-ptr thing)) 8))))))
+                     (sb-sys:int-sap (foo-cdf-ptr thing)) 8)))
+       (let* ((sap (sb-sys:int-sap (foo-sword-ptr thing)))
+              (slots (sb-kernel:dd-slots (sb-kernel:find-defstruct-description 'foo)))
+              (valtype (sb-kernel:dsd-raw-type
+                        (find 'sword slots :key #'sb-kernel:dsd-name))))
+         (assert (eq valtype 'sb-vm:signed-word))
+         (assert (= -9 (if (eq valtype 'sb-vm:signed-word)
+                           (sb-sys:signed-sap-ref-word sap 0)
+                           (sb-sys:sap-ref-lispobj sap 0))))))))
 
 (macrolet ((def ()
              `(defstruct foo-lotsaslots
