@@ -12,7 +12,7 @@
 ;;;; more information.
 
 
-;;;; WHITE-BOX TESTS
+;;;; STRUCTURAL TESTS
 
 (shadowing-import 'assertoid:assert-error)
 (use-package "SB-THREAD")
@@ -74,6 +74,16 @@
 
 #-sb-thread (sb-ext:exit :code 104)
 
+(with-test (:name :thread-name-from-pthread)
+  (let* ((pthread (sb-thread::thread-os-thread sb-thread:*current-thread*))
+         (string-data-sap
+          (alien-funcall (extern-alien "lisp_thread_name"
+                                       (function system-area-pointer unsigned))
+                         pthread)))
+    (assert (sap= (sb-sys:sap+ (vector-sap (sb-thread:thread-name sb-thread:*current-thread*))
+                               (* -2 sb-vm:n-word-bytes))
+                  string-data-sap))))
+
 ;;;; Now the real tests...
 
 (with-test (:name (with-mutex :timeout))
@@ -213,23 +223,23 @@
 (with-test (:name (:mutex :basics))
   (let ((l (make-mutex :name "foo"))
         (p *current-thread*))
-    (assert (eql (mutex-value l) nil) nil "1")
+    (assert (eql (mutex-owner l) nil) nil "1")
     (grab-mutex l)
-    (assert (eql (mutex-value l) p) nil "3")
+    (assert (eql (mutex-owner l) p) nil "3")
     (release-mutex l)
-    (assert (eql (mutex-value l) nil) nil "5")))
+    (assert (eql (mutex-owner l) nil) nil "5")))
 
 (with-test (:name (with-recursive-lock :basics))
   (labels ((ours-p (value)
              (eq *current-thread* value)))
     (let ((l (make-mutex :name "rec")))
-      (assert (eql (mutex-value l) nil) nil "1")
+      (assert (eql (mutex-owner l) nil) nil "1")
       (with-recursive-lock (l)
-        (assert (ours-p (mutex-value l)) nil "3")
+        (assert (ours-p (mutex-owner l)) nil "3")
         (with-recursive-lock (l)
-          (assert (ours-p (mutex-value l)) nil "4"))
-        (assert (ours-p (mutex-value l)) nil "5"))
-      (assert (eql (mutex-value l) nil) nil "6"))))
+          (assert (ours-p (mutex-owner l)) nil "4"))
+        (assert (ours-p (mutex-owner l)) nil "5"))
+      (assert (eql (mutex-owner l) nil) nil "6"))))
 
 (with-test (:name (with-recursive-lock :wait-p))
   (let ((m (make-mutex)))
@@ -282,19 +292,19 @@
         (n 0))
     (labels ((in-new-thread ()
                (with-mutex (lock)
-                 (assert (eql (mutex-value lock) *current-thread*))
+                 (assert (eql (mutex-owner lock) *current-thread*))
                  (format t "~A got mutex~%" *current-thread*)
                  ;; now drop it and sleep
                  (condition-wait queue lock)
                  ;; after waking we should have the lock again
-                 (assert (eql (mutex-value lock) *current-thread*))
+                 (assert (eql (mutex-owner lock) *current-thread*))
                  (assert (eql n 1))
                  (decf n))))
       (make-join-thread #'in-new-thread)
       (sleep 2)            ; give it  a chance to start
       ;; check the lock is free while it's asleep
       (format t "parent thread ~A~%" *current-thread*)
-      (assert (eql (mutex-value lock) nil))
+      (assert (eql (mutex-owner lock) nil))
       (with-mutex (lock)
         (incf n)
         (condition-notify queue))
@@ -307,18 +317,18 @@
                (eq *current-thread* value))
              (in-new-thread ()
                (with-recursive-lock (lock)
-                 (assert (ours-p (mutex-value lock)))
-                 (format t "~A got mutex~%" (mutex-value lock))
+                 (assert (ours-p (mutex-owner lock)))
+                 (format t "~A got mutex~%" (mutex-owner lock))
                  ;; now drop it and sleep
                  (condition-wait queue lock)
                  ;; after waking we should have the lock again
-                 (format t "woken, ~A got mutex~%" (mutex-value lock))
-                 (assert (ours-p (mutex-value lock))))))
+                 (format t "woken, ~A got mutex~%" (mutex-owner lock))
+                 (assert (ours-p (mutex-owner lock))))))
       (make-join-thread #'in-new-thread)
       (sleep 2)            ; give it  a chance to start
       ;; check the lock is free while it's asleep
       (format t "parent thread ~A~%" *current-thread*)
-      (assert (eql (mutex-value lock) nil))
+      (assert (eql (mutex-owner lock) nil))
       (with-recursive-lock (lock)
         (condition-notify queue))
       (sleep 1))))
@@ -667,7 +677,7 @@
   (sb-ext:gc)
   (incf *n-gcs-done*))
 
-#+(or x86 x86-64) ;the only platforms with a *binding-stack-pointer* variable
+#+(or x86 x86-64 riscv) ;the only platforms with a *binding-stack-pointer* variable
 (defun exercise-binding ()
   (loop
    (let ((*x* (make-something-big)))
@@ -687,7 +697,7 @@
      (wait-for-gc)
      (decf sb-vm::*binding-stack-pointer* binding-pointer-delta))))
 
-#+(or x86 x86-64) ;the only platforms with a *binding-stack-pointer* variable
+#+(or x86 x86-64 riscv) ;the only platforms with a *binding-stack-pointer* variable
 (with-test (:name (:binding-stack-gc-safety)
             :broken-on :win32)
   (let (threads)
@@ -699,7 +709,7 @@
                                       (sleep 0.1)
                                       (send-gc))))
                  threads)
-           (sleep 4))
+           (sleep 3))
       (mapc #'terminate-thread threads))))
 
 (with-test (:name :test-%thread-local-references)
@@ -878,7 +888,7 @@
               collect (make-thread #'subtypep-hash-cache-test)))
   (terpri))
 
-;;;; BLACK BOX TESTS
+;;;; FUNCTIONAL TESTS
 
 (with-test (:name (:parallel defclass))
   (write-line "WARNING, WILL HANG ON FAILURE!")
