@@ -11,7 +11,45 @@
 
 #-x86-64 (sb-ext:exit :code 104)
 
+(with-test (:name :lowtag-test-elision)
+  ;; This tests a certain behavior that while "undefined" should at least not
+  ;; be fatal. This is important for things like hash-table :TEST where we might
+  ;; call (EQUAL x y) with X being an unbound marker indicating an empty cell.
+  ;; After we started using IR1 type derivation to elide lowtag as a guard condition
+  ;; in some type tests, it became more likely to dereference an unbound marker
+  ;; which does not fit anywhere in the lisp type space.
+  (let ((f (compile nil
+                    '(lambda (x)
+                      (typecase x
+                        ((or character number list sb-kernel:instance function) 1)
+                        ;; After eliminating the preceding cases, the compiler knows
+                        ;; that the only remaining pointer type is OTHER-POINTER,
+                        ;; so it just tries to read the widetag.
+                        ;; If X is the unbound marker, this will read a byte preceding
+                        ;; the start of static space, but it holds a zero.
+                        (simple-vector 2))))))
+    (assert (not (funcall f (sb-kernel:make-unbound-marker)))))
+  (assert (not (equalp (sb-kernel:make-unbound-marker) "")))
+  (let ((a (- (sb-kernel:get-lisp-obj-address (sb-kernel:make-unbound-marker))
+              sb-vm:other-pointer-lowtag)))
+    (assert (> a sb-vm:static-space-start))))
+
 (load "compiler-test-util.lisp")
+(defun disassembly-lines (fun)
+  ;; FIXME: I don't remember what this override of the hook is for.
+  (sb-int:encapsulate 'sb-disassem::add-debugging-hooks 'test
+                      (lambda (f &rest args) (declare (ignore f args))))
+  (prog1
+      (mapcar (lambda (x) (string-left-trim " ;" x))
+              (cddr
+               (split-string
+                (with-output-to-string (s)
+                  (let ((sb-disassem:*disassem-location-column-width* 0)
+                        (*print-pretty* nil))
+                    (disassemble fun :stream s)))
+                #\newline)))
+    (sb-int:unencapsulate 'sb-disassem::add-debugging-hooks 'test)))
+
 (defun disasm (safety expr &optional (remove-epilogue t))
   ;; This lambda has a name because if it doesn't, then the name
   ;; is something stupid like (lambda () in ...) which pretty-prints
@@ -21,16 +59,7 @@
                          (declare (optimize (debug 0) (safety ,safety)
                                             (sb-c:verify-arg-count 0)))
                          ,expr))))
-    (sb-int:encapsulate 'sb-disassem::add-debugging-hooks 'test
-                        (lambda (f &rest args) (declare (ignore f args))))
-    (let ((lines
-            (split-string
-             (with-output-to-string (s)
-               (let ((sb-disassem:*disassem-location-column-width* 0))
-                 (disassemble fun :stream s)))
-             #\newline)))
-      (sb-int:unencapsulate 'sb-disassem::add-debugging-hooks 'test)
-      (setq lines (cddr lines))         ; remove "Disassembly for"
+    (let ((lines (disassembly-lines fun)))
       ;; For human-readability, kill the whitespace
       (setq lines (mapcar (lambda (x) (string-left-trim " ;" x)) lines))
       (when (string= (car (last lines)) "")
@@ -718,3 +747,35 @@ sb-vm::(define-vop (cl-user::test)
     (assert (zerop (sb-kernel:code-n-entries trampoline)))
     (assert (typep (sb-di::debug-fun-from-pc trampoline 8)
                    'sb-di::bogus-debug-fun))))
+
+(defstruct foo (s 0 :type (or null string)))
+(with-test (:name :reduce-stringp-to-not-null)
+  (let ((f1 (disassembly-lines
+             '(lambda (x) (if (null (foo-s (truly-the foo x))) 'not 'is))))
+        (f2 (disassembly-lines
+             '(lambda (x) (if (stringp (foo-s (truly-the foo x))) 'is 'not)))))
+    ;; the comparison of X to NIL should be a single-byte test
+    (assert (loop for line in f1
+                  thereis (search (format nil "CMP AL, ~D"
+                                          (logand sb-vm:nil-value #xff))
+                                  line)))
+    ;; the two variations of the test compile to the identical code
+    (dotimes (i 4)
+      (assert (string= (nth i f1) (nth i f2))))))
+
+(with-test (:name :make-list-ridiculously-huge)
+  (checked-compile '(lambda () (make-list 3826305079707827596))
+                   :allow-warnings t))
+
+(with-test (:name :with-foo-macro-elides-arg-count-trap)
+  (let ((lines
+          (split-string
+           (with-output-to-string (s)
+             (sb-c:dis '(lambda (x) (with-standard-io-syntax (eval x))) s))
+           #\newline)))
+    ;; The outer lambda checks its arg count, but the lambda
+    ;; passed to call-with-mutex does not.
+    (assert (= (count-if (lambda (line)
+                           (search "Invalid argument count trap" line))
+                         lines)
+               1))))
