@@ -78,7 +78,7 @@
   (find id (cdr spaces) :key #'space-id))
 (defun compute-nil-object (spaces)
   (let ((space (get-space static-core-space-id spaces)))
-    (%make-lisp-obj (logior (space-addr space) #x17))))
+    (%make-lisp-obj (logior (space-addr space) #x117))))
 
 ;;; Given OBJ which is tagged pointer into the target core, translate it into
 ;;; the range at which the core is now mapped during execution of this tool,
@@ -403,9 +403,9 @@
            (physaddr start))
      (loop
       (when (>= physaddr end) (return))
-      (multiple-value-bind (obj tag size)
-          (reconstitute-object (ash physaddr (- n-fixnum-tag-bits)))
-        (when (and (= tag symbol-widetag)
+      (let* ((obj (reconstitute-object (ash physaddr (- n-fixnum-tag-bits))))
+             (size (sb-vm::primitive-object-size obj)))
+        (when (and (symbolp obj)
                    (string= symbol-name (translate (symbol-name obj) spaces))
                    (%instancep (symbol-package obj))
                    (string= package-name
@@ -425,7 +425,7 @@
 (defparameter label-prefix (if (member :darwin *features*) "_" ""))
 (defun labelize (x) (concatenate 'string label-prefix x))
 
-(defun compute-linkage-symbols (spaces entry-size)
+(defun compute-linkage-symbols (spaces)
   (let* ((linkage-info (symbol-global-value
                         (find-target-symbol "SB-SYS" "*LINKAGE-INFO*"
                                             spaces :physical)))
@@ -459,8 +459,7 @@
           (symbol-global-value
            (find-target-symbol "SB-VM" "LINKAGE-TABLE-ENTRY-SIZE"
                                spaces :physical)))
-         (linkage-symbols (compute-linkage-symbols spaces linkage-entry-size))
-         (inst-space (get-inst-space))
+         (linkage-symbols (compute-linkage-symbols spaces))
          (nil-object (compute-nil-object spaces))
          (ambiguous-symbols (make-hash-table :test 'equal))
          (core
@@ -1012,7 +1011,10 @@
                                           (end (car (translate (cdr val) spaces))))
                         (list* (translate (symbol-name name) spaces) start end))
                       (target-hash-table-alist
-                       (car (translate (%code-debug-info code-component) spaces))
+                       ;; Can't use %CODE-DEBUG-INFO on a foreign core because
+                       ;; it would dereference the cons not at its current physically mapped
+                       ;; address, but at its logical address. %%CODE-DEBUG-INFO is ok though.
+                       (car (translate (sb-vm::%%code-debug-info code-component) spaces))
                        spaces))
               #'< :key #'cadr)))
         ;; Possibly a padding word
@@ -1690,9 +1692,7 @@
 (defun split-core
     (input-pathname asm-pathname
      &key enable-pie (verbose nil)
-     &aux (split-core-pathname
-           (merge-pathnames (make-pathname :type "core") asm-pathname))
-          (elf-core-pathname
+     &aux (elf-core-pathname
            (merge-pathnames
             (make-pathname :name (concatenate 'string (pathname-name asm-pathname) "-core")
                            :type "o")
@@ -1709,13 +1709,13 @@
 
   ;; Remove old files
   (ignore-errors (delete-file asm-pathname))
-  (ignore-errors (delete-file split-core-pathname))
   (ignore-errors (delete-file elf-core-pathname))
   ;; Ensure that all files can be opened
   (with-open-file (input input-pathname :element-type '(unsigned-byte 8))
     (with-open-file (asm-file asm-pathname :direction :output :if-exists :supersede)
-      (with-open-file (split-core split-core-pathname :direction :output
-                                  :element-type '(unsigned-byte 8) :if-exists :supersede)
+      ;;(with-open-file (split-core split-core-pathname :direction :output
+      ;;                            :element-type '(unsigned-byte 8) :if-exists :supersede)
+      (let ((split-core nil))
         (setq core-offset (read-core-header input core-header verbose))
         (do-core-header-entry ((id len ptr) core-header)
           (case id
@@ -1766,7 +1766,8 @@
                                   :element-type '(unsigned-byte 8)))
               (filepos))
           ;; Write the new core file
-          (write-sequence core-header split-core)
+          (when split-core
+            (write-sequence core-header split-core))
           (dolist (action (reverse copy-actions)) ; nondestructive
             ;; page index convention assumes absence of core header.
             ;; i.e. data page 0 is the file page immediately following the core header
@@ -1775,8 +1776,11 @@
               (when verbose
                 (format t "File offset ~10x: ~10x bytes~%" offset nbytes))
               (setq filepos (+ core-offset offset))
-              (file-position input filepos)
-              (copy-bytes input split-core nbytes buffer)))
+              (cond (split-core
+                     (file-position input filepos)
+                     (copy-bytes input split-core nbytes buffer))
+                    (t
+                     (file-position input (+ filepos nbytes))))))
           ;; Trailer (runtime options and magic number)
           (let ((nbytes (read-sequence buffer input)))
             ;; expect trailing magic number
@@ -1789,13 +1793,15 @@
               (format t "Trailer words:(~{~X~^ ~})~%"
                       (loop for i below (floor nbytes n-word-bytes)
                             collect (%vector-raw-bits buffer i))))
-            (write-sequence buffer split-core :end nbytes)
-            (finish-output split-core))
+            (when split-core
+              (write-sequence buffer split-core :end nbytes)
+              (finish-output split-core)))
           ;; Sanity test
-          (aver (= (+ core-offset
-                      (* page-adjust +backend-page-bytes+)
-                      (file-length split-core))
-                   (file-length input)))
+          (when split-core
+            (aver (= (+ core-offset
+                        (* page-adjust +backend-page-bytes+)
+                        (file-length split-core))
+                     (file-length input))))
           ;; Seek back to the PTE pages so they can be copied to the '.o' file
           (file-position input filepos)))
 
